#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE CPG(&
 !---------------------------------------------------------------------
 ! - INPUT .
 & CDCONF,YDSL,LDRETCFOU,LDWRTCFOU0,KGPCOMP, &
 & KBLKS,LDCPG_SPLIT,&
 !---------------------------------------------------------------------
 ! - INPUT/OUTPUT .
 & PGMV,PGMVS, &
 !---------------------------------------------------------------------
 ! - OUTPUT .
 & PB1, PB2, PGMVT1,PGMVT1S,PGFLT1,PEXTRA)

!**** *CPG* - Grid point calculations.

!     Purpose.
!     --------
!           Grid point calculations.

!           Abbreviation "vwv" stands for "vertical wind variable".

!**   Interface.
!     ----------
!        *CALL* *CPG(...)*

!        Explicit arguments :
!        --------------------

!     INPUT:
!     ------
!        CDCONF       : configuration of work
!        YDSL         : SL_STRUCT definition
!        LDRETCFOU    : .T. -> CPPFTTCINV is called under MF_PHYS.
!        LDWRTCFOU0   : .T. -> CPPFTTCDIR is called under MF_PHYS.
!        KGPCOMP      : total number of grid points:
!                       ndglg*ndlon in global / ndguxg*ndlon in LAM
!                       used for model computations.
!        KBLKS        : number of NPROMA blocks.
!        LDCPG_SPLIT  : see LCPG_SPLIT in YOMCPG.

!     INPUT/OUTPUT:
!     -------------
!        PGMV         : upper air GMV variables at t and t-dt.
!        PGMVS        : surface GMV variables at t and t-dt.

!     OUTPUT:
!     -------
!        PB1          : "SLBUF1" buffer for interpolations in SL scheme.
!        PB2          : "SLBUF2" buffer.
!        PGMVT1       : upper air GMV variables buffer.
!        PGMVT1S      : surface GMV variables buffer.
!        PGFLT1       : GFL variables buffer.
!        PEXTRA       : additional quantity for diagnostics.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ARPEGE documentation vol 2 ch 1 and vol 3 ch 6

!     Author.
!     -------
!      Philippe Courtier  *DMN*
!      Original : 90-03-19

! Modifications
! -------------
!   Modified 03-2002  J.Vivoda : PC schemes for NH dynamics (LPC_XXXX keys)
!   Modified 10-2002  C.Moussy : Allocation under the key LTENC
!   Modified 30-09-02  P.Smolikova : variable d4 in NH
!   Modified 12-10-2002 J.Masek: PC bugfix, removed YOMPONG
!   Modified 08-2002 C. Smith : use "w" as prognostic variable in the
!    semi-lag advection of vertical divergence in the NH model.
!   Modified 31-01-03  P.Smolikova : variable d4 in 2tl PC NH
!   Modified 2003-02-16 J. Hague - Single Parallel Loop Option
!   Modified 2003-03-16 M.Hamrud - Revised data flow (GFL)
!   Modified 2003-07-16 C. Fischer - gppaux goes into gmv and pb2aux
!   Modified 2003-08-25 F. Vana - SL diffusion
!   Modified 2003-08    P. Smolikova - option LRDBBC
!   Modified 2003-10-06 Y. Bouteloup - modifications for SCM extraction
!    (call mf_phys)
!   Modified 2003-10-17 M. Tudor - YCPF and YSPF from GFL
!   Modified 2003-10-31 M. Tudor - GFLPT and call cpg_pt
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   2003-11-25 M. Hortal - include extra-fields coming from the dynamics
!   12-Dec-2003 F. Vinit   new input for cpg_dia: output of APLPAR: 
!                          PALB, PGZ0, PGZ0H.
!   12-Jan-2004 K. YESSAD  remove ZSPT0.., ZZSPT9.., use GMVS instead.
!   09-Jun-2004 J. Masek   NH cleaning (LPC_NOTR, LFULLIMP)
!   01-Jul-2004 K. Yessad  Make clearer the tests for PC scheme.
!   06-Jul-2004 K. Yessad  Pointers of PTRGPPC are now computed in SUDIM2.
!   26-Mar-2004 Y. Bouteloup  Introduction of MTS radiance
!   28-Oct-2004 A. Trojakova phasing CY29 
!   20-Feb-2005 J. Vivoda  3TL PC Eulerian scheme, GWADV scheme for PC_FULL 
!   2004-11-19 Y. Seity - new argument for AROME physics. W computation.
!   12-Jan-2005 K. Yessad: GNHPDVD replaced by GP./GNH. routines in CPG_GP.
!   22-Nov-2005 D.Salmond & M.Hortal  Mods for coarser/finer physics
!   28-Sep-2005 Y. Seity : add YEXT to mf_phys and cpg_pt
!   05-APR-2006 T. Kovacic: added ZGPAR in CPG_DIA
!   07-Mar-2006 R. Zaaboul: new arguments for MF_PHYS (IBL,IGPCOMP) for 
!                           ARO_GROUND_PARAM
!   05-Oct-2005 A. Alias : ZDRNSHF added (JF Royer)
!   25-Jan-2006 F. Bouyssel: New ZFPEVPL/N Remove ZFPEVPP,ZFADVP,ZFHMLTS,ZFHEVPP
!   01-Jun-2006 B. Sass:  New pseudo-prognostic field YQVA in GFL.
!   08-Jun-2006 F. Vana two new arguments to CPG_DYN
!   06-03-2006  A. Alias : YSDSAT and YCVV fields added in MF_PHYS (JF GUEREMY)
!   04-Oct-2006 M. Bellus : added YDAL, YDOM, YUEN, YUAL, YUOM and
!                           YUNEBH (for L.Gerard's MAC scheme)
!   01-Jul-2006 M.Hamrud : Revised surface fields
!   24-Feb-2007 R. Brozkova: autoconversion and prec. evaporation fluxes split
!                            to resolved and convective contributions.
!   04-Apr-2007 M.Janousek: OMP bf.
!   10-Apr-2007 S.Ivatek-S: NEC Intialisation size of ZGPAR and ZGPNH 
!   13-Apr-2007 S.Ivatek-S: Bf. initalisation of ZVMX
!   17-Apr-2007 S.Ivatek-S: Over dimensioning of ZGPAR to NGPAR+1 and ZGPNH to 
!                           NFGPNH+1, boundary checking problem if NGPAR=0 bf
!   31-May-2007 T.Kovacic:  ZFCQRNG, ZFCQSNG, ZFEPFP, ZFCMPCQ, ZFCMPSN, ZFCMPSL
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   18-Dec-2007 Y.Seity  :  add Hail (YH GFL) for AROME  
!    4-Mar-2008 Y. Seity : Cleaning IR and WV similated sat pictures
!                            (replaced by Fullpos Calculations)
!   K. Yessad (Sep 2008): prune enhanced diffusion (lfrein).
!   18-Nov-2008 R.Brozkova: OpenMP bugfix 
!   K. Yessad (Dec 2008): remove dummy CDLOCK + cleanings
!   18-May-2009 S. Riette : mean cls wind added 
!   F. Vana  15-Oct-2009: NSPLTHOI option
!   15-October-2009 Y. Bouteloup : Store radiative cloud water and ice in GFL (YIRAD and YLRAD)
!   K. Yessad (Nov 2009): cleanings, DT/Dt now pre-computed in CPG_GP.
!   K. Yessad (Nov 2009): prune lpc_old.
!   24-Feb-2010 S. Riette : max gust avec NXGSTPERIOD seconds
!   11-Mai-2010 F. Bouyssel : RINDX, RINDY in argument
!   L. Bengtsson-Sedlar & F. Vana  18-Feb-2011 : CA arrays for MF physics
!   K. Yessad (Jan 2011): introduce INTDYN_MOD structures + cleanings.
!   F. Vana  21-Feb-2011: horiz. turbulence, N[x]LAG=4, diffus. on phys. tend.
!   G.Mozdzynski (Jan 2011): OOPS cleaning, use of derived type SL_STRUCT
!   G.Mozdzynski (Feb 2011): OOPS cleaning, use of derived type TGSGEOM
!   2011-09-07, J.M. Piriou and E. Bazile: open/close output LFA files for 1D model MUSC (LMUSCLFA).
!   K. Yessad (Fev 2012): various modifications.
!   R. El Khatib 16-Mar-2012 Automatic allocations + cleanings
!   F. Bouttier  Jul 2012: pass stochastic physics pattern ZGPSDT2D
!   2013-11, J. Masek: Passing intermittency arrays for ACRANEB2.
! End Modifications
!-------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NPROMM   ,NPROMNH  ,NPROMVC  , &
 & NPROMVC9 ,NPROMA9  ,NPROMM9  ,NPROMNH9  , &
 & NPROMDLW ,NPROMDLR ,NGPBLKS  ,NDGENL   ,NFLSA    ,NFLEN
USE YOMSLPHY , ONLY : LSLPHY   ,SAVTEND
USE YOMCT0   , ONLY : LSLAG    ,LSPRT    ,LPC_FULL ,LPC_CHEAP
USE YOMCT3   , ONLY : NSTEP
USE YOMDYN   , ONLY : TSTEP    ,TDT      ,NCURRENT_ITER  ,NSITER   , &
 &          SLHDA    ,SLHDD0   ,BETADT
USE YOMDYNA  , ONLY : LSLDIA
USE YOMGC    , ONLY : YRGSGEOM ,YROROG
USE YOMGEM   , ONLY : YRCSGEOM ,NGPTOT
USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : NFRSTLOFF
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2
USE YOM_YGFL , ONLY : YGFL
USE YOMGFL   , ONLY : GFL      ,GFLSLP   ,GFLPT    ,GFLPC
USE YOMPHY   , ONLY : LMPHYS
USE YOMSIMPHL, ONLY : LSIMPH
USE YOEPHY   , ONLY : LEPHYS   ,LAGPHY
USE YOMDPHY  , ONLY : NVEXTRDYN,NVCLIS   ,NTSSG
USE YOMLDDH  , ONLY : LSDDH
USE YOMMDDH  , ONLY : NDHCVSUN
USE YOMXFU   , ONLY : LRESET, NMEANSTEPS, NXGSTTS
USE SURFACE_FIELDS_MIX  , ONLY :  YSP_ACT_RR, YSD_ACT_VF, &
 &          SP_SB    ,YSP_SB   ,YSP_SBD  ,SP_RR    ,YSP_RR   ,SP_SG    , &
 &          YSP_SG   ,SD_VF    ,YSD_VF   ,SD_VP    ,SD_VV    ,SD_VH    , &
 &          SD_VA    ,SD_VC    ,SD_VD    ,SD_SFL   ,SD_SFO
USE YOMRADF  , ONLY : EMTD     ,EMTU     ,TRSW     ,RMOON
USE PTRSLB1  , ONLY : MSLB1UP9 ,MSLB1VP9 ,MSLB1TP9 ,MSLB1GFLP9,MSLB1SP9
USE PTRSLB2  , ONLY : MSLB2VVEL
USE YOMGMV   , ONLY : YT0      ,YT9      ,YT1      ,NDIMGMV  ,NDIMGMVS
USE OML_MOD  , ONLY : OML_MY_THREAD
USE YOMPARAR , ONLY : NGPAR
USE YOMCOAPHY, ONLY : NPHYINT
USE PAR_RDLR , ONLY : JP_DIMR9 ,JP_DIMR0
USE YOMGPDDH , ONLY : GMVTNDSI_DDH   ,GFLTNDHD_DDH   ,GMVTNDHD_DDH    ,&
 & GFLTNDSL_DDH   ,GMVTNDSL_DDH
USE YOE_CUCONVCA, ONLY : RCUCONVCA, RNLCONVCA
USE INTDYN_MOD,ONLY : YYTVC0   ,YYTTND   ,YYTHW0   ,YYTCTY0  , &
 & YYTRCP0  ,YYTRCP9  ,YYTXYB0_PHY,YYTXYB9_PHY,YYTXYB0  ,YYTXYB9
USE EINT_MOD , ONLY : SL_STRUCT
USE YOMLSFORC, ONLY : LMUSCLFA, IMUSCLFA
USE YOMSPSDT, ONLY: YGPSDT, LSPSDT
USE YOMTRC   , ONLY : GDEOSI   ,GUEOSI   ,GMU0     ,GMU0_MIN ,GMU0_MAX ,&
 &                    GDEOTI   ,GDEOTI2  ,GUEOTI   ,GUEOTI2  ,GEOLT    ,&
 &                    GEOXT    ,GRPROX   ,GMIXP    ,GFLUXC   ,GRSURF

!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
TYPE(SL_STRUCT),   INTENT(IN)    :: YDSL
LOGICAL           ,INTENT(IN)    :: LDRETCFOU 
LOGICAL           ,INTENT(IN)    :: LDWRTCFOU0 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGPCOMP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBLKS 
LOGICAL           ,INTENT(IN)    :: LDCPG_SPLIT 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV,NGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVS(NPROMA,NDIMGMVS,NGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PB1(YDSL%NASLB1,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PB2(NPROMA,NFLDSLB2,NGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM,NGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGMVT1S(NPROMA,YT1%NDIMS,NGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1,NGPBLKS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEXTRA(NPROMA,NFLEVG,NVEXTRDYN,NGPBLKS)

!     ------------------------------------------------------------------

! - semi-reduced derivatives of orography.
REAL(KIND=JPRB) :: ZOROGL(NPROMA,KBLKS)              ! cf. POROGL in CPG_GP
REAL(KIND=JPRB) :: ZOROGM(NPROMA,KBLKS)              ! cf. POROGM in CPG_GP
! - other auxiliary quantities at time 0.
REAL(KIND=JPRB) :: ZPRE0(NPROMA,0:NFLEVG,KBLKS)      ! cf. PRE0 in CPG_GP
REAL(KIND=JPRB) :: ZPRE0F(NPROMA,NFLEVG,KBLKS)       ! cf. PRE0F in CPG_GP
REAL(KIND=JPRB) :: ZPRE0L(NPROMA,KBLKS)              ! cf. PRE0L in CPG_GP
REAL(KIND=JPRB) :: ZPRE0M(NPROMA,KBLKS)              ! cf. PRE0M in CPG_GP
REAL(KIND=JPRB) :: ZNHPRE0F(NPROMNH,NFLEVG,KBLKS)    ! cf. PNHPRE0F in CPG_GP
REAL(KIND=JPRB) :: ZNHPRE0H(NPROMNH,0:NFLEVG,KBLKS)  ! cf. PNHPRE0H in CPG_GP
REAL(KIND=JPRB) :: ZPRE0_PHY(NPROMA,0:NFLEVG)        ! true pressure for physics (/= ZPRE0 in NH, = in H)
REAL(KIND=JPRB) :: ZPRE0F_PHY(NPROMA,NFLEVG)         ! true pressure for physics (/= ZPRE0F in NH, = in H)
REAL(KIND=JPRB) :: ZRT0L(NPROMA,NFLEVG,KBLKS)        ! cf. PRT0L in CPG_GP
REAL(KIND=JPRB) :: ZRT0M(NPROMA,NFLEVG,KBLKS)        ! cf. PRT0M in CPG_GP
REAL(KIND=JPRB) :: ZXYB0(NPROMA,NFLEVG,YYTXYB0%NDIM,KBLKS) ! cf. PXYB0 in CPG_GP
REAL(KIND=JPRB) :: ZUVH0(NPROMA,0:NFLEVG,YYTHW0%NDIM,KBLKS) ! cf. PUVH0 in CPG_GP
REAL(KIND=JPRB) :: ZRCP0(NPROMA,NFLEVG,YYTRCP0%NDIM,KBLKS)  ! cf. PRCP0 in CPG_GP
REAL(KIND=JPRB) :: ZPHI0(NPROMA,0:NFLEVG,KBLKS)      ! cf. PHI0 in CPG_GP
REAL(KIND=JPRB) :: ZPHIF0(NPROMA,NFLEVG,KBLKS)       ! cf. PHIF0 in CPG_GP
REAL(KIND=JPRB) :: ZPHI0FL(NPROMA,NFLEVG,KBLKS)      ! cf. PHI0FL in CPG_GP
REAL(KIND=JPRB) :: ZPHI0FM(NPROMA,NFLEVG,KBLKS)      ! cf. PHI0FM in CPG_GP
REAL(KIND=JPRB) :: ZCTY0(NPROMA,0:NFLEVG,YYTCTY0%NDIM,KBLKS) ! cf. PCTY0 in CPG_GP
REAL(KIND=JPRB) :: ZGWFT0(NPROMA,NFLEVG,KBLKS)       ! cf. PGWFT0 in CPG_GP
REAL(KIND=JPRB) :: ZKENE0(NPROMA,NFLEVG,KBLKS)       ! cf. PKENE0 in CPG_GP
REAL(KIND=JPRB) :: ZVCRS0(NPROMVC,0:NFLEVG,YYTVC0%NDIM,KBLKS) ! arrays for LVERCOR=T at time t
REAL(KIND=JPRB) :: ZVCW0F(NPROMVC,NFLEVG,KBLKS)      ! cf. PVCW0F in CPG_GP
REAL(KIND=JPRB) :: ZWDLW0F(NPROMDLW,NFLEVG,KBLKS)    ! cf. PWDLW0F in CPG_GP
REAL(KIND=JPRB) :: ZRDLR0(NPROMDLR,0:NFLEVG,JP_DIMR0,KBLKS)
REAL(KIND=JPRB) :: ZRDLR9(NPROMDLR,0:NFLEVG,JP_DIMR9,KBLKS)
REAL(KIND=JPRB) :: ZW0_PHY(NPROMM,NFLEVG)            ! for 3Dturb or AROME
REAL(KIND=JPRB) :: ZW0L_PHY(NPROMM,NFLEVG)           ! for 3Dturb
REAL(KIND=JPRB) :: ZW0M_PHY(NPROMM,NFLEVG)           ! for 3Dturb
REAL(KIND=JPRB) :: ZNHX0(NPROMA,NFLEVG,KBLKS)        ! cf. PNHXT0 in CPG_GP
REAL(KIND=JPRB) :: ZQCHA0L(NPROMNH,NFLEVG,KBLKS)     ! cf. PQCHA0L in CPG_GP
REAL(KIND=JPRB) :: ZQCHA0M(NPROMNH,NFLEVG,KBLKS)     ! cf. PQCHA0M in CPG_GP
REAL(KIND=JPRB) :: ZDBBC(NPROMNH,KBLKS)              ! cf. PDBBC in CPG_GP
REAL(KIND=JPRB) :: ZRDPHI(NPROMNH,NFLEVG,KBLKS)      ! cf. PRDPHI in CPG_GP
REAL(KIND=JPRB) :: ZGWT0(NPROMNH,NFLEVG,KBLKS)       ! cf. PGWT0 in CPG_GP
REAL(KIND=JPRB) :: ZGWS(NPROMNH,KBLKS)               ! cf. PGWS in CPG_GP
REAL(KIND=JPRB) :: ZGWFL(NPROMNH,NFLEVG,KBLKS)       ! cf. PGWFL in CPG_GP
REAL(KIND=JPRB) :: ZGWFM(NPROMNH,NFLEVG,KBLKS)       ! cf. PGWFM in CPG_GP
REAL(KIND=JPRB) :: ZPREHYD0F_PHY(NPROMM,NFLEVG)
REAL(KIND=JPRB) :: ZPREHYD0_PHY(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZXYB0_PHY(NPROMM,NFLEVG,YYTXYB0_PHY%NDIM)
! - adiabatic Lagrangian tendencies
REAL(KIND=JPRB) :: ZATND(NPROMA,NFLEVG,YYTTND%NDIM,KBLKS)
! - miscellaneous NPROMA-dimensioned arrays.
REAL(KIND=JPRB) :: ZKOZO(NPROMA,NFLEVG*NVCLIS+1,KBLKS) ! cf. PKOZO in CPG_GP
REAL(KIND=JPRB) :: ZNEB(NPROMA,NFLEVG)                 ! cf. PNEB in MF_PHYS
REAL(KIND=JPRB) :: ZQLI(NPROMA,NFLEVG,KBLKS)           ! cf. PQLI in CPG_GP
REAL(KIND=JPRB) :: ZQICE(NPROMA,NFLEVG,KBLKS)          ! cf. PQICE in CPG_GP
REAL(KIND=JPRB) :: ZRH(NPROMA,NFLEVG)                  ! cf. PRH in MF_PHYS
REAL(KIND=JPRB) :: ZQS(NPROMA,KBLKS)                   ! cf. PQS in MF_PHYS
REAL(KIND=JPRB) :: ZQSOL(NPROMA,KBLKS)                 ! cf. PQSOL in MF_PHYS
REAL(KIND=JPRB) :: ZDHSF(NPROMA)                       ! cf. PDHSF in CPG_DIA
REAL(KIND=JPRB) :: ZDHCV(NPROMA,0:NFLEVG,NDHCVSUN)     ! cf. PDHCV in CPG_DIA
REAL(KIND=JPRB) :: ZCLCT(NPROMA)                       ! cf. PCLCT in MF_PHYS
REAL(KIND=JPRB) :: ZGPSDT2D(NPROMA, YGPSDT%NG2D, NGPBLKS) ! stochastic physics patterns

! - quantities for physics (with dimension NPROMM, to be gathered in bigger structures later).
REAL(KIND=JPRB) :: ZDIFCQ(NPROMM,0:NFLEVG)           ! cf. PDIFCQ in MF_PHYS
REAL(KIND=JPRB) :: ZDIFCQL(NPROMM,0:NFLEVG)          ! cf. PDIFCQL in MF_PHYS
REAL(KIND=JPRB) :: ZDIFCQN(NPROMM,0:NFLEVG)          ! cf. PDIFCQN in MF_PHYS
REAL(KIND=JPRB) :: ZDIFCS(NPROMM,0:NFLEVG)           ! cf. PDIFCS in MF_PHYS
REAL(KIND=JPRB) :: ZDIFTQ(NPROMM,0:NFLEVG)           ! cf. PDIFTQ in MF_PHYS
REAL(KIND=JPRB) :: ZDIFTQL(NPROMM,0:NFLEVG)          ! cf. PDIFTQL in MF_PHYS
REAL(KIND=JPRB) :: ZDIFTQN(NPROMM,0:NFLEVG)          ! cf. PDIFTQN in MF_PHYS
REAL(KIND=JPRB) :: ZDIFTS(NPROMM,0:NFLEVG)           ! cf. PDIFTS in MF_PHYS
REAL(KIND=JPRB) :: ZFCCQL(NPROMM,0:NFLEVG)           ! cf. PFCCQL in MF_PHYS
REAL(KIND=JPRB) :: ZFCCQN(NPROMM,0:NFLEVG)           ! cf. PFCCQN in MF_PHYS
REAL(KIND=JPRB) :: ZFCSQL(NPROMM,0:NFLEVG)           ! cf. PFCSQL in MF_PHYS
REAL(KIND=JPRB) :: ZFCSQN(NPROMM,0:NFLEVG)           ! cf. PFCSQN in MF_PHYS
REAL(KIND=JPRB) :: ZFCQLNG(NPROMM,0:NFLEVG)          ! cf. PFCQLNG in MF_PHYS
REAL(KIND=JPRB) :: ZFCQNNG(NPROMM,0:NFLEVG)          ! cf. PFCQNNG in MF_PHYS
REAL(KIND=JPRB) :: ZFCQNG(NPROMM,0:NFLEVG)           ! cf. PFCQNG in MF_PHYS
REAL(KIND=JPRB) :: ZFPLCL(NPROMM,0:NFLEVG)           ! cf. PFPLCL in MF_PHYS
REAL(KIND=JPRB) :: ZFPLCN(NPROMM,0:NFLEVG)           ! cf. PFPLCN in MF_PHYS
REAL(KIND=JPRB) :: ZFPLCG(NPROMM,0:NFLEVG)           ! cf. PFPLCG in MF_PHYS
REAL(KIND=JPRB) :: ZFPLCH(NPROMM,0:NFLEVG)           ! cf. PFPLCH in MF_PHYS
REAL(KIND=JPRB) :: ZFPLSL(NPROMM,0:NFLEVG)           ! cf. PFPLSL in MF_PHYS
REAL(KIND=JPRB) :: ZFPLSN(NPROMM,0:NFLEVG)           ! cf. PFPLSN in MF_PHYS
REAL(KIND=JPRB) :: ZFPLSG(NPROMM,0:NFLEVG)           ! cf. PFPLSG in MF_PHYS
REAL(KIND=JPRB) :: ZFPLSH(NPROMM,0:NFLEVG)           ! cf. PFPLSH in MF_PHYS
REAL(KIND=JPRB) :: ZFPFPSL(NPROMM,0:NFLEVG)          ! cf. PFPFPSL in MF_PHYS.
REAL(KIND=JPRB) :: ZFPFPSN(NPROMM,0:NFLEVG)          ! cf. PFPFPSN in MF_PHYS.
REAL(KIND=JPRB) :: ZFPFPCL(NPROMM,0:NFLEVG)          ! cf. PFPFPCL in MF_PHYS.
REAL(KIND=JPRB) :: ZFPFPCN(NPROMM,0:NFLEVG)          ! cf. PFPFPCN in MF_PHYS.
REAL(KIND=JPRB) :: ZFPEVPSL(NPROMM,0:NFLEVG)         ! cf. PFPEVPSL in MF_PHYS.
REAL(KIND=JPRB) :: ZFPEVPSN(NPROMM,0:NFLEVG)         ! cf. PFPEVPSN in MF_PHYS.
REAL(KIND=JPRB) :: ZFPEVPCL(NPROMM,0:NFLEVG)         ! cf. PFPEVPCL in MF_PHYS.
REAL(KIND=JPRB) :: ZFPEVPCN(NPROMM,0:NFLEVG)         ! cf. PFPEVPCN in MF_PHYS.
REAL(KIND=JPRB) :: ZFRSO(NPROMM,0:NFLEVG,NTSSG+1)    ! cf. PFRSO in MF_PHYS
REAL(KIND=JPRB) :: ZFRTH(NPROMM,0:NFLEVG,NTSSG+1)    ! cf. PFRTH in MF_PHYS
REAL(KIND=JPRB) :: ZSTRCU(NPROMM,0:NFLEVG)           ! cf. PSTRCU in MF_PHYS
REAL(KIND=JPRB) :: ZSTRCV(NPROMM,0:NFLEVG)           ! cf. PSTRCV in MF_PHYS
REAL(KIND=JPRB) :: ZSTRDU(NPROMM,0:NFLEVG)           ! cf. PSTRDU in MF_PHYS
REAL(KIND=JPRB) :: ZSTRDV(NPROMM,0:NFLEVG)           ! cf. PSTRDV in MF_PHYS
REAL(KIND=JPRB) :: ZSTRTU(NPROMM,0:NFLEVG)           ! cf. PSTRTU in MF_PHYS
REAL(KIND=JPRB) :: ZSTRTV(NPROMM,0:NFLEVG)           ! cf. PSTRTV in MF_PHYS
REAL(KIND=JPRB) :: ZSTRMU(NPROMM,0:NFLEVG)           ! cf. PSTRMU in MF_PHYS
REAL(KIND=JPRB) :: ZSTRMV(NPROMM,0:NFLEVG)           ! cf. PSTRMV in MF_PHYS
REAL(KIND=JPRB) :: ZDIFCQLC(NPROMM,0:NFLEVG)         ! cf. PDIFCQLC in MF_PHYS
REAL(KIND=JPRB) :: ZDIFCQIC(NPROMM,0:NFLEVG)         ! cf. PDIFCQIC in MF_PHYS
REAL(KIND=JPRB) :: ZFIMCC (NPROMM,0:NFLEVG)          ! cf. PFIMCC in MF_PHYS
REAL(KIND=JPRB) :: ZFEDQLC(NPROMM,0:NFLEVG)          ! cf. PFEDQLC in MF_PHYS
REAL(KIND=JPRB) :: ZFEDQIC(NPROMM,0:NFLEVG)          ! cf. PFEDQIC in MF_PHYS
REAL(KIND=JPRB) :: ZFEDQRC(NPROMM,0:NFLEVG)          ! cf. PFEDQRC in MF_PHYS
REAL(KIND=JPRB) :: ZFEDQSC(NPROMM,0:NFLEVG)          ! cf. PFEDQSC in MF_PHYS
REAL(KIND=JPRB) :: ZFCNEGQLC(NPROMM,0:NFLEVG)        ! cf. PFCNEGQLC in MF_PHYS
REAL(KIND=JPRB) :: ZFCNEGQIC(NPROMM,0:NFLEVG)        ! cf. PFCNEGQIC in MF_PHYS
REAL(KIND=JPRB) :: ZFCNEGQRC(NPROMM,0:NFLEVG)        ! cf. PFCNEGQRC in MF_PHYS
REAL(KIND=JPRB) :: ZFCNEGQSC(NPROMM,0:NFLEVG)        ! cf. PFCNEGQSC in MF_PHYS
REAL(KIND=JPRB) :: ZFRMH(NPROMM,0:NFLEVG)            ! cf. PFRMH in MF_PHYS
REAL(KIND=JPRB) :: ZFCHOZ(NPROMM,0:NFLEVG)           ! cf. PFCHOZ in MF_PHYS
REAL(KIND=JPRB) :: ZFDIS (NPROMM,0:NFLEVG)           ! cf. PFDIS in MF_PHYS
REAL(KIND=JPRB) :: ZFHPSL(NPROMM,0:NFLEVG)           ! cf. PFHPSL in MF_PHYS
REAL(KIND=JPRB) :: ZFHPSN(NPROMM,0:NFLEVG)           ! cf. PFHPSN in MF_PHYS
REAL(KIND=JPRB) :: ZFHPCL(NPROMM,0:NFLEVG)           ! cf. PFHPCL in MF_PHYS
REAL(KIND=JPRB) :: ZFHPCN(NPROMM,0:NFLEVG)           ! cf. PFHPCN in MF_PHYS
REAL(KIND=JPRB) :: ZFHSCL(NPROMM,0:NFLEVG)           ! cf. PFHSCL in MF_PHYS
REAL(KIND=JPRB) :: ZFHSCN(NPROMM,0:NFLEVG)           ! cf. PFHSCN in MF_PHYS
REAL(KIND=JPRB) :: ZFHSSL(NPROMM,0:NFLEVG)           ! cf. PFHSSL in MF_PHYS
REAL(KIND=JPRB) :: ZFHSSN(NPROMM,0:NFLEVG)           ! cf. PFHSSN in MF_PHYS
REAL(KIND=JPRB) :: ZFEPFP(NPROMM,0:NFLEVG)           ! cf. PFEPFP  in MF_PHYS
REAL(KIND=JPRB) :: ZFCMPCQ(NPROMM,0:NFLEVG)          ! cf. PFCMPCQ in MF_PHYS
REAL(KIND=JPRB) :: ZFCMPSN(NPROMM,0:NFLEVG)          ! cf. PFCMPSN in MF_PHYS
REAL(KIND=JPRB) :: ZFCMPSL(NPROMM,0:NFLEVG)          ! cf. PFCMPSL in MF_PHYS
REAL(KIND=JPRB) :: ZFRSOC(NPROMM,0:1)                ! cf. PFRSOC in MF_PHYS
REAL(KIND=JPRB) :: ZFRTHC(NPROMM,0:1)                ! cf. PFRTHC in MF_PHYS
REAL(KIND=JPRB) :: ZFCHSP(NPROMM,YSP_SBD%NLEVS)      ! cf. PFCHSP in MF_PHYS
REAL(KIND=JPRB) :: ZFCLL(NPROMM,NTSSG+1)             ! cf. PFCLL in MF_PHYS
REAL(KIND=JPRB) :: ZFCLN(NPROMM,NTSSG+1)             ! cf. PFCLN in MF_PHYS
REAL(KIND=JPRB) :: ZFCS(NPROMM,NTSSG+1)              ! cf. PFCS in MF_PHYS
REAL(KIND=JPRB) :: ZFEVL(NPROMM,NTSSG+1)             ! cf. PFEVL in MF_PHYS
REAL(KIND=JPRB) :: ZFEVN(NPROMM,NTSSG+1)             ! cf. PFEVN in MF_PHYS
REAL(KIND=JPRB) :: ZFEVV(NPROMM)                     ! cf. PFEVV in MF_PHYS
REAL(KIND=JPRB) :: ZFLWSP(NPROMM)                    ! cf. PFLWSP in MF_PHYS
REAL(KIND=JPRB) :: ZFTR(NPROMM)                      ! cf. PFTR in MF_PHYS
REAL(KIND=JPRB) :: ZFRSODS(NPROMM)                   ! cf. PFRSODS in MF_PHYS
REAL(KIND=JPRB) :: ZFRSOPS(NPROMM)                   ! cf. PFRSOPS in MF_PHYS
REAL(KIND=JPRB) :: ZFRTHDS(NPROMM)                   ! cf. PFRTHDS in MF_PHYS
REAL(KIND=JPRB) :: ZFONTE(NPROMM)                    ! cf. PFONTE in MF_PHYS
REAL(KIND=JPRB) :: ZFGEL(NPROMM)                     ! cf. PFGEL in MF_PHYS
REAL(KIND=JPRB) :: ZFGELS(NPROMM)                    ! cf. PFGELS in MF_PHYS
REAL(KIND=JPRB) :: ZALB(NPROMM)                      ! cf. PALB in MF_PHYS
REAL(KIND=JPRB) :: ZGZ0(NPROMM)                      ! cf. PGZ0 in MF_PHYS
REAL(KIND=JPRB) :: ZGZ0H(NPROMM)                     ! cf. PGZ0H in MF_PHYS
REAL(KIND=JPRB) :: ZRUISL(NPROMM)                    ! cf. PRUISL in MF_PHYS
REAL(KIND=JPRB) :: ZRUISP(NPROMM)                    ! cf. PRUISP in MF_PHYS
REAL(KIND=JPRB) :: ZRUISS(NPROMM)                    ! cf. PRUISS in MF_PHYS
REAL(KIND=JPRB) :: ZFRSOPT(NPROMM)                   ! cf. PFRSOPT in MF_PHYS
REAL(KIND=JPRB) :: ZFRSOLU(NPROMM)                   ! cf. PFRSOLU in MF_PHYS
REAL(KIND=JPRB) :: ZQCLS(NPROMM)                     ! cf. PQCLS in MF_PHYS
REAL(KIND=JPRB) :: ZTCLS(NPROMM)                     ! cf. PTCLS in MF_PHYS
REAL(KIND=JPRB) :: ZUCLS(NPROMM)                     ! cf. PUCLS in MF_PHYS
REAL(KIND=JPRB) :: ZVCLS(NPROMM)                     ! cf. PVCLS in MF_PHYS
REAL(KIND=JPRB) :: ZRHCLS(NPROMM)                    ! cf. PRHCLS in MF_PHYS
REAL(KIND=JPRB) :: ZCLCH(NPROMM)                     ! cf. PCLCH in MF_PHYS
REAL(KIND=JPRB) :: ZCLCM(NPROMM)                     ! cf. PCLCM in MF_PHYS
REAL(KIND=JPRB) :: ZCLCL(NPROMM)                     ! cf. PCLCL in MF_PHYS
REAL(KIND=JPRB) :: ZCLCC(NPROMM)                     ! cf. PCLCC in MF_PHYS
REAL(KIND=JPRB) :: ZCLPH(NPROMM)                     ! cf. PCLPH in MF_PHYS
REAL(KIND=JPRB) :: ZVEIN(NPROMM)                     ! cf. PVEIN in MF_PHYS
REAL(KIND=JPRB) :: ZDRNSHF(NPROMM)                   ! cf. PDERNSHF in MF_PHYS
REAL(KIND=JPRB) :: ZCAPE(NPROMM)                     ! cf. PCAPE in MF_PHYS
REAL(KIND=JPRB) :: ZMOCON(NPROMM)                    ! cf. PMOCON in MF_PHYS
REAL(KIND=JPRB) :: ZUGST(NPROMM)                     ! cf. PUGST in MF_PHYS
REAL(KIND=JPRB) :: ZVGST(NPROMM)                     ! cf. PVGST in MF_PHYS
REAL(KIND=JPRB) :: ZCT(NPROMA)                       ! cf. PCT in MF_PHYS
REAL(KIND=JPRB) :: ZTENDU(NPROMM,NFLEVG)             ! cf. PTENDU in MF_PHYS
REAL(KIND=JPRB) :: ZTENDV(NPROMM,NFLEVG)             ! cf. PTENDV in MF_PHYS
REAL(KIND=JPRB) :: ZFCQRNG(NPROMM,0:NFLEVG)          ! cf. PFCQRNG in MF_PHYS
REAL(KIND=JPRB) :: ZFCQSNG(NPROMM,0:NFLEVG)          ! cf. PFCQSNG in MF_PHYS
REAL(KIND=JPRB) :: ZDIAGH(NPROMM)                    ! cf. PDIAGH in MF_Phys
! - quantities at time 9 (private)
REAL(KIND=JPRB) :: ZW9_PHY(NPROMM,NFLEVG)            ! for AROME Physics
REAL(KIND=JPRB) :: ZPRE9_PHY(NPROMM9,0:NFLEVG)       ! true pressure for physics (/= ZPRE9 in NH, = in H)
REAL(KIND=JPRB) :: ZPRE9F_PHY(NPROMM9,NFLEVG)        ! true pressure for physics (/= ZPRE9F in NH, = in H)
REAL(KIND=JPRB) :: ZPREHYD9F_PHY(NPROMM9,NFLEVG)
REAL(KIND=JPRB) :: ZPREHYD9_PHY(NPROMM9,0:NFLEVG)
REAL(KIND=JPRB) :: ZXYB9_PHY(NPROMM9,NFLEVG,YYTXYB9_PHY%NDIM)

! - quantities at time 9 (shared)
REAL(KIND=JPRB) :: ZPRE9(NPROMA,0:NFLEVG,KBLKS)      ! cf. PRE9 in CPG_GP

! - other auxiliary quantities at time 9 (shared).
REAL(KIND=JPRB) :: ZPRE9F(NPROMA9,NFLEVG,KBLKS)               ! cf. PRE9F in CPG_GP
REAL(KIND=JPRB) :: ZNHPRE9F(NPROMNH9,NFLEVG,KBLKS)            ! cf. PNHPRE9F in CPG_GP
REAL(KIND=JPRB) :: ZNHPRE9H(NPROMNH9,0:NFLEVG,KBLKS)          ! cf. PNHPRE9H in CPG_GP
REAL(KIND=JPRB) :: ZXYB9(NPROMA9,NFLEVG,YYTXYB9%NDIM,KBLKS)   ! cf. PXYB9 in CPG_GP
REAL(KIND=JPRB) :: ZRCP9(NPROMA9,NFLEVG,YYTRCP9%NDIM,KBLKS)   ! cf. PRCP9 in CPG_GP
REAL(KIND=JPRB) :: ZPHI9(NPROMA9,0:NFLEVG,KBLKS)              ! cf. PHI9 in CPG_GP
REAL(KIND=JPRB) :: ZPHIF9(NPROMA9,NFLEVG,KBLKS)               ! cf. PHIF9 in CPG_GP
REAL(KIND=JPRB) :: ZGWFT9(NPROMA9,NFLEVG,KBLKS)               ! cf. PGWFT9 in CPG_GP
REAL(KIND=JPRB) :: ZGWT9(NPROMNH9,NFLEVG,KBLKS)               ! cf. PGWT9 in CPG_GP
REAL(KIND=JPRB) :: ZVCRSSA9F(NPROMVC9,NFLEVG,KBLKS)           ! cf. PVCRSSA9F in CPG_GP
REAL(KIND=JPRB) :: ZNHX9(NPROMA9,NFLEVG,KBLKS)                ! cf. PNHXT9 in CPG_GP

! - other quantities (shared).
REAL(KIND=JPRB)             :: ZGPAR(NPROMM,NGPAR+1,KBLKS)      ! cf. PGPAR in CPG_GP
REAL(KIND=JPRB)             :: ZSLBUF1AU(NPROMA,NFLDSLB1,KBLKS) ! local version of SLBUF1

! - other quantities (private).
REAL(KIND=JPRB)             :: ZLSM(NPROMA)          ! cf. PLSM in CPG_DYN
REAL(KIND=JPRB)             :: ZTSOL(NPROMA)         ! cf. PTSOL in CPG_DYN

REAL(KIND=JPRB) :: ZDT, ZDTPHY, ZTE, ZBETADT

INTEGER(KIND=JPIM) :: IDDHI(NPROMA)                  ! cf. KDDHI in CPG_GP

INTEGER(KIND=JPIM) :: JROF, JKGLO, JFLD
INTEGER(KIND=JPIM) :: IEND, IENDC, IST, ISTC
INTEGER(KIND=JPIM) :: IBL
INTEGER(KIND=JPIM) :: ICEND, ICENDE
INTEGER(KIND=JPIM) :: IGL1, IGL2, IGL3, IGL4
INTEGER(KIND=JPIM) :: IGPCOMP
INTEGER(KIND=JPIM) :: IOFF
INTEGER(KIND=JPIM) :: ISTGLO
INTEGER(KIND=JPIM) :: ILL, ILP, JSPLIT
INTEGER(KIND=JPIM) :: ISLB1GFL9,ISLB1T9,ISLB1V9,ISLB1U9,ISLB1VD9

LOGICAL :: LLADNMI, LLDIAB, LLFSTEP, LLRESET, LLSLPHY, LLUSEPB1, LLCT, LLCTC
LOGICAL :: LLGET
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "cpdysldia.intfb.h"
#include "cpg_dia.intfb.h"
#include "cpg_dyn.intfb.h"
#include "cpg_end.intfb.h"
#include "cpg_gp.intfb.h"
#include "cpg_pt_ulp.intfb.h"
#include "cp_ptrslb1.intfb.h"
#include "ec_phys_lslphy.intfb.h"
#include "gpiniddh.intfb.h"
#include "mf_phys_prep.intfb.h"
#include "mf_phys.intfb.h"
#include "open_output_lfa.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY INITIALISATIONS 
!              ---------------------------

IF (LMUSCLFA) CALL OPEN_OUTPUT_LFA

LLCT =LPC_FULL.AND.(NCURRENT_ITER > 0)
LLCTC=LPC_CHEAP.AND.(NCURRENT_ITER > 0)
IF(NSTEP > 0) THEN
  LLFSTEP=.FALSE.
ELSE
  LLFSTEP=.TRUE.
ENDIF
! Physics is called in CPG
LLDIAB= (LMPHYS.OR.LEPHYS) .AND. .NOT.LAGPHY .AND. (NCURRENT_ITER == 0)
! Adiabatic NNMI iteration
LLADNMI= CDCONF == 'E' .OR. CDCONF == 'F'
LLSLPHY = LSLPHY.AND.LSLAG.AND..NOT.LLADNMI.AND.(.NOT.LLCTC)
! NNMI iteration or initial fluxes
IF (LLADNMI .OR. CDCONF == 'X' ) THEN
  LLFSTEP=.TRUE.
ENDIF
LLRESET=LLFSTEP.OR.LRESET

IF (LSLAG .AND. ((NCURRENT_ITER == 0).OR.(LLCT.AND.(.NOT.LLCTC))) ) THEN 
  LLUSEPB1=.TRUE.
ELSE
  LLUSEPB1=.FALSE.
ENDIF

IF (CDCONF == 'A'.OR.CDCONF == 'B') THEN
  ! usual time-step
  ZDT=TDT
  ZDTPHY=TDT
  ZTE=1.0_JPRB
  ZBETADT=BETADT
ELSEIF (CDCONF == 'X') THEN
  ! usual time-step for diagnostics but no SI scheme.
  ZDT=TDT
  ZDTPHY=TDT
  ZTE=1.0_JPRB
  ZBETADT=0.0_JPRB
ELSEIF (LLADNMI) THEN
  ! dynamical tendencies (adiabatic NNMI iteration : CDCONF='E' or 'F')
  ZDT=TSTEP
  ZDTPHY=0.0_JPRB
  IF (LSPRT) THEN
    ZTE=1.0_JPRB
  ELSE
    ZTE=0.0_JPRB
  ENDIF
  ZBETADT=0.0_JPRB
ELSE
  ZTE=0
  WRITE(NULOUT,*)'*********ERROR ON CDCONF'
  CALL ABOR1('CPG: ABOR1 CALLED')
ENDIF

IGPCOMP=KGPCOMP

IF (LSPSDT) ZGPSDT2D(:,:,:)=YGPSDT%GP2D(:,:,:)    !Load stochastic physics random patterns

!     ------------------------------------------------------------------

!*       2.     SET UP OUTER LOOP JSPLIT
!              -------------------------

!  IF LDCPG_SPLIT FALSE: 1 PASS TRHU "PARALLEL DO"
!  IF LDCPG_SPLIT TRUE:  3 PASSES THRU "PARALLEL DO".  SETUP, PHYSICS, AND
!                        DYNAMICS DONE IN CONSECUTIVE PASSES

IF(LDCPG_SPLIT) THEN
  ILP=3
ELSE
  ILP=1
ENDIF

!     ------------------------------------------------------------------

!*       3.    READ BUFFERS, COMPUTE AUXILIARY QUANTITIES.
!              -------------------------------------------

DO JSPLIT=1,ILP

  IGL1=1
  IGL2=NDGENL
  IST=1
  ISTC=1

  IF(LDCPG_SPLIT) THEN
    IF(JSPLIT == 1) CALL GSTATS(1025,0)
    IF(JSPLIT == 2) CALL GSTATS(1026,0)
    IF(JSPLIT == 3) CALL GSTATS(1027,0)
  ELSE
    CALL GSTATS(1025,0)
  ENDIF

!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) &
!$OMP& PRIVATE (JKGLO,IOFF,ISTGLO,IBL,ICEND,ICENDE,IEND,IENDC,JROF,JFLD) &
!$OMP& PRIVATE (IGL3,IGL4,ILL) &
!$OMP& PRIVATE (ZDHSF,ZALB,ZCAPE,ZCLCC,ZCLCH,ZCLCL,ZCLCM,ZCLCT,ZCLPH,ZVEIN,ZCT) &
!$OMP& PRIVATE (ZDIFCQ,ZDIFCQN,ZDIFCQL,ZDIFCS,ZDIFTQ,ZDIFTQN,ZDIFTQL,ZDIFTS) &
!$OMP& PRIVATE (ZFCCQL,ZFCCQN,ZFCHOZ,ZFCHSP,ZFCLL,ZFCLN,ZFCQNNG,ZFCQLNG,ZFCQNG)&
!$OMP& PRIVATE (ZFCS) &
!$OMP& PRIVATE (ZFCSQL,ZFCSQN,ZFEVL,ZFEVN,ZFEVV,ZFGEL,ZFGELS,ZFLWSP,ZFONTE) &
!$OMP& PRIVATE (ZFPLCL,ZFPLCN,ZFPLCG,ZFPLCH,ZFPLSL,ZFPLSN,ZFPLSG,ZFPLSH) &
!$OMP& PRIVATE (ZFRSOLU,ZFRMH,ZFRSO,ZFRSOC,ZFRSODS)&
!$OMP& PRIVATE (ZFPFPSL,ZFPFPSN,ZFPFPCL,ZFPFPCN)&
!$OMP& PRIVATE (ZFPEVPSL,ZFPEVPSN,ZFPEVPCL,ZFPEVPCN,ZDRNSHF)&
!$OMP& PRIVATE (ZFRSOPS,ZFRSOPT,ZFRTH,ZFRTHC,ZFRTHDS,ZFTR,ZGZ0,ZGZ0H,ZNEB) &
!$OMP& PRIVATE (ZQCLS,ZRH,ZRHCLS,ZRUISL,ZRUISP,ZRUISS) &
!$OMP& PRIVATE (ZSTRCU,ZSTRCV,ZSTRDU,ZSTRDV,ZSTRMU,ZSTRMV,ZSTRTU,ZSTRTV) &
!$OMP& PRIVATE (ZDIFCQLC,ZDIFCQIC,ZFIMCC) &
!$OMP& PRIVATE (ZFEDQLC,ZFEDQIC,ZFEDQRC,ZFEDQSC,ZFCNEGQLC,ZFCNEGQIC,ZFCNEGQRC,ZFCNEGQSC) &
!$OMP& PRIVATE (ZUCLS,ZVCLS,ZTCLS,ZUGST,ZVGST,ZMOCON,ZFDIS) &
!$OMP& PRIVATE (ZFHPCL,ZFHPCN,ZFHPSL,ZFHPSN,ZFHSCL,ZFHSCN,ZFHSSL,ZFHSSN) &
!$OMP& PRIVATE (ZFEPFP, ZFCMPCQ, ZFCMPSN, ZFCMPSL, ZFCQRNG, ZFCQSNG, ZDIAGH) &
!$OMP& PRIVATE (ZTENDU,ZTENDV,ZDHCV,IDDHI) &
!$OMP& PRIVATE (ZW0_PHY,ZW9_PHY,ZW0L_PHY,ZW0M_PHY,ZPRE0F_PHY,ZPRE9F_PHY,ZPRE0_PHY,ZPRE9_PHY) &
!$OMP& PRIVATE (ZPREHYD0F_PHY,ZPREHYD9F_PHY,ZPREHYD0_PHY,ZPREHYD9_PHY) &
!$OMP& PRIVATE (ZXYB0_PHY,ZXYB9_PHY)

  DO JKGLO=1,IGPCOMP,NPROMA

    IOFF=JKGLO
    ISTGLO=JKGLO
    IBL=(JKGLO-1)/NPROMA+1
    ICEND=MIN(NPROMA,IGPCOMP-JKGLO+1)
    ICENDE=MIN(NPROMA,NGPTOT-JKGLO+1)
    IEND=ICEND
    IENDC=ICENDE

    IF(LDCPG_SPLIT) THEN
      ILL=IBL
    ELSE
      ILL=OML_MY_THREAD()
    ENDIF

    IF(.NOT.LDCPG_SPLIT .OR. JSPLIT == 1) THEN

      CALL CPG_GP(&
       !---------------------------------------------------------------------
       ! - INPUT .
       & IST,IEND,ISTC,IENDC,IBL,ISTGLO,LLFSTEP,LLDIAB,LLADNMI,ZDT,ZTE, &
       & YRGSGEOM(IBL),YRCSGEOM(IBL),YROROG(IBL), &
       !---------------------------------------------------------------------
       ! - INPUT/OUTPUT .
       & GFL(:,:,:,IBL),PGMV(1,1,1,IBL),PGMVS(1,1,IBL), &
       & GMVTNDSL_DDH(1,1,1,IBL),GFLTNDSL_DDH(1,1,1,IBL), &
       !---------------------------------------------------------------------
       ! - OUTPUT .
       & ZOROGL(1,ILL),ZOROGM(1,ILL), &
       & ZPRE0(1,0,ILL),ZPRE0L(1,ILL),ZPRE0M(1,ILL),ZPRE0F(1,1,ILL), &
       & ZNHPRE0F(1,1,ILL),ZNHPRE0H(1,0,ILL),ZXYB0(1,1,1,ILL), &
       & ZUVH0(1,0,1,ILL), &
       & ZPHI0(1,0,ILL),ZPHIF0(1,1,ILL),ZPHI0FL(1,1,ILL),ZPHI0FM(1,1,ILL), &
       & ZRCP0(1,1,1,ILL),ZCTY0(1,0,1,ILL),ZGWFT0(1,1,ILL),ZKENE0(1,1,ILL), &
       & ZRT0L(1,1,ILL),ZRT0M(1,1,ILL), &
       & ZVCRS0(1,0,1,ILL),ZVCW0F(1,1,ILL),ZWDLW0F(1,1,ILL), &
       & ZRDLR0(1,0,1,ILL),ZRDLR9(1,0,1,ILL), &
       & ZPRE9(1,0,ILL),ZPRE9F(1,1,ILL), &
       & ZNHPRE9F(1,1,ILL),ZNHPRE9H(1,0,ILL),ZXYB9(1,1,1,ILL), &
       & ZPHI9(1,0,ILL),ZPHIF9(1,1,ILL),ZRCP9(1,1,1,ILL), &
       & ZGWFT9(1,1,ILL), &
       & ZVCRSSA9F(1,1,ILL), &
       & ZGPAR(1,1,ILL),PB2(1,1,IBL),&
       & PGMVT1(1,1,1,IBL),PGMVT1S(1,1,IBL),PGFLT1(1,1,1,IBL), &
       & ZKOZO(1,1,ILL),ZQS(1,ILL),ZQICE(1,1,ILL),ZQLI(1,1,ILL), &
       & ZATND(1,1,1,ILL), &
       & ZDBBC(1,ILL),ZRDPHI(1,1,ILL),ZGWT0(1,1,ILL),ZGWT9(1,1,ILL),&
       & ZGWS(1,ILL),ZGWFL(1,1,ILL),ZGWFM(1,1,ILL),ZNHX0(1,1,ILL),ZNHX9(1,1,ILL),&
       & ZQCHA0L(1,1,ILL),ZQCHA0M(1,1,ILL))

    ENDIF

!     ------------------------------------------------------------------

!*       4.    PHYSICS + DIAGNOSTICS
!              ---------------------

    IF(.NOT.LDCPG_SPLIT .OR. JSPLIT == 2) THEN

      ! * IGL3: first DM-local lat index of the current NPROMA-packet.
      IGL3=YRGSGEOM(IBL)%NGPLAT(1)-NFRSTLOFF
      ! * IGL4: last DM-local lat index of the current NPROMA-packet.
      IGL4=YRGSGEOM(IBL)%NGPLAT(ICEND)-NFRSTLOFF

!     ------------------------------------------------------------------

!*       4.1    Fill ZSLBUF1AU.

      IF (LLUSEPB1) THEN
        DO JFLD=1,NFLDSLB1
          DO JROF=ISTGLO,MIN(ISTGLO-1+NPROMA,IGPCOMP)
            ZSLBUF1AU(JROF-ISTGLO+1,JFLD,ILL)= 0.0_JPRB
          ENDDO
        ENDDO
      ENDIF

!     ------------------------------------------------------------------

!*       4.2    Sets-up some quantities for DDH.
!               A subset of these quantities may be modified by the MF-physics.

      IF (LSDDH.AND. NCURRENT_ITER == 0 .AND. NPHYINT == 0) THEN
        CALL GPINIDDH(IST,IEND,IDDHI,ZDHSF,ZDHCV,ZNEB,ZCLCT,ISTGLO)  
      ENDIF

!     ------------------------------------------------------------------
! -- need probably a test on LEPHYS here.

!*       4.3    ECMWF-PHYSICS.

!*       4.3.1   Call ECMWF unlagged physics.

!   CALL EC_PHYS(...) to plug here.

!*       4.3.2   Call ECMWF split physics.

      IF (LLSLPHY) THEN
        CALL EC_PHYS_LSLPHY(&
         & IST,IEND,LLFSTEP,ZDTPHY, &
         & ZSLBUF1AU(1,MSLB1UP9,ILL),ZSLBUF1AU(1,MSLB1VP9,ILL), &
         & ZSLBUF1AU(1,MSLB1TP9,ILL),ZSLBUF1AU(1,MSLB1GFLP9,ILL), &
         & SAVTEND(1,1,1,IBL),GFLSLP(:,:,:,IBL))
      ENDIF

!     ------------------------------------------------------------------

!*       4.4    MF-PHYSICS.

      IF( (LMPHYS.OR.LSIMPH) .AND. (NCURRENT_ITER == 0) ) THEN  

!*       4.4.1   Call MF unlagged physics (predictor only if PC scheme).

        CALL MF_PHYS_PREP( &
         & IST,IEND,YROROG(IBL)%OROG, &
         & PGMV(1,1,1,IBL),ZRCP0(1,1,YYTRCP0%M_R,ILL),ZRCP9(1,1,YYTRCP9%M_CP,ILL), &
         & ZGWFT0(1,1,ILL),ZGWFT9(1,1,ILL),ZGWFL(1,1,ILL),ZGWFM(1,1,ILL), &
         & ZNHPRE0F(1,1,ILL),ZNHPRE9F(1,1,ILL),ZNHPRE0H(1,0,ILL),ZNHPRE9H(1,0,ILL), &
         & ZPRE0F(1,1,ILL),ZPRE9F(1,1,ILL),ZPRE0(1,0,ILL),ZPRE9(1,0,ILL), &
         & ZXYB0(1,1,1,ILL),ZXYB9(1,1,1,ILL), &
         & ZW0_PHY,ZW9_PHY,ZW0L_PHY,ZW0M_PHY,ZPRE0F_PHY,ZPRE9F_PHY,ZPRE0_PHY,ZPRE9_PHY,&
         & ZPREHYD0F_PHY,ZPREHYD9F_PHY,ZPREHYD0_PHY,ZPREHYD9_PHY, &
         & ZXYB0_PHY,ZXYB9_PHY)

        CALL MF_PHYS &
         & (CDCONF,IBL,IGPCOMP,IST,IEND,IGL1,IGL2,IGL3,IGL4,IOFF,ISTGLO, &
         & LDRETCFOU,LDWRTCFOU0,LLADNMI,ZDTPHY, &
         & YRCSGEOM(IBL),YRGSGEOM(IBL),YROROG(IBL),ZOROGL(1,ILL),ZOROGM(1,ILL), &
         & RCUCONVCA(IOFF),RNLCONVCA(IOFF), &
         & PGMV(1,1,1,IBL),PGMVS(1,1,IBL),GFL(:,:,:,IBL),&
         & ZW0_PHY,ZW0L_PHY,ZW0M_PHY,ZRCP0(1,1,1,ILL),ZPHI0(1,0,ILL),ZPHIF0(1,1,ILL), &
         & ZPREHYD0_PHY,ZPREHYD0F_PHY,ZPRE0_PHY,ZPRE0F_PHY,ZXYB0_PHY, &
         & ZVCRS0(1,0,YYTVC0%M_VCASRSH,ILL), &
         & ZW9_PHY,ZRCP9(1,1,1,ILL),ZPHI9(1,0,ILL),ZPHIF9(1,1,ILL), &
         & ZPREHYD9_PHY,ZPREHYD9F_PHY,ZPRE9_PHY,ZPRE9F_PHY,ZXYB9_PHY, &
         & ZKOZO(1,1,ILL), &
         & ZGPSDT2D(1,1,IBL), &
         & ZCTY0(1,0,1,ILL), &
         & ZSLBUF1AU(1,1,ILL),PB2(1,1,IBL),&
         & PGMVT1(1,1,1,IBL),PGMVT1S(1,1,IBL),PGFLT1(1,1,1,IBL), &
         & SP_SB(:,:,:,IBL),SP_SG(:,:,IBL),SP_RR(:,:,IBL),&
         & SD_VF(:,:,IBL),SD_VP(:,:,IBL),SD_VV(:,:,IBL),SD_VH(:,:,IBL), &
         & SD_VA(:,:,IBL),SD_VC(:,:,IBL),SD_VD(:,:,IBL),SD_SFL(:,:,IBL),SD_SFO(:,:,IBL),&
         & EMTD(1,1,IBL), EMTU(1,1,IBL), TRSW(1,1,IBL),RMOON(1,IBL),&
         & ZGPAR(1,1,ILL), &
         & GDEOSI(1,0,1,IBL),GUEOSI(1,0,1,IBL),&
         & GMU0(1,1,IBL),GMU0_MIN(1,IBL),GMU0_MAX(1,IBL),&
         & GDEOTI(1,0,IBL),GDEOTI2(1,0,IBL),GUEOTI(1,0,IBL),&
         & GUEOTI2(1,0,IBL),GEOLT(1,1,IBL),GEOXT(1,1,IBL),&
         & GRPROX(1,0,IBL),GMIXP(1,0,IBL),GFLUXC(1,0,IBL),GRSURF(1,IBL),&
         & ZDHSF,ZALB,ZCAPE,ZCLCC,ZCLCH,ZCLCL,ZCLCM,ZCLCT,ZCLPH,ZVEIN,ZCT, &
         & ZDIFCQ,ZDIFCQN,ZDIFCQL,ZDIFCS,ZDIFTQ,ZDIFTQN,ZDIFTQL,ZDIFTS, &
         & ZFCCQL,ZFCCQN,ZFCHOZ,ZFCHSP,ZFCLL,ZFCLN,ZFCQNNG,ZFCQLNG,ZFCQNG, &
         & ZFCS,ZFCSQL,ZFCSQN,ZFEVL,ZFEVN,ZFEVV,ZFGEL,ZFGELS,ZFLWSP,ZFONTE, &
         & ZFPLCL,ZFPLCN,ZFPLCG,ZFPLCH,ZFPLSL,ZFPLSN,ZFPLSG,ZFPLSH,&
         & ZFRMH,ZFRSO,ZFRSOC,ZFRSODS,ZFRSOLU, &
         & ZFRSOPS,ZFRSOPT,ZFRTH,ZFRTHC,ZFRTHDS,ZFTR,ZGZ0,ZGZ0H,ZNEB, &
         & ZQCLS,ZQICE(1,1,ILL),ZQLI(1,1,ILL),ZQS(1,ILL), &
         & ZRH,ZRHCLS,ZRUISL,ZRUISP,ZRUISS, &
         & ZSTRCU,ZSTRCV,ZSTRDU,ZSTRDV,ZSTRMU,ZSTRMV,ZSTRTU,ZSTRTV, &
         & ZDIFCQLC,ZDIFCQIC,ZFIMCC, &
         & ZFEDQLC,ZFEDQIC,ZFEDQRC,ZFEDQSC,ZFCNEGQLC,ZFCNEGQIC,ZFCNEGQRC,ZFCNEGQSC, &
         & ZUCLS,ZVCLS,ZTCLS,ZUGST,ZVGST, &
         & ZMOCON,ZFDIS, &
         & ZFHPCL,ZFHPCN,ZFHPSL,ZFHPSN,ZFHSCL,ZFHSCN,ZFHSSL,ZFHSSN, &
         & ZFEPFP, ZFCMPCQ, ZFCMPSN, ZFCMPSL,&
         & ZTENDU,ZTENDV, &
         & ZQSOL(1,ILL), &
         & ZFPFPSL,ZFPFPSN,ZFPFPCL,ZFPFPCN,ZFPEVPSL,ZFPEVPSN, &
         & ZFPEVPCL,ZFPEVPCN,ZDRNSHF, &
         & ZFCQRNG, ZFCQSNG, ZDIAGH)

      ENDIF

!*       4.4.2   Store phy. tends.

      IF ( LPC_FULL.AND.(.NOT.LPC_CHEAP).AND.LSLAG.AND.(LMPHYS.OR.LSIMPH) ) THEN

        CALL CP_PTRSLB1(ISLB1U9,ISLB1V9,ISLB1T9,ISLB1VD9,ISLB1GFL9)

        ! * currently valid only for SL2TL advection scheme.
        !   Remarks KY:
        !   - PC scheme with Eulerian scheme: physics is passed differently
        !     from predictor to corrector step, and no call of CPG_PT_ULP is done.
        !   - LPC_CHEAP: physics is passed differently from predictor to
        !     corrector step (in LAPINEB), and no call of CPG_PT_ULP is done.
        !   - pressure departure variable: its diabatic tendency is currently
        !     assumed to be zero and it is ignored.
        LLGET=(NCURRENT_ITER > 0)
        CALL CPG_PT_ULP(IST,IEND,LLGET,NFLSA,NFLEN, &
         & PTENDU=ZSLBUF1AU(1,ISLB1U9,ILL), &
         & PTENDV=ZSLBUF1AU(1,ISLB1V9,ILL), &
         & PTENDT=ZSLBUF1AU(1,ISLB1T9,ILL), &
         & PTENDVD=ZSLBUF1AU(1,ISLB1VD9,ILL), &
         & PTENDGFL=ZSLBUF1AU(1,ISLB1GFL9,ILL), &
         & PTENDSP=ZSLBUF1AU(1,MSLB1SP9,ILL), &
         & PGFLPT=GFLPT(:,:,:,IBL),&
         & PGFLT1=PGFLT1(1,1,1,IBL),PGFL=GFL(:,:,:,IBL),&
         & PGMV=PGMV(1,1,1,IBL),PGMVS=PGMVS(1,1,IBL))

      ENDIF

!     ------------------------------------------------------------------

!*       4.5    DIAGNOSTICS.
!*              - for dynamics, uses outputs of CPG_GP (MF and ECMWF).
!*              - for physics, uses outputs of MF_PHYS (MF only).

      IF (NCURRENT_ITER == 0 .AND. NPHYINT == 0) THEN
        CALL CPG_DIA(CDCONF,IST,IEND,ISTC,IENDC,ISTGLO,IDDHI, &
         & LLDIAB,LLADNMI,LLFSTEP,LLRESET,NMEANSTEPS,NXGSTTS,&
         & YRGSGEOM(IBL),YRCSGEOM(IBL),ZOROGL(1,ILL),ZOROGM(1,ILL), &
         & PGMV(1,1,1,IBL),GFL(:,:,:,IBL), &
         & ZPRE0(1,0,ILL),ZPRE0F(1,1,ILL),ZXYB0(1,1,1,ILL), &
         & ZPHI0(1,0,ILL),ZPHIF0(1,1,ILL),ZPHI0FL(1,1,ILL),ZPHI0FM(1,1,ILL), &
         & ZRCP0(1,1,1,ILL),ZCTY0(1,0,1,ILL),PB2(1,MSLB2VVEL,IBL),ZKENE0(1,1,ILL), &
         & ZRT0L(1,1,ILL),ZRT0M(1,1,ILL),ZPRE0L(1,ILL),ZPRE0M(1,ILL), &
         & ZNHPRE0F(1,1,ILL),ZNHPRE0H(1,0,ILL), &
         & ZQCHA0L(1,1,ILL),ZQCHA0M(1,1,ILL), &
         & ZVCRS0(1,0,1,ILL),ZVCW0F(1,1,ILL), &
         & ZRDLR0(1,0,1,ILL), &
         & SD_VF(:,:,IBL),SP_SG(:,:,IBL),SP_SB(:,:,:,IBL),SP_RR(:,:,IBL),&
         & ZDHSF, &
         & ZALB,ZCAPE,ZCLCC,ZCLCH,ZCLCL,ZCLCM,ZCLCT,ZCLPH,ZVEIN,ZCT, &
         & ZDIFCQ,ZDIFCQN,ZDIFCQL,ZDIFCS,ZDIFTQ,ZDIFTQN,ZDIFTQL,ZDIFTS, &
         & ZFCCQL,ZFCCQN,ZFCHSP,ZFCLL,ZFCLN,ZFCQNNG,ZFCQLNG,ZFCQNG,ZFCS, &
         & ZFCSQL,ZFCSQN,ZFEVL,ZFEVN,ZFEVV,ZFGEL,ZFGELS,ZFLWSP,ZFONTE, &
         & ZFPLCL,ZFPLCN,ZFPLCG,ZFPLCH,ZFPLSL,ZFPLSN,ZFPLSG,ZFPLSH,&
         & ZFRMH,ZFRSO,ZFRSOC,ZFRSODS,ZFRSOLU, &
         & ZFRSOPS,ZFRSOPT,ZFRTH,ZFRTHC,ZFRTHDS,ZFTR,ZGZ0,ZGZ0H,ZNEB, &
         & ZQICE(1,1,ILL),ZQLI(1,1,ILL),ZQS(1,ILL), &
         & ZRH,ZRHCLS,ZRUISL,ZRUISP,ZRUISS, &
         & ZSTRCU,ZSTRCV,ZSTRDU,ZSTRDV,ZSTRTU,ZSTRTV, &
         & ZUCLS,ZVCLS,ZTCLS,ZQCLS,ZUGST,ZVGST, &
         & ZMOCON,ZFDIS, &
         & ZFHPCL,ZFHPCN,ZFHPSL,ZFHPSN,ZFHSCL,ZFHSCN,ZFHSSL,ZFHSSN, &
         & ZFEPFP, ZFCMPCQ, ZFCMPSN, ZFCMPSL, &
         & ZFPFPSL,ZFPFPSN,ZFPFPCL,ZFPFPCN,ZFPEVPSL,ZFPEVPSN, &
         & ZFPEVPCL,ZFPEVPCN,ZGPAR(1,1,ILL), &
         & ZFCQRNG, ZFCQSNG,ZDIAGH, &
         & GMVTNDSI_DDH(1,1,1,IBL),GMVTNDHD_DDH(1,1,1,IBL),&
         & GFLTNDHD_DDH(1,1,1,IBL),ZATND(1,1,1,ILL),&
         & ZDHCV, ZDRNSHF)
      ENDIF

      IF (LSLDIA .AND. (NCURRENT_ITER == NSITER) .AND. NPHYINT == 0 ) THEN
        ! * Calculate and print SL dynamics diagnostics.
        CALL CPDYSLDIA(NPROMA,LLFSTEP,IST,IEND,NFLEVG, &
         & YRGSGEOM(IBL),YRCSGEOM(IBL), &
         & PGMVS(1,YT0%MSP,IBL),ZXYB0(1,1,YYTXYB0%M_DELP,ILL),ZPRE0F(1,1,ILL), &
         & PGMV(1,1,YT0%MT,IBL),GFL(:,:,:,IBL),PEXTRA(1,1,1,IBL))
      ENDIF

    ENDIF

!     ------------------------------------------------------------------

    IF(.NOT.LDCPG_SPLIT .OR. JSPLIT == 3) THEN

!*       5.    DYNAMICS.
!              ---------

!*       5.1   Call dynamics.

      IF (YSD_ACT_VF%L_LSM) THEN
        ZLSM(IST:IEND)=SD_VF(IST:IEND,YSD_VF%YLSM%MP,IBL)
      ELSE
        ZLSM(IST:IEND)=0.0_JPRB
      ENDIF
      IF (YSP_ACT_RR%L_T) THEN
        ZTSOL(IST:IEND)=SP_RR(IST:IEND,YSP_RR%YT%MP0,IBL)
      ELSE
        ZTSOL(IST:IEND)=PGMV(IST:IEND,NFLEVG,YT0%MT,IBL)
      ENDIF

      CALL CPG_DYN( &
       !---------------------------------------------------------------------
       ! - INPUT .
       & IST,IEND,ISTGLO,LLADNMI,ZBETADT,ZDT, &
       & SLHDA(IOFF),SLHDD0(IOFF), &
       & YRCSGEOM(IBL),YRGSGEOM(IBL),YROROG(IBL),ZOROGL(1,ILL),ZOROGM(1,ILL), &
       & ZPRE0(1,0,ILL),ZXYB0(1,1,YYTXYB0%M_RDELP,ILL),ZUVH0(1,0,1,ILL), &
       & ZPHI0(1,0,ILL),ZPHIF0(1,1,ILL),ZRCP0(1,1,1,ILL), &
       & ZCTY0(1,0,1,ILL),ZKENE0(1,1,ILL), &
       & ZVCRS0(1,0,1,ILL),ZVCW0F(1,1,ILL),ZWDLW0F(1,1,ILL), &
       & ZVCRSSA9F(1,1,ILL),ZRDLR0(1,0,1,ILL),ZRDLR9(1,0,1,ILL), &
       & ZLSM,ZTSOL,ZQSOL(1,ILL), &
       & ZATND(1,1,1,ILL), &
       & ZDBBC(1,ILL),ZRDPHI(1,1,ILL),ZGWT0(1,1,ILL),ZGWT9(1,1,ILL),&
       & ZNHX0(1,1,ILL),ZNHX9(1,1,ILL),&
       & GFL(:,:,:,IBL), &
       !---------------------------------------------------------------------
       ! - INPUT/OUTPUT .
       & GFLPC(:,:,:,IBL),ZGWS(1,ILL),PGMV(1,1,1,IBL),PGMVS(1,1,IBL), &
       & ZSLBUF1AU(1,1,ILL),PB2(1,1,IBL),PGMVT1(1,1,1,IBL),PGMVT1S(1,1,IBL), &
       & PGFLT1(1,1,1,IBL),GMVTNDSI_DDH(1,1,1,IBL))
    
!*       5.2   Update PB1.

      IF (LLUSEPB1) THEN
        DO JFLD=1,NFLDSLB1
          DO JROF=ISTGLO,MIN(ISTGLO-1+NPROMA,IGPCOMP)
            PB1(YDSL%NSLCORE(JROF),JFLD)=ZSLBUF1AU(JROF-ISTGLO+1,JFLD,ILL)
          ENDDO
        ENDDO
      ENDIF

!     ------------------------------------------------------------------

!*       6.    FINAL PART OF NON-LAGGED GRID-POINT COMPUTATIONS.
!              -------------------------------------------------

      CALL CPG_END( &
       !---------------------------------------------------------------------
       ! - INPUT .
       & CDCONF,IST,IEND,ISTC,IENDC,IBL,ISTGLO,LLADNMI,LLDIAB, &
       & YRGSGEOM(IBL)%GM,ZQS(1,ILL), &
       !---------------------------------------------------------------------
       ! - INPUT/OUTPUT .
       & GFL(:,:,:,IBL),ZGPAR(1,1,ILL),PGMV(1,1,1,IBL),PGMVS(1,1,IBL))

!     ------------------------------------------------------------------

    ENDIF

  ENDDO

!$OMP END PARALLEL DO

  IF(LDCPG_SPLIT) THEN
    IF(JSPLIT == 1) CALL GSTATS(1025,1)
    IF(JSPLIT == 2) CALL GSTATS(1026,1)
    IF(JSPLIT == 3) CALL GSTATS(1027,1)
  ELSE
    CALL GSTATS(1025,1)
  ENDIF

ENDDO

!     ------------------------------------------------------------------

!*       7.    FINALIZATIONS.
!              --------------

IF (LMUSCLFA) CALL LFAFER(IMUSCLFA)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPG',1,ZHOOK_HANDLE)
END SUBROUTINE CPG
