SUBROUTINE CPG_DIA( &
 !---------------------------------------------------------------------
 ! - INPUT .
 & CDCONF, &
 & KST,KEND,KSTC,KENDC,KSTGLO,KDDHI, &
 & LDLDIAB,LDLADNMI,LDLFSTEP,LDLRESET,KMEANSTEPS, KXGSTTS, &
 & YDGSGEOM,YDCSGEOM,POROGL,POROGM, &
 & PGMV,PGFL, &
 & PRE0,PRE0F,PXYB0, &
 & PHI0,PHIF0,PHI0FL,PHI0FM,PRCP0, &
 & PCTY0,POMEGA,PKENE0, &
 & PRT0L,PRT0M,PRE0L,PRE0M, &
 & PNHPRE0F,PNHPRE0H,PQCHA0L,PQCHA0M, &
 & PVCRS0,PVCW0F,PRDLR, &
 & PSD_VF,PSP_SG,PSP_SB,PSP_RR, &
 & PDHSF, &
 & PALB,PCAPE,PCLCC,PCLCH,PCLCL,PCLCM,PCLCT,PCLPH,PVEIN,PCT, &
 & PDIFCQ,PDIFCQN,PDIFCQL,PDIFCS,PDIFTQ,PDIFTQN,PDIFTQL,PDIFTS, &
 & PFCCQL,PFCCQN,PFCHSP,PFCLL,PFCLN,PFCQNNG,PFCQLNG,PFCQNG,PFCS, &
 & PFCSQL,PFCSQN,PFEVL,PFEVN,PFEVV,PFGEL,PFGELS,PFLWSP,PFONTE, &
 & PFPLCL,PFPLCN,PFPLCG,PFPLCH,PFPLSL,PFPLSN,PFPLSG,PFPLSH,&
 & PFRMH,PFRSO,PFRSOC,PFRSODS,PFRSOLU, &
 & PFRSOPS,PFRSOPT,PFRTH,PFRTHC,PFRTHDS,PFTR,PGZ0,PGZ0H,PNEB, &
 & PQICE,PQLI,PQS, &
 & PRH,PRHCLS,PRUISL,PRUISP,PRUISS, &
 & PSTRCU,PSTRCV,PSTRDU,PSTRDV,PSTRTU,PSTRTV, &
 & PUCLS,PVCLS,PTCLS,PQCLS,PUGST,PVGST, &
 & PMOCON,PFDIS, &
 & PFHPCL,PFHPCN,PFHPSL,PFHPSN,PFHSCL,PFHSCN,PFHSSL,PFHSSN, &
 & PFEPFP, PFCMPCQ, PFCMPSN, PFCMPSL,&
 & PFPFPSL,PFPFPSN,PFPFPCL,PFPFPCN,PFPEVPSL,PFPEVPSN, &
 & PFPEVPCL,PFPEVPCN,PGPAR, &
 & PFCQRNG, PFCQSNG,PDIAGH,&
 & PGMVTNDSI,PGMVTNDHD,PGFLTNDHD,PATND,&
 !---------------------------------------------------------------------
 ! - INPUT/OUTPUT .
 & PDHCV, PDRNSHF)

!**** *CPG_DIA* - Grid point calculations: diagnostics.

!     Purpose.
!     --------
!           Grid point calculations: diagnostics using non lagged dynamics
!           and non lagged physics (DDH, CFU, XFU).

!**   Interface.
!     ----------
!        *CALL* *CPG_DIA(...)*

!        Explicit arguments :
!        --------------------

!     INPUT:
!     ------
!        CDCONF    : configuration.
!        KST       : first element of work.
!        KEND      : last element of work.
!        KSTC,KENDC: the same as KST,KEND but including zone C for ALADIN.
!        KSTGLO    : global offset.
!        KDDHI     : internal domain to which each point belongs (DDH).
!        LDLDIAB   : .T. if complete physics is activated.
!        LDLADNMI  : .T. if adiabatic NNMI iteration.
!        LDLFSTEP  : .T. if first step.
!        LDLRESET  : .T. if we reset Tn, Tx, HUn, HUx calculation (cpxfu)
!        KMEANSTEPS: number of time steps involved in mean cls wind calculation
!        KXGSTTS   : number of time steps involved in max gust calculation
!        YDGSGEOM  : structure for geographical sphere horizontal geometry
!        YDCSGEOM  : structure for computational sphere horizontal geometry
!        POROGL    : zonal component of "rssavnabla(surf orography)"
!        POROGM    : meridian component of "rssavnabla(surf orography)"
!        PGMV      : upper air GMV variables at time t and t-dt.
!        PGFL      : unified_treatment grid-point fields at t.
!        PRE0      : hydrostatic pressure "prehyd" on interlayers at time t.
!        PRE0F     : hydrostatic pressure "prehyd" on layers at time t.
!        PXYB0     : contains pressure depth, "delta", "alpha" at t.
!        PHI0      : geopotential height "gz" at t on interlayers.
!        PHIF0     : geopotential height "gz" at t on layers.
!        PHI0FL    : zonal component of "rssavnabla (gz)" on layers.
!        PHI0FM    : meridian component of "rssavnabla (gz)" on layers.
!        PRCP0     : contains "cp", "R" and "Kap=R/Cp" at t.
!        PCTY0     : contains vertical velocities, vertical integral of divergence at t.
!        POMEGA    : "omega" on layers at t, including the "lrubc" and
!                    "delta m=1" effects.
!        PKENE0    : kinetic energy at t.
!        PRT0L     : zonal component of "rssavnabla RT" on layers.
!        PRT0M     : meridian component of "rssavnabla RT" on layers.
!        PRE0L     : zonal component of "rssavnabla prehyds" on layers.
!        PRE0M     : meridian component of "rssavnabla prehyds" on layers.
!        PNHPRE0F  : "pre" at full levels (time t).
!        PNHPRE0H  : "pre" at half levels (time t).
!        PQCHA0L   : zonal comp grad(log(pre/prehyd)).
!        PQCHA0M   : merid comp grad(log(pre/prehyd)).
!        PVCRS0    : array containing quantities for LVERCOR=T (for ex. rs/a) at t.
!        PVCW0F    : pseudo-vertical velocity " W = Drs/Dt " on layers at t.
!        PRDLR     : array containing "r/a" and "a/r".
!        PSD_VF    : diagnostic surface fields VF=VARSF.
!        PSP_SG    : prognostic surface fields SG=SNOWG.
!        PSP_SB    : prognostic surface fields SB=SOILB.
!        PSP_RR    : prognostic surface fields RR=RESVR.
!        PDHSF     : distribution of horizontal mean weights used for
!                    simplified radiation scheme.
!        ---------------------- output of aplpar ------------------------------
!        PALB      : model surface shortwave albedo.
!        PCAPE     : CAPE.
!        PCLCC     : convective cloud cover (diagnostic).
!        PCLCH     : high cloud cover (diagnostic).
!        PCLCL     : low cloud cover (diagnostic).
!        PCLCM     : medium cloud cover (diagnostic).
!        PCLCT     : total cloud cover (diagnostic).
!        PCLPH     : height (in meters) of the PBL.
!        PVEIN     : ventilation index in the PBL.
!        PCT       : thermical coefficient of soil-vegetation middle.
!        PDIFCQ    : convective flux of specific humidity (not rain/snow).
!        PDIFCQN   : convective flux of solid water (not rain/snow).
!        PDIFCQL   : convective flux of liquid water (not rain/snow).
!        PDIFCS    : convective flux of enthalpy (not rain/snow).
!        PDIFTQ    : turbulent flux (inc. "q" negative) of specific humidity.
!        PDIFTQN   : turbulent flux (inc. "q" negative) of solid water.
!        PDIFTQL   : turbulent flux (inc. "q" negative) of liquid water.
!        PDIFTS    : turbulent flux of enthalpy (or dry static energy).
!        PFCCQL    : convective condensation flux for liquid water.
!        PFCCQN    : convective condensation flux for ice.
!        PFCHSP    : heat flux from surface to deep soil.
!        PFCLL     : latent heat flux over liquid water (or wet soil).
!        PFCLN     : latent heat flux over snow (or ice).
!        PFCQNNG   : pseudo-flux of ice to correct for "qi"<0.
!        PFCQLNG   : pseudo-flux of liquid water to correct for "ql"<0.
!        PFCQNG    : pseudo-flux of water to correct for Q<0.
!        PFCS      : sensible heat flux at surface level.
!        PFCSQL    : stratiform condensation flux for liquid water.
!        PFCSQN    : stratiform condensation flux for ice.
!        PFEVL     : water vapour flux over liquid water (or wet soil).
!        PFEVN     : water vapour flux over snow (or ice) and frozen soil.
!        PFEVV     : evapotranspiration flux.
!        PFGEL     : freezing flux of soil water.
!        PFGELS    : freezing flux of soil water at surface level.
!        PFLWSP    : water flux from surface to deep soil.
!        PFONTE    : water flux corresponding to surface snow melt.
!        PFPLCL    : convective precipitation as rain.
!        PFPLCN    : convective precipitation as snow.
!        PFPLCG    : convective precipitation as graupel.
!        PFPLCH    : convective precipitation as hail.
!        PFPLSL    : stratiform precipitation as rain.
!        PFPLSN    : stratiform precipitation as snow.
!        PFPLSG    : stratiform precipitation as graupel.
!        PFPLSH    : stratiform precipitation as hail.
!        PFRMH     : mesospheric enthalpy flux.
!        PFRSO     : shortwave radiative flux.
!        PFRSOC    : shortwave clear sky radiative flux.
!        PFRSODS   : surface downwards solar flux.
!        PFRSOLU   : downward lunar flux at surface.
!        PFRSOPS   : surface parallel solar flux.
!        PFRSOPT   : top parallel solar flux.
!        PFRTH     : longwave radiative flux.
!        PFRTHC    : longwave clear sky radiative flux.
!        PFRTHDS   : surface downwards IR flux.
!        PFTR      : transpiration flux.
!        PGZ0      : g*roughness lenght (current).
!        PGZ0H     : current g*thermal roughness lenght (if KVCLIV >= 8).
!        PNEB      : fractional cloudiness for radiation.
!        PQICE     : specific humidity of solid water for radiation.
!        PQLI      : specific humidity of liquid water for radiation.
!        PQS       : specific humidity at surface level.
!        PRH       : relative humidity.
!        PRHCLS    : relative humidity at 2 meters (diagnostic).
!        PRUISL    : run-off flux out the interception water-tank.
!        PRUISP    : run-off flux in soil.
!        PRUISS    : run-off flux at surface level.
!        PSTRCU    : convective flux of momentum "U".
!        PSTRCV    : convective flux of momentum "V".
!        PSTRDU    : gravity wave drag flux "U".
!        PSTRDV    : gravity wave drag flux "V".
!        PSTRTU    : turbulent flux of momentum "U".
!        PSTRTV    : turbulent flux of momentum "V".
!        PUCLS     : U-component of wind at 10 meters (diagnostic).
!        PVCLS     : V-component of wind at 10 meters (diagnostic).
!        PTCLS     : temperature at 2 meters (diagnostic).
!        PQCLS     : specific humidity at 2 meters (diagnostic).
!        PUGST     : U-component of gusts (diagnostic).
!        PVGST     : V-component of gusts (diagnostic).
!        ---------------------- end of output of aplpar -----------------------
!        PMOCON    : moisture convergence.
!        PFDIS     : enthalpy flux due to dissipation of kinetic energy. 
!        PFHPCL    : liquid water convective condensation enthalpy flux.
!        PFHPCN    : snow convective condensation enthalpy flux.
!        PFHPSL    : liquid water stratiform condensation enthalpy flux.
!        PFHPSN    : snow stratiform condensation enthalpy flux.
!        PFHSCL    : sensible heat flux due to liquid convective precipitations
!        PFHSCN    : sensible heat flux due to snow convective precipitations.
!        PFHSSL    : sensible heat flux due to liquid stratiform precipitations
!        PFHSSN    : sensible heat flux due to snow stratiform precipitations.
!        PFPFPSL   : flux of liquid resolved precipitation: the generation term.
!        PFPFPSN   : flux of solid resolved precipitation: the generation term.
!        PFPFPCL   : flux of liquid conv. precipitation: the generation term.
!        PFPFPCN   : flux of solid conv. precipitation: the generation term.
!        PFPEVPSL  : resolved precipitation flux due to evaporation.
!        PFPEVPSN  : resolved precipitation flux due to sublimation.
!        PFPEVPCL  : convective precipitation flux due to evaporation.
!        PFPEVPCN  : convective precipitation flux due to sublimation.
!        PGMVTNDSI : tendencies of semi-implicit scheme.
!        PGMVTNDHD : tendencies of horizontal diffusion scheme for GMV.
!        PGFLTNDHD : tendencies of horizontal diffusion for spectrally treated GFL.
!        PATND     : adiabatic Lagrangian tendencies.


!     INPUT/OUTPUT:
!     -------------
!        PDHCV     : result array containing flux divergences and fluxes.
!                    Entering this routine this array is assumed to be
!                    pre-initialised with zeros.
!        PDRNSHF   : Derivative of non solar surface fluxes

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ARPEGE documentation about Eulerian and semi-Lagrangian schemes.

! Author
! ------
!   13-Aug-2001 K. YESSAD after part 8 of CPG. 

! Modifications
! -------------
!   04-Jan 2002 K. YESSAD  pruning; thin layer hyp relaxation for SL.
!      Mar-2002 J. Vivoda  PC schemes for NH dynamics (LPC_XXXX keys)
!   16-Mar-2003 M. Hamrud  Revised data flow (GFL)
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   12-Dec_2003 F. Vinit   New Input for CPPHDDH: Surface Albedo, surfaces 
!                          Roughness diagnosis, surface downwards IR flux,
!                          surface downwards solar flux. 
!   09-Jun-2004 J. Masek   NH cleaning (LDFULLIMP)
!   01-Jul-2004 K. Yessad  Make clearer the tests for PC scheme.
!   26-Mar-2004 Y. Bouteloup  Introduction of MTS radiance
!   20-Jun-2005 F. Bouyssel Lopez's prognostic scheme
!   05-10-2005  A. Alias : PDRNSHF added (JF Royer)
!                          Use of CUMCPL instead of CUMCODM
!   10-Nov-2005 F. Bouyssel New arguments (PQLI,PQICE) in CPXFU
!   05-APR-2006 T. Kovacic Introduced changes needed for AROME.
!   27-Jan-2006 F. Bouyssel New PFPEVPL/N Remove PFPEVPP,PFADVP,PFHMLTS,PFHEVPP
!   15-Jan-2007 Y. Seity add graupel and hain in CFU/XFU
!   08-Mar-2007 T. Kovacic  Surface variables and fluxes set to zero for AROME
!   17-Apr-2007 S. Ivatek-S Over dimensioning of PGPAR to NGPAR+1, boundary
!                           checking problem if NGPAR=0
!   16-Mar-2007 T. Kovacic  DDH surface arrays to zero for AROME. 
!   31-May-2007 T.Kovacic:  New arguments PFCQRNG, PFCQSNG, PFEPFP, PFCMPCQ, 
!                           PFCMPSN, PFCMPSL.
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   04-Dec-2007 F. Bouyssel Bug correction on PQICE,PQLI to have full (stra+conv)
!                           3D radiative clouds in XFU and CFU diagnostics
!   04-Mar-2008 Y. Seity  Cleaning MTS pictures (IR and WV) replaced by Fullpos
!   01-Oct-2008 O.riviere Introduction of new data flow for DDH
!   K. Yessad (Dec 2008): remove dummy CDLOCK + cleanings
!   18-May-2009 S. Riette : mean cls wind added
!   14-Oct-2009 A. Alias   ZUCLS, ZVCLS added in CUMCPLDM (E.Maisonnave)
!   04-Jan-2011 Y. Seity Add PDIAGH (AROME hail daignostic)
!   K. Yessad (Nov 2009): cleanings, DT/Dt now pre-computed in CPG_GP.
!   K. Yessad (Jan 2011): introduce INTDYN_MOD structures.
!   2011-10-04 J.M. Piriou: do not call CPPHDDH if flexible DDH.
!   K. Yessad (Nov 2011): various modifications.
! End Modifications
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEPHY   , ONLY : LEPHYS   ,LAGPHY 
USE YOMCT0   , ONLY : LVERCOR  ,LAROME   ,NSTOP
USE YOMCT3   , ONLY : NSTEP
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NPROMM   ,NPROMVC  ,&
 & NPROMDLR ,NPROMNH  ,NFTHER
USE YOMLDDH  , ONLY : LSDDH    ,LFLEXDIA
USE YOMINI   , ONLY : LINITER
USE YOMMDDH  , ONLY : NDHCSSU  ,NDHCVSUN
USE YOMPHY   , ONLY : LMPHYS
USE YOMSIMPHL, ONLY : LSIMPH   ,LTRAJPS 
USE YOMCFU   , ONLY : LCUMFU
USE YOMGMV   , ONLY : YT0      ,NDIMGMV
USE YOM_YGFL , ONLY : YGFL,YQ, YL, YI, YA, YO3
USE YOMPPC   , ONLY : LMOVPH
USE YOMDPHY  , ONLY : NTSSG
USE SURFACE_FIELDS_MIX  , ONLY : YSD_VF   ,YSD_VFD  ,&
 & YSP_SB   ,YSP_SBD  ,YSP_RR   ,YSP_RRD  ,YSP_SG   ,YSP_SGD
USE YOMXFU   , ONLY : LXFU
USE YOMPARAR  , ONLY : NGPAR    ,MVQS     ,MVTS
USE YOMARPHY , ONLY : LMPA
USE YOMMCC   , ONLY : NFRCPL
USE YOMLUN   , ONLY : NULOUT
USE YOMWM    , ONLY : FTHSOR
USE YOMGEM   , ONLY : TCSGEOM
USE YOMGC    , ONLY : TGSGEOM
USE INTDYN_MOD,ONLY : YYTVC0   ,YYTTND   ,YYTCTY0  ,YYTRCP0  ,YYTXYB0
USE PAR_RDLR , ONLY : JP_DIMR
USE DDH_MIX  , ONLY : NTOTFIELD ,RDDH_FIELD ,RESET_DDHFLEX ,&
 & RDDHSURF_FIELD

!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KENDC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDDHI(NPROMA) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMEANSTEPS
INTEGER(KIND=JPIM),INTENT(IN)    :: KXGSTTS
LOGICAL           ,INTENT(IN)    :: LDLDIAB 
LOGICAL           ,INTENT(IN)    :: LDLADNMI 
LOGICAL           ,INTENT(IN)    :: LDLFSTEP 
LOGICAL           ,INTENT(IN)    :: LDLRESET 
TYPE(TGSGEOM)     ,INTENT(IN)    :: YDGSGEOM
TYPE(TCSGEOM)     ,INTENT(IN)    :: YDCSGEOM
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE0F(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXYB0(NPROMA,NFLEVG,YYTXYB0%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHI0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHIF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHI0FL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHI0FM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCP0(NPROMA,NFLEVG,YYTRCP0%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCTY0(NPROMA,0:NFLEVG,YYTCTY0%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMEGA(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKENE0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRT0L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRT0M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE0L(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE0M(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNHPRE0F(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNHPRE0H(NPROMNH,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQCHA0L(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQCHA0M(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCRS0(NPROMVC,0:NFLEVG,YYTVC0%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCW0F(NPROMVC,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDLR(NPROMDLR,0:NFLEVG,JP_DIMR)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSD_VF(NPROMA,YSD_VFD%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP_SG(NPROMA,YSP_SGD%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP_SB(NPROMA,YSP_SBD%NLEVS,YSP_SBD%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP_RR(NPROMA,YSP_RRD%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDHSF(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALB(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCAPE(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCC(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCH(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCL(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCM(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLCT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLPH(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEIN(NPROMM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCT(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQ(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCS(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQ(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTS(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCCQL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCCQN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCHSP(NPROMM,YSP_SBD%NLEVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCLL(NPROMM,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCLN(NPROMM,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQNNG(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQLNG(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQNG(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCS(NPROMM,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCSQL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCSQN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVL(NPROMM,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVN(NPROMM,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVV(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGEL(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFGELS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLWSP(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFONTE(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCH(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSH(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRMH(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSO(NPROMM,0:NFLEVG,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSOC(NPROMM,0:1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSODS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSOLU(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSOPS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSOPT(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTH(NPROMM,0:NFLEVG,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTHC(NPROMM,0:1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTHDS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFTR(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0H(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEB(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQICE(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQLI(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRH(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHCLS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISL(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISP(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCU(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCV(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDU(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDV(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTU(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTV(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUCLS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCLS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTCLS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQCLS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUGST(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVGST(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMOCON(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFDIS(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPCL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPCN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPSL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPSN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSCL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSCN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSSL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSSN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFEPFP(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPCQ(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPSN(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPSL(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPSL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPSN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPCL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPCN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPSL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPSN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPCL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPCN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGPAR(NPROMA,NGPAR+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQRNG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQSNG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIAGH(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVTNDSI(NPROMA,NFLEVG,2+NFTHER)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVTNDHD(NPROMA,NFLEVG,2+NFTHER)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFLTNDHD(NPROMA,NFLEVG,1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PATND(NPROMA,NFLEVG,YYTTND%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDHCV(NPROMA,0:NFLEVG,NDHCVSUN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDRNSHF(NPROMM)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: ILEVCO, JLEV,  JROF, JCV
INTEGER (KIND=JPIM) :: IPOS

LOGICAL :: LLDYN, LLPHY
LOGICAL :: LLCALL_CUMCPLDM,LLCALL_CPCFU,LLCALL_CPXFU,LLCALL_WMCUM

REAL(KIND=JPRB) :: ZDHCS(NPROMM,NDHCSSU)
REAL(KIND=JPRB) :: ZDUM1(NPROMA)
REAL(KIND=JPRB) :: ZENTRA(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZENTRV(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZNEB(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZQSAT(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRH(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSTRCU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRCV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRDU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRDV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRTU(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZSTRTV(NPROMM,0:NFLEVG)
REAL(KIND=JPRB) :: ZUCLS(NPROMM)
REAL(KIND=JPRB) :: ZUGST(NPROMM)
REAL(KIND=JPRB) :: ZUZGEO(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZVCLS(NPROMM)
REAL(KIND=JPRB) :: ZVGST(NPROMM)
REAL(KIND=JPRB) :: ZVMGEO(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZFPLCL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB) :: ZFPLCN(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB) :: ZFPLSL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB) :: ZFPLSN(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB) :: ZQICE(NPROMA,NFLEVG) 
REAL(KIND=JPRB) :: ZQLI(NPROMA,NFLEVG) 
REAL(KIND=JPRB) :: ZQS(NPROMA) 
REAL(KIND=JPRB) :: ZTSOL(NPROMA) 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!#include "posinsub.intfb.h"

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "cpcfu.intfb.h"
#include "cpcuddh.intfb.h"
#include "cpdyddh.intfb.h"
#include "cpphddh.intfb.h"
#include "cpxfu.intfb.h"
#include "cumcpl.intfb.h"
#include "gprh.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG_DIA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
IPOS=1
!CALL POSINSUB('CPG_DIA',IPOS)
!*       1.    DDH:
!              ----


IF ((NSTEP >= 0).AND.LSDDH.AND.(.NOT.LINITER).AND.(CDCONF /= 'X')) THEN 

  IF (LMPA) THEN
    ZQICE(1:NPROMA,1:NFLEVG) = PGFL(1:NPROMA,1:NFLEVG,YI%MP)
    ZQLI(1:NPROMA,1:NFLEVG) = PGFL(1:NPROMA,1:NFLEVG,YL%MP)
  ELSE
    IF(LMPHYS.OR.(LEPHYS.AND..NOT.LAGPHY).OR.LSIMPH) THEN
      ZFPLCL(1:NPROMA,0:NFLEVG) = PFPLCL(1:NPROMA,0:NFLEVG)
      ZFPLCN(1:NPROMA,0:NFLEVG) = PFPLCN(1:NPROMA,0:NFLEVG)
      ZFPLSL(1:NPROMA,0:NFLEVG) = PFPLSL(1:NPROMA,0:NFLEVG)
      ZFPLSN(1:NPROMA,0:NFLEVG) = PFPLSN(1:NPROMA,0:NFLEVG)
    ENDIF
    ZQICE(1:NPROMA,1:NFLEVG) = PQICE(1:NPROMA,1:NFLEVG)
    ZQLI(1:NPROMA,1:NFLEVG) = PQLI(1:NPROMA,1:NFLEVG)
  ENDIF

  IF(LAGPHY.OR.(.NOT.LDLDIAB)) THEN

    LLPHY=.FALSE.
    CALL GPRH(.FALSE.,NPROMA,KST,KEND,NFLEVG,1.0_JPRB,0.0_JPRB,&
     & PGFL(1,1,YQ%MP),PGMV(1,1,YT0%MT),PRE0F,ZQSAT,ZRH)  
  
    ZDUM1(KST:KEND)=0.0_JPRB
  
    IF(YA%LACTIVE) THEN
      ZNEB(KST:KEND,1:NFLEVG)=PGFL(KST:KEND,1:NFLEVG,YA%MP)
    ELSE
      IF (LMPA) THEN
        ZNEB(KST:KEND,1:NFLEVG)=0.0_JPRB    ! ???
      ELSE
        ZNEB(KST:KEND,1:NFLEVG)=PNEB(KST:KEND,1:NFLEVG)
      ENDIF
    ENDIF

  
    ! * COMP. OF DYNAMICAL VARIABLES AND FLUX/TENDENCIES
  
    CALL CPDYDDH (NPROMA,NPROMNH,NPROMVC,NPROMDLR,KST,KEND,NFLEVG,&
     & PVCRS0,PVCW0F,PRDLR,&
     & YDGSGEOM,YDCSGEOM,PXYB0,&
     & PRCP0,PHI0,PHIF0,PHI0FL,PHI0FM,&
     & PKENE0, PRT0L, PRT0M, POROGL, POROGM, &
     & PRE0L, PRE0M, PRE0F, PRE0,&
     & PNHPRE0F,PNHPRE0H,PQCHA0L,PQCHA0M, &
     & PCTY0, POMEGA,&
     & ZFPLCL, ZFPLCN, ZFPLSL, ZFPLSN,&
     & ZDUM1, ZDUM1, ZRH,&
     & ZNEB,ZQLI,ZQICE,PGFL,PGMV, &
     & PGMVTNDSI,PGMVTNDHD,PGFLTNDHD,PATND,&
     & ZUZGEO, ZVMGEO, PDHCV, ZENTRA, ZENTRV)

  ELSE

    IF(.NOT.(LMPHYS.OR.LEPHYS)) THEN
      LLPHY=.FALSE.
      CALL GPRH(.FALSE.,NPROMA,KST,KEND,NFLEVG,&
       & 1.0_JPRB,0.0_JPRB,PGFL(1,1,YQ%MP),PGMV(1,1,YT0%MT),PRE0F,ZQSAT,ZRH)  
    ELSE
      IF (LMPA) THEN
        CALL GPRH(.FALSE.,NPROMA,KST,KEND,NFLEVG,&
         & 1.0_JPRB,0.0_JPRB,PGFL(1,1,YQ%MP),PGMV(1,1,YT0%MT),PRE0F,ZQSAT,ZRH)  
      ELSE
        ZRH(KST:KEND,1:NFLEVG)=PRH(KST:KEND,1:NFLEVG)
      ENDIF
      LLPHY=.TRUE.
    ENDIF

    IF (LMPA) THEN
      ZQS(1:NPROMA) =  PGPAR(1:NPROMA,MVQS)
      ZTSOL(1:NPROMA) = PGPAR(1:NPROMA,MVTS)
    ELSE 
      ZQS(1:NPROMA) =  PQS(1:NPROMA)
      ZTSOL(1:NPROMA) = PSP_RR(1:NPROMA,YSP_RR%YT%MP0)
    ENDIF

    ! * COMP. OF DYNAMICAL VARIABLES AND FLUX/TENDENCIES

!CALL POSINSUB('CPG_DIA',IPOS)
    CALL CPDYDDH (NPROMA,NPROMNH,NPROMVC,NPROMDLR,KST,KEND,NFLEVG,&
     & PVCRS0,PVCW0F,PRDLR,&
     & YDGSGEOM,YDCSGEOM,PXYB0,&
     & PRCP0,PHI0,PHIF0,PHI0FL,PHI0FM,&
     & PKENE0, PRT0L, PRT0M, POROGL, POROGM, &
     & PRE0L, PRE0M, PRE0F, PRE0,&
     & PNHPRE0F,PNHPRE0H,PQCHA0L,PQCHA0M, &
     & PCTY0, POMEGA,&
     & ZFPLCL, ZFPLCN, ZFPLSL, ZFPLSN,&
     & ZQS, ZTSOL, ZRH, &
     & PNEB,ZQLI,ZQICE,PGFL,PGMV, &
     & PGMVTNDSI,PGMVTNDHD,PGFLTNDHD,PATND,&
     & ZUZGEO, ZVMGEO, PDHCV, ZENTRA, ZENTRV)
  
!CALL POSINSUB('CPG_DIA',IPOS)
    ! * COMP. OF PHYS. FIELDS AND SURFACE FLUXES
  
    IF((LMPHYS.OR.LEPHYS).AND..NOT.LMPA.AND..NOT.LFLEXDIA) THEN
      CALL CPPHDDH (NPROMA,NPROMVC,NPROMDLR,KST,KEND,NFLEVG,.TRUE.,&
       & LVERCOR,PVCRS0,PRDLR,YDGSGEOM,ZUZGEO,ZVMGEO,&
       & PSP_SG(1,YSP_SG%YF%MP0),PSP_SB(1,1,YSP_SB%YT%MP0),PSP_RR(1,YSP_RR%YT%MP0),&
       & PSP_SB(1,1,YSP_SB%YQ%MP0),PSP_RR(1,YSP_RR%YW%MP0),PGMV(1,1,YT0%MT),PSD_VF(1,YSD_VF%YLSM%MP),&
       & PDIFCQ, PDIFCS, PDIFTQ, PDIFTS, PFCQNG,&
       & PFPLCL, PFPLCN, PFPLSL, PFPLSN, PFRSO, PFRTH,&
       & PFRTHDS,PFHPCL, PFHPCN, PFHPSL, PFHPSN,&
       & PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU, PSTRTV,&
       & PCT,&
       & PFCHSP, PFCLL, PFCLN, PFCS, PFEVL, PFEVN, PFLWSP,&
       & PFONTE, PRUISP, PRUISS,&
       & ZENTRA, ZENTRV,&
       & PDIFTQL,PDIFTQN,PDIFCQL,PDIFCQN,PFCQLNG,PFCQNNG,&
       & PFHSCL,PFHSCN,PFHSSL,PFHSSN,PFEPFP, PFCMPCQ, PFCMPSN, PFCMPSL,&
       & PFCCQL,PFCCQN,PFCSQL,PFCSQN,&
       & PFPFPSL,PFPFPSN,PFPFPCL, PFPFPCN,&
       & PFPEVPSL,PFPEVPSN,PFPEVPCL,PFPEVPCN,&
       & PFCQRNG, PFCQSNG,&
       & PHI0,PUCLS,PVCLS,PTCLS,PQCLS,PCLPH,PALB,PGZ0,PGZ0H,&
       & PDHCV,ZDHCS)
    ENDIF

    IF(LMPHYS.AND.LMPA) THEN

      ! Surface variables and fluxes set to zero.
      ! There are no surface variables for AROME in DDH, yet.
      IF(LLPHY) THEN
        DO JROF = 1, NPROMM
          DO JCV = 1, NDHCSSU
            ZDHCS(JROF,JCV)=0_JPRB
          ENDDO
        ENDDO
      ENDIF

    ENDIF

!CALL POSINSUB('CPG_DIA',IPOS)
  ENDIF
  
  ! * STORE/ACCUM. IN HORIZONTAL MEAN ARRAYS
  
  LLDYN=.TRUE.

  IF (LFLEXDIA) THEN
    CALL CPCUDDH (NPROMA,KST,KEND,KDDHI,PDHSF,RDDH_FIELD,RDDHSURF_FIELD,LLDYN,LLPHY,NTOTFIELD) 
  ELSE 
    CALL CPCUDDH (NPROMA,KST,KEND,KDDHI,PDHSF,PDHCV,ZDHCS,LLDYN,LLPHY,NDHCVSUN) 
  ENDIF 
  
ENDIF

!     ------------------------------------------------------------------

!*       2.    CFU AND XFU:
!              ------------

!*    2.1   DEFINE THE CONDITIONS OF CALLING SOME ROUTINES.

LLCALL_CPCFU= &
 & LCUMFU.AND.(.NOT.LINITER).AND.(CDCONF /= 'X').AND.(.NOT.LAGPHY)
LLCALL_CPXFU=LXFU.AND.LDLDIAB.AND.(.NOT.LINITER)
LLCALL_CUMCPLDM= &
 & (NFRCPL /= 0).AND.(NFRCPL <= NSTOP).AND.(CDCONF /= 'X').AND.(.NOT.LAGPHY).AND.(.NOT.LAROME)
LLCALL_WMCUM=LMOVPH.AND.(CDCONF /= 'X')
! Check that if NPROMM /= NPROMA, all these LLCALL... keys are .F.:
IF ((LLCALL_CPCFU.OR.LLCALL_CPXFU.OR.LLCALL_CUMCPLDM.OR.LLCALL_WMCUM) &
 & .AND.(NPROMM /= NPROMA)) THEN
  WRITE(NULOUT,*) ' CPG_DIA part 2: if NPROMM /= NPROMA,'
  WRITE(NULOUT,*) '  all the LLCALL_... keys must be equal to .F.'
  WRITE(NULOUT,*) '  Check the calculation of these keys.'
  WRITE(NULOUT,*) '  In particular, in adiabatic runs, CFU, XFU'
  WRITE(NULOUT,*) '  and cumul of coupled fields MUST NOT be called.'
  CALL ABOR1(' CPG_DIA: ABOR1 CALLED')
ENDIF

!*    2.2   DIVIDE SOME QUANTITIES LINKED TO (U,V) BY THE MAPPING FACTOR.

! * ZSTR.. arrays are used for CFU, XFU and in CUMCPL.
IF (LLCALL_CPCFU.OR.LLCALL_CPXFU.OR.LLCALL_CUMCPLDM) THEN
  DO JLEV=0,NFLEVG
    ZSTRCU(KST:KEND,JLEV)=PSTRCU(KST:KEND,JLEV)
    ZSTRCV(KST:KEND,JLEV)=PSTRCV(KST:KEND,JLEV)
    ZSTRDU(KST:KEND,JLEV)=PSTRDU(KST:KEND,JLEV)
    ZSTRDV(KST:KEND,JLEV)=PSTRDV(KST:KEND,JLEV)
    ZSTRTU(KST:KEND,JLEV)=PSTRTU(KST:KEND,JLEV)
    ZSTRTV(KST:KEND,JLEV)=PSTRTV(KST:KEND,JLEV)
  ENDDO
ENDIF

! * Z(U,V)CLS and Z(U,V)GST are used for XFU.
IF (LLCALL_CPXFU) THEN
  ZUCLS(KST:KEND)=PUCLS(KST:KEND)
  ZVCLS(KST:KEND)=PVCLS(KST:KEND)
  ZUGST(KST:KEND)=PUGST(KST:KEND)
  ZVGST(KST:KEND)=PVGST(KST:KEND)
ENDIF

! * Divide the arrays by the mapping factor for the cases
!   where the quantities to diagnose are the reduced ones.
!!! -- Test to be checked, seems suspicious -- !!!
IF ((.NOT.LDLADNMI).AND.(LDLDIAB.OR.(LSIMPH.AND.(.NOT.LTRAJPS)))) THEN  
  IF (LLCALL_CPCFU.OR.LLCALL_CPXFU.OR.LLCALL_CUMCPLDM) THEN
    DO JLEV=0,NFLEVG
      ZSTRCU(KST:KEND,JLEV)=ZSTRCU(KST:KEND,JLEV)/YDGSGEOM%GM(KST:KEND)
      ZSTRCV(KST:KEND,JLEV)=ZSTRCV(KST:KEND,JLEV)/YDGSGEOM%GM(KST:KEND)
      ZSTRDU(KST:KEND,JLEV)=ZSTRDU(KST:KEND,JLEV)/YDGSGEOM%GM(KST:KEND)
      ZSTRDV(KST:KEND,JLEV)=ZSTRDV(KST:KEND,JLEV)/YDGSGEOM%GM(KST:KEND)
      ZSTRTU(KST:KEND,JLEV)=ZSTRTU(KST:KEND,JLEV)/YDGSGEOM%GM(KST:KEND)
      ZSTRTV(KST:KEND,JLEV)=ZSTRTV(KST:KEND,JLEV)/YDGSGEOM%GM(KST:KEND)
    ENDDO
  ENDIF
  IF (LLCALL_CPXFU) THEN
    ZUCLS(KST:KEND)=ZUCLS(KST:KEND)/YDGSGEOM%GM(KST:KEND)
    ZVCLS(KST:KEND)=ZVCLS(KST:KEND)/YDGSGEOM%GM(KST:KEND)
    ZUGST(KST:KEND)=ZUGST(KST:KEND)/YDGSGEOM%GM(KST:KEND)
    ZVGST(KST:KEND)=ZVGST(KST:KEND)/YDGSGEOM%GM(KST:KEND)
  ENDIF
ENDIF

!*    2.3   CUMUL OF FLUX DIAGNOSTICS

IF(LLCALL_CPCFU)THEN  
  CALL CPCFU(KST,KEND,KSTC,KENDC,KSTGLO,LDLFSTEP,&
   & PDIFCQ, PDIFCS, PDIFTQ, PDIFTS,&
   & PFCCQL, PFCCQN, PFCSQL, PFCSQN,&
   & PFPLCL, PFPLCN, PFPLCG, PFPLCH, PFPLSL, PFPLSN, PFPLSG, PFPLSH,&
   & PFRMH,PFRSO, PFRSOC, PFRTH, PFRTHC,PFDIS(1,NFLEVG),&
   & ZSTRCU, ZSTRCV, ZSTRDU, ZSTRDV, ZSTRTU, ZSTRTV,&
   & PXYB0(1,1,YYTXYB0%M_DELP),PNEB,PQICE,PQLI,PRE0(1,NFLEVG),PSP_SG(1,YSP_SG%YF%MP0),&
   & PGFL(1,1,YO3%MP),PGFL(1,1,YQ%MP),PSP_RR(1,YSP_RR%YW%MP0),PFEVL,PFEVN,&
   & PCLCT,PCLCH,PCLCM,PCLCL,PCLCC,&
   & PFCLL,PFCLN,PFCS,PFONTE,PFCHSP,PFLWSP,PRUISS,&
   & PRUISP,PFEVV,PFTR,PRUISL,PFGEL,PFGELS,&
   & PFRSODS,PFRSOPS,PFRSOPT,PFRSOLU,PFRTHDS)  
ENDIF

!*    2.4   CUMUL OF COUPLED FIELDS

IF (LLCALL_WMCUM) THEN
  DO JROF=KST,KEND
    FTHSOR(JROF+KSTGLO-1)=PFRTH(JROF,1,1)
  ENDDO
ENDIF

IF(LLCALL_CUMCPLDM) THEN  
  ILEVCO=NFLEVG
  CALL CUMCPL(ILEVCO,KST,KEND,KSTGLO,NTSSG,&
   & ZSTRTU(1,NFLEVG),ZSTRTV(1,NFLEVG),&
   & PFRTH,PFRSO,PFCLL,PFCLN,PFCS,PFEVL,PFEVN,&
   & PFPLCL(1,NFLEVG),PFPLCN(1,NFLEVG),&
   & PFPLSL(1,NFLEVG),PFPLSN(1,NFLEVG),&
   & PRUISS,PRUISP,PSP_RR(1,YSP_RR%YT%MP0),PDRNSHF,PALB,&
   & ZUCLS,ZVCLS)
ENDIF

!*    2.5   INTERFACE FOR INSTANTANEOUS FLUXES

IF (LLCALL_CPXFU) THEN
  CALL CPXFU(KST,KEND,KSTC,KENDC,KSTGLO,LDLRESET,KMEANSTEPS,KXGSTTS,&
   & PDIFCQ, PDIFCS, PDIFTQ, PDIFTS, PFPLCL, PFPLCN, PFPLCG, PFPLCH,&
   & PFPLSL, PFPLSN, PFPLSG, PFPLSH, PFRSO,&
   & PFRTH,  PCLCT,  ZSTRCU, ZSTRCV, ZSTRDU, ZSTRDV, ZSTRTU, ZSTRTV, PNEB,&
   & PQICE,  PQLI,&
   & ZUCLS,  ZVCLS,  PTCLS,  PQCLS,  PRHCLS, PCLCH,  PCLCM,  PCLCL,  PCLCC,&
   & PFCS , PFONTE, PFCHSP, PFLWSP,   PFEVL,PRUISP, PRUISS,  PCAPE, PMOCON,&
   & PCLPH,  PVEIN, ZUGST,  ZVGST,  PFEVV,    PFTR,PRUISL,PDIAGH)
ENDIF

!     ------------------------------------------------------------------

!*       3.    RESETS DDH FLEXIBLE STRUCTURES:
!              -------------------------------

! resets ddh flexible structures if call only to phys

IF( (CDCONF=='X') .AND. LFLEXDIA) CALL RESET_DDHFLEX

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPG_DIA',1,ZHOOK_HANDLE)
END SUBROUTINE CPG_DIA
