SUBROUTINE CPPHDDH( &
 !     INPUT.
 & KPROMA,KPROMVC,KPROMDLR,KSTART,KPROF,KFLEV,LDDYN,&
 & LDVERCOR,PVCRS,PRDLR,YDGSGEOM,PUZGEO,PVMGEO,&
 & PSNS,PTP,PTS,&
 & PWP,PWS,PT0,PLSM,&
 & PDIFCQ, PDIFCS, PDIFTQ, PDIFTS, PFCQNG,&
 & PFPLCL, PFPLCN, PFPLSL, PFPLSN, PFRSO, PFRTH,&
 & PFRTHDS,PFHPCL, PFHPCN, PFHPSL, PFHPSN,&
 & PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU, PSTRTV,&
 & PCT,&
 & PFCHSP, PFCLL, PFCLN, PFCS, PFEVL, PFEVN, PFLWSP,&
 & PFONTE, PRUISP, PRUISS,&
 & PENTRA, PENTRV,&
 & PDIFTQL,PDIFTQN,PDIFCQL,PDIFCQN,PFCQLNG,PFCQNNG,&
 & PFHSCL,PFHSCN,PFHSSL,PFHSSN, PFEPFP, PFCMPCQ, PFCMPSN, PFCMPSL,&
 & PFCQLF,PFCQIF,PFSQLF,PFSQIF,&
 & PFPFPL,PFPFPN,PFPFPCL,PFPFPCN,&
 & PFPEVPL,PFPEVPN,PFPEVPCL,PFPEVPCN,&
 & PFCQRNG, PFCQSNG,&
 & PHI0,PUCLS,PVCLS,PTCLS,PQCLS,PCLPH,PALB,PGZ0,PGZ0H,&
 !     OUTPUT.
 & PDHCV, PDHCS)
!---------------------------------------------------------------------------
!**** *CPPHDDH*  - CALCUL DES FLUX/TENDANCES DYNAMIQUES
!                  DIAGNOSTICS PAR DOMAINES HORIZONTAUX
!---------------------------------------------------------------------------
!     BUT
!     ---
!      CALCULS PHYSIQUES POUR LES DIAGNOSTICS DDH
!      AINSI QUE DES VARIABLES AU SOL A L'INSTANT COURANT

!   ** SURVEILLER LE POSITIONNEMENT DES CHAMPS AU MOYEN DE L'INDICATEUR
!                           IDHCV

!     ARGUMENTS D ENTREE (INPUT)
!     --------------------------

! - DIMENSIONS
!     KPROMA : DIMENSION HORIZONTALE.
!     KPROMVC: horizontal dimension for arrays used only if "lvercor=T".
!     KPROMDLR: horizontal dimension for arrays used only if "lrwsdlr=T".
!     KSTART : DEBUT DE LA BOUCLE HORIZONTALE.
!     KPROF  : FIN DE LA BOUCLE HORIZONTALE.
!     KFLEV  : DIMENSION VERTICALE ET NB DE NIVEAUX

! - LOGIQUES DE CONTROLE
!     LDDYN  : activation des DDH pour la dynamique.
!     LDVERCOR: .T./.F.: hypothese de couche mince abandonnee/appliquee

! - TABLEAUX DE DONNEES
!     PVCRS          : array containing quantities for LVERCOR=T (for ex. rs/a) at t.
!     PRDLR          : array containing "r/a" and "a/r".
!     YDGSGEOM       : structure for geographical sphere horizontal geometry.
!     PUZGEO, PVMGEO : VENT ZONAL ET MERIDIEN VRAIS GEOGRAPHIQUES
!                      (UNIQUEMENT SI LHDHKS OU LHDMCI OU LHDENT VRAIS)
!     PSNS       : MASSE PAR UNITE DE SURFACE DE LA COUCHE NEIGEUSE.
!     PTP        : TEMPERATURE EN PROFONDEUR.
!     PTS        : TEMPERATURE DE SURFACE.
!     PWP        : CONTENU EN EAU DU RESERVOIR EN PROFONDEUR.
!     PWS        : CONTENU EN EAU DU RESERVOIR DE SURFACE.
!     PT0        : TEMPERATURE A INSTANT 0 / TEMPERATURE AT TIME LEVEL 0
!     PHI0       : OROGRAPHIE (HAUTEUR GEOPOTENTIELLE "gz" A L'INSTANT t SUR LES INTERCOUCHES).
!     PLSM       : INDICE TERRE/MER.
!     PUCLS      : COMPOSANTE U DU VENT A 10 METRES (DIAGNOSTIQUE).
!     PVCLS      : COMPOSANTE V DU VENT A 10 METRES (DIAGNOSTIQUE).
!     PTCLS      : TEMPERATURE A 2 METRES (DIAGNOSTIQUE).
!     PQCLS      : HUMIDITE SPECIFIQUE A 2 METRES (DIAGNOSTIQUE).
!     PCLPH      : HAUTEUR (EN METRES) DE LA COUCHE LIMITE PLANETAIRE.
!     PALB       : ALBEDO DE SURFACE COURANT.
!     PGZ0       : G*LONGUEUR DE RUGOSITE DYNAMIQUE.
!     PGZ0H      : G*LONGUEUR DE RUGOSITE THERMIQUE.
!     PDIFCQ     : FLUX CONVECTIF D'HUMIDITE SPECIFIQUE (HORS PLUIE/NEIGE).
!     PDIFCS     : FLUX CONVECTIF D'ENTHALPIE (HORS PLUIE/NEIGE).
!     PDIFTQ     : FLUX TURBULENT (ET Q NEGATIF) D'HUMIDITE SPECIFIQUE.
!     PDIFTS     : FLUX TURBULENT D'ENTHALPIE (OU D'ENERGIE STATIQUE SECHE).
!     PFCQNG     : FLUX FICTIF D'EAU POUR CORRIGER LES Q<0.
!     PFPLCL     : PRECIPITATIONS CONVECTIVES SOUS FORME LIQUIDE.
!     PFPLCN     : PRECIPITATIONS CONVECTIVES SOUS FORME NEIGE.
!     PFPLSL     : PRECIPITATIONS STRATIFORMES SOUS FORME LIQUIDE.
!     PFPLSN     : PRECIPITATIONS STRATIFORMES SOUS FORME NEIGE.
!     PFRSO      : FLUX DE RAYONNEMENT SOLAIRE.
!     PFRTH      : FLUX DE RAYONNEMENT THERMIQUE.
!     PFRTHDS    : FLUX DE RAYONNEMENT THERMIQUE DESCENDANT A LA SURFACE.
!     PFHPCL     : FLUX ENTHALPIE CONVECTIVES SOUS FORME LIQUIDE.
!     PFHPCN     : FLUX ENTHALPIE CONVECTIVES SOUS FORME NEIGE.
!     PFHPSL     : FLUX ENTHALPIE STRATIFORMES SOUS FORME LIQUIDE.
!     PFHPSN     : FLUX ENTHALPIE STRATIFORMES SOUS FORME NEIGE.
!     PSTRCU     : FLUX CONVECTIF DE QUANTITE DE MOUVEMENT "U".
!     PSTRCV     : FLUX CONVECTIF DE QUANTITE DE MOUVEMENT "V".
!     PSTRDU     : FLUX "GRAVITY WAVE DRAG" "U".
!     PSTRDV     : FLUX "GRAVITY WAVE DRAG" "V".
!     PSTRTU     : FLUX TURBULENT DE QUANTITE DE MOUVEMENT "U".
!     PSTRTV     : FLUX TURBULENT DE QUANTITE DE MOUVEMENT "V".
!     PCT        : ???
!     PFCHSP     : FLUX DE CHALEUR DE LA SURFACE VERS LE SOL PROFOND.
!     PFCLL      : FLUX DE CHALEUR LATENTE SUR EAU LIQUIDE (OU SOL HUMIDE).
!     PFCLN      : FLUX DE CHALEUR LATENTE SUR NEIGE (OU GLACE).
!     PFCS       : FLUX DE CHALEUR SENSIBLE EN SURFACE.
!     PFEVL      : FLUX DE VAPEUR D'EAU SUR EAU LIQUIDE (OU SOL HUMIDE).
!     PFEVN      : FLUX DE VAPEUR D'EAU SUR NEIGE (OU GLACE).
!     PFLWSP     : FLUX D'EAU LIQUIDE DE LA SURFACE VERS LE SOL PROFOND.
!     PFONTE     : FLUX D'EAU CORRESPONDANT A LA FONTE DE NEIGE EN SURFACE.
!     PRUISP     : FLUX DE RUISSELLEMENT EN PROFONDEUR.
!     PRUISS     : FLUX DE RUISSELLEMENT EN SURFACE.
!     PENTRA     : ENTROPIE AIR SEC / SPECIFIC ENTROPY OF DRY AIR
!     PENTRV     : ENTROPIE VAPEUR D'EAU / SPECIFIC ENTROPY OF WATER VAPOUR
!     PDIFTQL    : FLUX D'EAU LIQUIDE TURBULENT.
!     PDIFTQN    : FLUX D'EAU NEIGEUSE TURBULENT.
!     PDIFCQL    : FLUX D'EAU LIQUIDE TURBULENT CONVECTIF.
!     PDIFCQN    : FLUX D'EAU NEIGEUSE TURBULENT CONVECTIF.
!     PFCQLNG    : FLUX D'EAU LIQUIDE DE CORR. DES VALEURS < 0.
!     PFCQNNG    : FLUX D'EAU NEIGEUSE DE CORR. DES VALEURS < 0.
!     PFHSCL     : FLUX DE CHALEUR SENSIBLE LIE AUX PRECIPITATIONS CONV. LIQ.
!     PFHSCN     : FLUX DE CHALEUR SENSIBLE LIE AUX PRECIPITATIONS CONV. NEI.
!     PFHSSL     : FLUX DE CHALEUR SENSIBLE LIE AUX PRECIPITATIONS STRA. LIQ.
!     PFHSSN     : FLUX DE CHALEUR SENSIBLE LIE AUX PRECIPITATIONS STRA. NEI.
!     PFEPFP     : PSEUDO FLUX D'ENTHALPIE LIE A L'ENERGIE PRECIPITATIONS.
!     PFCQLF     : FLUX DE CONDENSATION CONV. LIQ.
!     PFCQIF     : FLUX DE CONDENSATION CONV. NEI.
!     PFSQLF     : FLUX DE CONDENSATION STRA. LIQ.
!     PFSQIF     : FLUX DE CONDENSATION STRA. NEI.

!     ENTREES IMPLICITES
!     ------------------
!       UNITE NULOUT (/YOMLUN/)
!       CONSTANTES UNIVERSELLES, COMMON /YOMCST/
!       FONCTIONS THERMODYNAMIQUES, COMDECK *FCTTRM*

!       CONTROLE PHYSIQUE DE BASE : YOMPHY : NDPSFI
!       CONSTANTES PHYSIQUES (NORMALEMENT INUTILES)
!       YOMPHY0, YOMPHY1, YOMPHY2, YOMPHY3

!       ELEMENTS DE CONTROLE DDH
!       INDICATEURS LOGIQUES, COMMON /YOMLDDH/
!        DANS LE COMDECK * YOMMDDH * POUR LES MASQUES
!       DIMENSIONS PROPRES AUX MASQUES DIAGNOSTIQUES DDH, COMMON /DDHDIM/
!       ET DIMENSIONS POUR LES TABLEAUX DE CUMULS
!       INDICATEURS DE POSITIONS DANS LES TABLEAUX DE CUMUL

!     ARGUMENTS DE SORTIES (OUTPUT)
!     -----------------------------
!      DANS PDHCV, FLUX/TENDANCES PHYSIQUES INSTANTANEES
!      DANS PDHCS  VARIABLES ET FLUX AU SOL

!     REMARQUES SUPPLEMENTAIRES
!     -------------------------

!     L'abandon d'hypothese de couche mince conduit a adopter les conventions
!     suivantes pour les flux: ce qu'on stocke dans les tableaux contenant les
!     flux ce sont des quantites extensives (par exemple une masse de vapeur
!     d'eau) par unite de "pseudo-surface Delta x_a * Delta y_a", la pseudo-
!     surface etant egale a la vraie surface sur le niveau de pression ou
!     le pseudo-rayon "rs" vaut exactement le rayon moyen "a". Ce ne sont donc
!     pas les vrais flux (qui eux devraient etre calcules par rapport a la
!     vraie surface "S = Delta x Delta y = (rs/a)**2 Delta x_a * Delta y_a"),
!     mais ces pseudo-flux ont l'avantage de rester des quantites additives
!     (contrairement aux vrais flux), donc en particulier on peut prendre
!     la difference de deux pseudo-flux a deux niveaux differents pour calculer
!     la divergence verticale de flux (et donc la tendance),
!     ou additionner les pseudo-flux pour
!     calculer un flux moyen sur une grande surface. C'est cette convention
!     qui minimise les modifications a apporter dans les DDH (et aussi la
!     physique) vis a vis de l'abandon d'hypothese de couche mince.
!     Par abus de langage on appellera "flux" ces pseudo-flux.

!     Une des consequences de cette convention est que dans cette routine
!     le facteur "rs/a" ne va intervenir qu'a deux endroits:
!     - dans le calcul des flux lies au moment "MT" (car la definition de "MT"
!       fait intervenir "rs").
!     - dans le calcul de moyennes horizontales de champs-sol ou la il faut
!       ajouter la ponderation par "(rs/a)**2".

!     ORIGINAL
!     --------
!     91-03-04, Alain Joly.

!     MODIFICATIONS
!     -------------
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   2004-02-03, F. Vinit : ajout de variables et flux free-style pour Meteo-France.
!   05-06-20, F. Bouyssel: Lopez's prognostic scheme
!   06-01-25, F.Bouyssel: Add PFPEVPL,N Remove PFHMLTS,PFHEVPP,PFPEVPP,PFADVP
!   07-05-31, T.Kovacic: Added PFPEVPCL,PFPEVPCN,PFPFPCL,PFPFPCN,PFCQRNG,
!                                PFCQSNG, PFEPFP, PFCMPCQ, PFCMPSN, PFCMPSL
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   08-12-18 O.Riviere Add new dataflow for DDH under key LFLEXDIA
!   K. Yessad (Dec 2008): remove dummy CDLOCK + cleanings
!   K. Yessad (Jan 2011): introduce INTDYN_MOD structures.
!   K. Yessad (Dec 2011): use YDGSGEOM.
!-------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMRIP   , ONLY : NSSSSS   ,RSTATI
USE YOMPHY   , ONLY : NDPSFI   ,LCONDWT  ,LPROCLD, L3MT, LSTRAPRO
USE YOMCST   , ONLY : ROMEGA   ,RA  ,RG,  RCW, RCS ,RCPD  ,RCPV  ,RLMLT
USE YOMPHY1  , ONLY : RTINER
USE YOEPHY   , ONLY : LEPHYS
USE YOMARPHY  ,ONLY : LMSE
USE YOMLDDH  , ONLY : LHDHKS   ,LHDMCI   ,LHDENT   ,LHDLIST
USE YOMMDDH  , ONLY : NDHCVSU  ,NDHCSSU  ,NDHVV    ,NDHFVD   ,&
 & NDHVS    ,NDHFSD   ,NDHFSP   ,NDHVFS   ,NDHAVP   ,NDHAHKP  ,&
 & NDHBHKP  ,NDHBMCP  
USE YOMSDDH  , ONLY : LHDQLN   
USE YOMDYNA  , ONLY : LRWSDLR
USE YOMGC    , ONLY : TGSGEOM
USE PAR_RDLR , ONLY : JP_RSAH  ,JP_DIMR
USE INTDYN_MOD,ONLY : YYTVC0
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMVC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMDLR
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LDDYN 
LOGICAL           ,INTENT(IN)    :: LDVERCOR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCRS(KPROMVC,0:KFLEV,YYTVC0%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDLR(KPROMDLR,0:KFLEV,JP_DIMR)
TYPE(TGSGEOM)     ,INTENT(IN)    :: YDGSGEOM
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUZGEO(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVMGEO(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQ(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCS(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQ(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTS(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQNG(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRSO(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRTHDS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHPSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCU(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRCV(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDU(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRDV(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTU(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRTV(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCHSP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCLL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCLN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEVN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLWSP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFONTE(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRUISS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PENTRA(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PENTRV(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFCQN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQLNG(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQNNG(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSCL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSCN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSSL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFHSSN(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFEPFP(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPCQ(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPSN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPSL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQLF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQIF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFSQLF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFSQIF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPCL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPFPCN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPCL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPEVPCN(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQRNG(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFCQSNG(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHI0(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLPH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0H(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDHCV(KPROMA,0:KFLEV,NDHCVSU) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDHCS(KPROMA,NDHCSSU) 

!     ------------------------------------------------------------------
REAL(KIND=JPRB) :: ZCLON(KPROMA), ZSLON(KPROMA)
REAL(KIND=JPRB) :: ZFPLL(KPROMA,0:KFLEV) ! ZFPLL : FLUX DE PRECIPITATIONS LIQUIDES
REAL(KIND=JPRB) :: ZFPLN(KPROMA,0:KFLEV) ! ZFPLN : FLUX DE PRECIPITATIONS SOLIDES
!   FLUX DE QTE DE MVT TURBULENTS DANS LE REPERE LOCAL GEOGRAPHIQUE
REAL(KIND=JPRB) :: ZSTTUG(KPROMA,0:KFLEV), ZSTTVG(KPROMA,0:KFLEV)
!   FLUX DE QTE DE MVT CONVECTIFS DANS LE REPERE LOCAL GEOGRAPHIQUE
REAL(KIND=JPRB) :: ZSTCUG(KPROMA,0:KFLEV), ZSTCVG(KPROMA,0:KFLEV)
!   FLUX DE QTE DE MVT ONDES OROG.DANS LE REPERE LOCAL GEOGRAPHIQUE
REAL(KIND=JPRB) :: ZSTDUG(KPROMA,0:KFLEV), ZSTDVG(KPROMA,0:KFLEV)
!   PSEUDO-RAYON "rs" SUR LES INTERCOUCHES:
REAL(KIND=JPRB) :: ZRS(KPROMA,0:KFLEV)
!   INDICE TERRE-MER * (rs/a)**2
REAL(KIND=JPRB) :: ZLSM(KPROMA)

INTEGER(KIND=JPIM) :: IDHCS, IDHCV, IDH, JLEV, JROF

REAL(KIND=JPRB) :: ZDISKC, ZDISKD, ZDISKT, ZDPSFI, ZUSRG, &
 & ZEPS, ZFHSSL, ZFHSSN, ZFHSCL, ZFHSCN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPPHDDH',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!**   1.- INITIALISATIONS DIVERSES

ZUSRG = 1.0_JPRB/RG
ZDPSFI = REAL(NDPSFI,JPRB)
ZEPS = 1.E-4_JPRB
IDHCV=0
IDH=0

DO JLEV = 0, KFLEV
  DO JROF=KSTART,KPROF
    ZFPLL(JROF,JLEV) = PFPLCL(JROF,JLEV) + PFPLSL(JROF,JLEV)
    ZFPLN(JROF,JLEV) = PFPLCN(JROF,JLEV) + PFPLSN(JROF,JLEV)
    ZSTTUG(JROF,JLEV) = YDGSGEOM%GNORDM(JROF)*PSTRTU(JROF,JLEV)-&
     & YDGSGEOM%GNORDL(JROF)*PSTRTV(JROF,JLEV)  
    ZSTTVG(JROF,JLEV) = YDGSGEOM%GNORDL(JROF)*PSTRTU(JROF,JLEV)+&
     & YDGSGEOM%GNORDM(JROF)*PSTRTV(JROF,JLEV)  
    ZSTCUG(JROF,JLEV) = YDGSGEOM%GNORDM(JROF)*PSTRCU(JROF,JLEV)-&
     & YDGSGEOM%GNORDL(JROF)*PSTRCV(JROF,JLEV)  
    ZSTCVG(JROF,JLEV) = YDGSGEOM%GNORDL(JROF)*PSTRCU(JROF,JLEV)+&
     & YDGSGEOM%GNORDM(JROF)*PSTRCV(JROF,JLEV)  
    ZSTDUG(JROF,JLEV) = YDGSGEOM%GNORDM(JROF)*PSTRDU(JROF,JLEV)-&
     & YDGSGEOM%GNORDL(JROF)*PSTRDV(JROF,JLEV)  
    ZSTDVG(JROF,JLEV) = YDGSGEOM%GNORDL(JROF)*PSTRDU(JROF,JLEV)+&
     & YDGSGEOM%GNORDM(JROF)*PSTRDV(JROF,JLEV)  
  ENDDO
ENDDO

IF ( LHDMCI ) THEN
  DO JROF=KSTART,KPROF
    ZCLON(JROF) = COS(YDGSGEOM%GELAM(JROF)+ROMEGA*(RSTATI-NSSSSS))
    ZSLON(JROF) = SIN(YDGSGEOM%GELAM(JROF)+ROMEGA*(RSTATI-NSSSSS))
  ENDDO
ENDIF

!     ------------------------------------------------------------------
!**   2.- CALCUL DES FLUX PHYSIQUES

IF ( LHDHKS ) THEN
  IF(LDDYN) THEN
    IDHCV=NDHVV+NDHFVD
  ELSE
    IDHCV=0
  ENDIF
  IDH=IDHCV
  
  DO JLEV = 0, KFLEV
    DO JROF=KSTART,KPROF
!     FPPSUMFPL
      PDHCV(JROF,JLEV,IDH+1) = ZDPSFI*(ZFPLL(JROF,JLEV)+ZFPLN(JROF,JLEV))
!     FQVTUR
      PDHCV(JROF,JLEV,IDH+2) = PDIFTQ(JROF,JLEV)
!     FQVTURCONV
      PDHCV(JROF,JLEV,IDH+3) = PDIFCQ(JROF,JLEV)
!     FQVPRECISTL
      PDHCV(JROF,JLEV,IDH+4) = PFPLSL(JROF,JLEV)
!     FQVPRECISTN
      PDHCV(JROF,JLEV,IDH+5) = PFPLSN(JROF,JLEV)
!     FQVPRECICOL
      PDHCV(JROF,JLEV,IDH+6) = PFPLCL(JROF,JLEV)
!     FQVPRECICON
      PDHCV(JROF,JLEV,IDH+7) = PFPLCN(JROF,JLEV)
!     FQVTURQNEGAT
      PDHCV(JROF,JLEV,IDH+8) = PFCQNG(JROF,JLEV)
!     FUUTUR
!     FVVTUR
      PDHCV(JROF,JLEV,IDH+9) = ZSTTUG(JROF,JLEV)
      PDHCV(JROF,JLEV,IDH+10) = ZSTTVG(JROF,JLEV)
!     FUUTURCONV
!     FVVTURCONV
      PDHCV(JROF,JLEV,IDH+11) = ZSTCUG(JROF,JLEV)
      PDHCV(JROF,JLEV,IDH+12) = ZSTCVG(JROF,JLEV)
!     FUUONDEGREL
!     FVVONDEGREL
      PDHCV(JROF,JLEV,IDH+13) = ZSTDUG(JROF,JLEV)
      PDHCV(JROF,JLEV,IDH+14) = ZSTDVG(JROF,JLEV)
!     FCTTUR
      PDHCV(JROF,JLEV,IDH+15) = PDIFTS(JROF,JLEV)
!     FCTTURCONV
      PDHCV(JROF,JLEV,IDH+16) = PDIFCS(JROF,JLEV)
!     FCTRAYSOL1
      PDHCV(JROF,JLEV,IDH+17) = PFRSO(JROF,JLEV)
!     FCTRAYTER1
      PDHCV(JROF,JLEV,IDH+18) = PFRTH(JROF,JLEV)
      
      IF  ( L3MT.OR.LSTRAPRO )  THEN
!       This is ALARO!      
        IF (NDPSFI==1) THEN
          ZFHSSL = (RCPD/RCW)*PFHSSL(JROF,JLEV)
          ZFHSSN = (RCPD/RCS)*PFHSSN(JROF,JLEV)
          ZFHSCL = (RCPD/RCW)*PFHSCL(JROF,JLEV)
          ZFHSCN = (RCPD/RCS)*PFHSCN(JROF,JLEV)
!         FCTPRECISTL
          PDHCV(JROF,JLEV,IDH+19) = PFHSSL(JROF,JLEV)-ZFHSSL
!         FCTPRECISTN
          PDHCV(JROF,JLEV,IDH+20) = PFHSSN(JROF,JLEV)-ZFHSSN
!         FCTPRECICOL
          PDHCV(JROF,JLEV,IDH+21) = PFHSCL(JROF,JLEV)-ZFHSCL
!         FCTPRECICON
          PDHCV(JROF,JLEV,IDH+22) = PFHSCN(JROF,JLEV)-ZFHSCN
!         FCTCMP 
          IDH=IDH+1
          PDHCV(JROF,JLEV,IDH+22)=ZFHSSL+ZFHSSN+ZFHSCL+ZFHSCN+PFEPFP(JROF,JLEV)
!         FQVCMPC
          IDH=IDH+1
          PDHCV(JROF,JLEV,IDH+22) = PFCMPCQ(JROF,JLEV)
        ELSE
!         FCTPRECISTL
          PDHCV(JROF,JLEV,IDH+19) = PFHSSL(JROF,JLEV)
!         FCTPRECISTN
          PDHCV(JROF,JLEV,IDH+20) = PFHSSN(JROF,JLEV)
!         FCTPRECICOL
          PDHCV(JROF,JLEV,IDH+21) = PFHSCL(JROF,JLEV)
!         FCTPRECICON
          PDHCV(JROF,JLEV,IDH+22) = PFHSCN(JROF,JLEV)
        ENDIF
!       FCTPRECCSSTL
        PDHCV(JROF,JLEV,IDH+23) = PFHPSL(JROF,JLEV)
!       FCTPRECCSSTN
        PDHCV(JROF,JLEV,IDH+24) = PFHPSN(JROF,JLEV)
!       FCTPRECCSCOL
        PDHCV(JROF,JLEV,IDH+25) = PFHPCL(JROF,JLEV)
!       FCTPRECCSCON
        PDHCV(JROF,JLEV,IDH+26) = PFHPCN(JROF,JLEV)
      ELSE
!       FCTPRECISTL
        PDHCV(JROF,JLEV,IDH+19) = PFHPSL(JROF,JLEV)
!       FCTPRECISTN
        PDHCV(JROF,JLEV,IDH+20) = PFHPSN(JROF,JLEV)
!       FCTPRECICOL
        PDHCV(JROF,JLEV,IDH+21) = PFHPCL(JROF,JLEV)
!       FCTPRECICON
        PDHCV(JROF,JLEV,IDH+22) = PFHPCN(JROF,JLEV)
!       FCTPRECCSCOL
        PDHCV(JROF,JLEV,IDH+23) = PFHSCL(JROF,JLEV)
!       FCTPRECCSCON
        PDHCV(JROF,JLEV,IDH+24) = PFHSCN(JROF,JLEV)
!       FCTPRECCSSTL
        PDHCV(JROF,JLEV,IDH+25) = PFHSSL(JROF,JLEV)
!       FCTPRECCSSTN
        PDHCV(JROF,JLEV,IDH+26) = PFHSSN(JROF,JLEV)
      ENDIF
    ENDDO  
  ENDDO
  IF(LHDQLN) THEN
    DO JLEV = 0, KFLEV
      DO JROF=KSTART,KPROF
!       FQLTUR
        PDHCV(JROF,JLEV,IDH+27) = PDIFTQL(JROF,JLEV)
!       FQNTUR
        PDHCV(JROF,JLEV,IDH+28) = PDIFTQN(JROF,JLEV)
!       FQLTURCONV
        PDHCV(JROF,JLEV,IDH+29) = PDIFCQL(JROF,JLEV)
!       FQNTURCONV
        PDHCV(JROF,JLEV,IDH+30) = PDIFCQN(JROF,JLEV)
!       FQLTURQNEGAT
        PDHCV(JROF,JLEV,IDH+31) = PFCQLNG(JROF,JLEV)
!       FQNTURQNEGAT
        PDHCV(JROF,JLEV,IDH+32) = PFCQNNG(JROF,JLEV)
!       FQTCONDECOL
        PDHCV(JROF,JLEV,IDH+33) = PFCQLF(JROF,JLEV)
!       FQTCONDECON
        PDHCV(JROF,JLEV,IDH+34) = PFCQIF(JROF,JLEV)
!       FQTCONDESTL
        PDHCV(JROF,JLEV,IDH+35) = PFSQLF(JROF,JLEV)
!       FQTCONDESTN
        PDHCV(JROF,JLEV,IDH+36) = PFSQIF(JROF,JLEV)

        IF ( (LCONDWT.AND.LPROCLD).OR. & 
          & ( L3MT.OR.LSTRAPRO ) ) THEN
!         FPEVPL
          PDHCV(JROF,JLEV,IDH+37) = PFPEVPL(JROF,JLEV)
!         FPEVPN
          PDHCV(JROF,JLEV,IDH+38) = PFPEVPN(JROF,JLEV)
!         FPFPL
          PDHCV(JROF,JLEV,IDH+39) = PFPFPL(JROF,JLEV)
!         FPFPN
          PDHCV(JROF,JLEV,IDH+40) = PFPFPN(JROF,JLEV)
          IF ( L3MT.OR.LSTRAPRO ) THEN
!           ... , FQRPEVC
            PDHCV(JROF,JLEV,IDH+41) = PFPEVPCL (JROF,JLEV)
!           ... , FQSPEVC
            PDHCV(JROF,JLEV,IDH+42) = PFPEVPCN (JROF,JLEV)
!           ... , FQLPFPC
            PDHCV(JROF,JLEV,IDH+43) = PFPFPCL (JROF,JLEV)
!           ... , FQNPFPC
            PDHCV(JROF,JLEV,IDH+44) = PFPFPCN (JROF,JLEV)
!           ... , FQRNG
            PDHCV(JROF,JLEV,IDH+45) = PFCQRNG (JROF,JLEV)
!           ... , FQSNG
            PDHCV(JROF,JLEV,IDH+46) = PFCQSNG (JROF,JLEV)
   
            IF (NDPSFI==1) THEN
!             FQLCMP
              IDH=IDH+1
              PDHCV(JROF,JLEV,IDH+46) = PFCMPSL(JROF,JLEV)
!             FQNCMP
              IDH=IDH+1
              PDHCV(JROF,JLEV,IDH+46) = PFCMPSN(JROF,JLEV)
            ENDIF
          ENDIF
        ENDIF
      ENDDO
    ENDDO
  ENDIF
ENDIF

IF ( LHDMCI ) THEN
  IF (LDVERCOR) THEN
    ZRS(KSTART:KPROF,0:KFLEV)=RA*PVCRS(KSTART:KPROF,0:KFLEV,YYTVC0%M_VCRSSAH)
  ELSEIF (LRWSDLR) THEN
    ZRS(KSTART:KPROF,0:KFLEV)=RA*PRDLR(KSTART:KPROF,0:KFLEV,JP_RSAH)
  ELSE
    ZRS(KSTART:KPROF,0:KFLEV)=RA
  ENDIF
  IDHCV=NDHVV+NDHFVD+NDHAHKP
  DO JLEV = 0, KFLEV
    DO JROF=KSTART,KPROF
!     FA1TUR
!     FA2TUR
!     FA3TUR
      PDHCV(JROF,JLEV,IDHCV+1) = ZRS(JROF,JLEV)*( ZSTTVG(JROF,JLEV)*&
       & ZSLON(JROF)-&
       & ZSTTUG(JROF,JLEV)*ZCLON(JROF)*YDGSGEOM%GEMU(JROF))  
      PDHCV(JROF,JLEV,IDHCV+2) = ZRS(JROF,JLEV)*(-ZSTTVG(JROF,JLEV)*&
       & ZCLON(JROF)-&
       & ZSTTUG(JROF,JLEV)*ZSLON(JROF)*YDGSGEOM%GEMU(JROF))  
      PDHCV(JROF,JLEV,IDHCV+3) = ZRS(JROF,JLEV)*ZSTTUG(JROF,JLEV)*YDGSGEOM%GSQM2(JROF)
!     FA1TURCONV
!     FA2TURCONV
!     FA3TURCONV
      PDHCV(JROF,JLEV,IDHCV+4) = ZRS(JROF,JLEV)*( ZSTCVG(JROF,JLEV)*&
       & ZSLON(JROF)-&
       & ZSTCUG(JROF,JLEV)*ZCLON(JROF)*YDGSGEOM%GEMU(JROF))  
      PDHCV(JROF,JLEV,IDHCV+5) = ZRS(JROF,JLEV)*(-ZSTCVG(JROF,JLEV)*&
       & ZCLON(JROF)-&
       & ZSTCUG(JROF,JLEV)*ZSLON(JROF)*YDGSGEOM%GEMU(JROF))  
      PDHCV(JROF,JLEV,IDHCV+6) = ZRS(JROF,JLEV)*ZSTCUG(JROF,JLEV)*YDGSGEOM%GSQM2(JROF)
!     FA1ONDEGREL
!     FA2ONDEGREL
!     FA3ONDEGREL
      PDHCV(JROF,JLEV,IDHCV+7) = ZRS(JROF,JLEV)*( ZSTDVG(JROF,JLEV)*&
       & ZSLON(JROF)-&
       & ZSTDUG(JROF,JLEV)*ZCLON(JROF)*YDGSGEOM%GEMU(JROF))  
      PDHCV(JROF,JLEV,IDHCV+8) = ZRS(JROF,JLEV)*(-ZSTDVG(JROF,JLEV)*&
       & ZCLON(JROF)-&
       & ZSTDUG(JROF,JLEV)*ZSLON(JROF)*YDGSGEOM%GEMU(JROF))  
      PDHCV(JROF,JLEV,IDHCV+9) = ZRS(JROF,JLEV)*ZSTDUG(JROF,JLEV)*YDGSGEOM%GSQM2(JROF)
    ENDDO
  ENDDO

ENDIF
!     ------------------------------------------------------------------
!**   3.- CALCUL DES TENDANCES PHYSIQUES

!*         - DISSIPATION D'ENERGIE CINETIQUE.

IF ( LHDHKS ) THEN
  IF(LDDYN) THEN
    IDHCV=NDHVV+NDHFVD+NDHAVP
  ELSE
    IDHCV=NDHAVP
  ENDIF
  DO JLEV = 1, KFLEV
    DO JROF=KSTART,KPROF
!     TKKDISSIPTUR
      ZDISKT = -( PUZGEO(JROF,JLEV)*&
       & (ZSTTUG(JROF,JLEV)-ZSTTUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTTVG(JROF,JLEV)-ZSTTVG(JROF,JLEV-1)) )  
      PDHCV(JROF,JLEV,IDHCV+1) = ZDISKT
!     TKKDISSIPCONV
      ZDISKC = -( PUZGEO(JROF,JLEV)*&
       & (ZSTCUG(JROF,JLEV)-ZSTCUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTCVG(JROF,JLEV)-ZSTCVG(JROF,JLEV-1)) )  
      PDHCV(JROF,JLEV,IDHCV+2) = ZDISKC
!     TKKDISSIPGREL
      ZDISKD = -( PUZGEO(JROF,JLEV)*&
       & (ZSTDUG(JROF,JLEV)-ZSTDUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTDVG(JROF,JLEV)-ZSTDVG(JROF,JLEV-1)) )  
      PDHCV(JROF,JLEV,IDHCV+3) = ZDISKD
    ENDDO
  ENDDO
ENDIF

IF ( LHDENT ) THEN
  IDHCV=NDHVV+NDHFVD+NDHAVP+NDHBHKP+NDHBMCP
  DO JLEV = 1, KFLEV
    DO JROF=KSTART,KPROF
!     TSSDISSIPTUR
      PDHCV(JROF,JLEV,IDHCV+1) = ( PUZGEO(JROF,JLEV)*&
       & (ZSTTUG(JROF,JLEV)-ZSTTUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTTVG(JROF,JLEV)-ZSTTVG(JROF,JLEV-1)) ) / PT0(JROF,JLEV)  
!     TSSDISSIPCONV
      PDHCV(JROF,JLEV,IDHCV+2) = ( PUZGEO(JROF,JLEV)*&
       & (ZSTCUG(JROF,JLEV)-ZSTCUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTCVG(JROF,JLEV)-ZSTCVG(JROF,JLEV-1)) ) / PT0(JROF,JLEV)  
!     TSSDISSIPGREL
      PDHCV(JROF,JLEV,IDHCV+3) = ( PUZGEO(JROF,JLEV)*&
       & (ZSTDUG(JROF,JLEV)-ZSTDUG(JROF,JLEV-1)) + PVMGEO(JROF,JLEV)&
       & *(ZSTDVG(JROF,JLEV)-ZSTDVG(JROF,JLEV-1)) ) / PT0(JROF,JLEV)  
!     TSSTURQVTOT
      PDHCV(JROF,JLEV,IDHCV+4) =&
       & (PENTRA(JROF,JLEV)-PENTRV(JROF,JLEV) + RCPV-RCPD)*&
       & ( PDIFTQ(JROF,JLEV)-PDIFTQ(JROF,JLEV-1) +&
       & PDIFCQ(JROF,JLEV)-PDIFCQ(JROF,JLEV-1) )  
!     TSSRAY1
      PDHCV(JROF,JLEV,IDHCV+5) =&
       & - ( PFRSO(JROF,JLEV)-PFRSO(JROF,JLEV-1) +&
       & PFRTH(JROF,JLEV)-PFRTH(JROF,JLEV-1) ) / PT0(JROF,JLEV)  
!     TSSTURCTTOT
      PDHCV(JROF,JLEV,IDHCV+6) =&
       & - ( PDIFTS(JROF,JLEV)-PDIFTS(JROF,JLEV-1) +&
       & PDIFCS(JROF,JLEV)-PDIFCS(JROF,JLEV-1) ) / PT0(JROF,JLEV)  
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------
!**   4.- CHAMPS AU SOL

IDHCS=0

IF (LDVERCOR) THEN
  ZLSM(KSTART:KPROF)=PLSM(KSTART:KPROF) &
   & *PVCRS(KSTART:KPROF,KFLEV,YYTVC0%M_VCRSSAH)*PVCRS(KSTART:KPROF,KFLEV,YYTVC0%M_VCRSSAH)  
ELSEIF (LRWSDLR) THEN
  ZLSM(KSTART:KPROF)=PLSM(KSTART:KPROF) &
   & *PRDLR(KSTART:KPROF,KFLEV,JP_RSAH)*PRDLR(KSTART:KPROF,KFLEV,JP_RSAH)  
ELSE
  ZLSM(KSTART:KPROF)=PLSM(KSTART:KPROF)
ENDIF

!     VARIABLES
!     ---------
IF ( LHDHKS ) THEN
  DO JROF=KSTART,KPROF
    PDHCS(JROF,IDHCS+1) = ZLSM(JROF)
    IF(LEPHYS.OR.LMSE) THEN
      PDHCS(JROF,IDHCS+2) = ZLSM(JROF)*PTS(JROF)
      PDHCS(JROF,IDHCS+3) = ZLSM(JROF)*PTP(JROF)
    ELSE
      PDHCS(JROF,IDHCS+2) = ZLSM(JROF)/PCT(JROF)*PTS(JROF)
      PDHCS(JROF,IDHCS+3) = ZLSM(JROF)/PCT(JROF)*RTINER*PTP(JROF)
    ENDIF
    PDHCS(JROF,IDHCS+4) = ZLSM(JROF)*PWS(JROF)
    PDHCS(JROF,IDHCS+5) = ZLSM(JROF)*PWP(JROF)
    PDHCS(JROF,IDHCS+6) = ZLSM(JROF)*PSNS(JROF)
  ENDDO

  IDHCS = IDHCS + 6

ENDIF

!    FREE_STYLE VARIABLES
!    --------------------

IDHCS = NDHVS
IF ( LHDHKS ) THEN
  DO JROF=KSTART,KPROF
    PDHCS(JROF,IDHCS+1) = PTCLS(JROF)
    PDHCS(JROF,IDHCS+2) = PQCLS(JROF)
    PDHCS(JROF,IDHCS+3) = PUCLS(JROF)
    PDHCS(JROF,IDHCS+4) = PVCLS(JROF)
    PDHCS(JROF,IDHCS+5) = PHI0(JROF,KFLEV)
    PDHCS(JROF,IDHCS+6) = PGZ0(JROF)
    PDHCS(JROF,IDHCS+7) = PGZ0H(JROF)
    PDHCS(JROF,IDHCS+8) = PALB(JROF)
    PDHCS(JROF,IDHCS+9) = PCLPH(JROF)
  ENDDO
ENDIF

!     FLUX EN SURFACE
!     ---------------

IDHCS = NDHVS + NDHVFS + NDHFSD
IF ( LHDHKS ) THEN
  DO JROF=KSTART,KPROF
    PDHCS(JROF,IDHCS+1) = ZLSM(JROF)*PFRSO(JROF,KFLEV)
    PDHCS(JROF,IDHCS+2) = ZLSM(JROF)*PFRTH(JROF,KFLEV)
    PDHCS(JROF,IDHCS+3) = ZLSM(JROF)*PFCLL (JROF)
    PDHCS(JROF,IDHCS+4) = ZLSM(JROF)*PFCLN (JROF)
    PDHCS(JROF,IDHCS+5) = ZLSM(JROF)*PFCS  (JROF)
    PDHCS(JROF,IDHCS+6) = ZLSM(JROF)*PFCHSP(JROF)
    PDHCS(JROF,IDHCS+7) = -ZLSM(JROF)*PFONTE(JROF)*RLMLT
    PDHCS(JROF,IDHCS+8) = ZLSM(JROF)*PFPLSL(JROF,KFLEV)
    PDHCS(JROF,IDHCS+9) = ZLSM(JROF)*PFPLSN(JROF,KFLEV)
    PDHCS(JROF,IDHCS+10) = ZLSM(JROF)*PFPLCL(JROF,KFLEV)
    PDHCS(JROF,IDHCS+11) = ZLSM(JROF)*PFPLCN(JROF,KFLEV)
    PDHCS(JROF,IDHCS+12) = ZLSM(JROF)*PFEVL (JROF)
    PDHCS(JROF,IDHCS+13) = ZLSM(JROF)*PFEVN (JROF)
    PDHCS(JROF,IDHCS+14) = ZLSM(JROF)*PFLWSP(JROF)
    PDHCS(JROF,IDHCS+15) = ZLSM(JROF)*PFONTE(JROF)
    PDHCS(JROF,IDHCS+16) = ZLSM(JROF)*PRUISS(JROF)
    PDHCS(JROF,IDHCS+17) = ZLSM(JROF)*PRUISP(JROF)
  ENDDO

  IDHCS = IDHCS + 17

ENDIF

!       FREE_STYLE FLUXES
!       -----------------

IDHCS = NDHVS + NDHVFS + NDHFSD + NDHFSP
IF ( LHDHKS ) THEN
  DO JROF=KSTART,KPROF
    PDHCS(JROF,IDHCS+1) = PFRSO(JROF,KFLEV)/MAX(1.0_JPRB-PALB(JROF),ZEPS)
    PDHCS(JROF,IDHCS+2) = PFRTHDS(JROF)
  ENDDO

  IDHCS = IDHCS + 2

ENDIF

!     ------------------------------------------------------------------

IF ( LHDLIST ) THEN
  WRITE (NULOUT,*) ' DDH * CPPH IDHCV=',IDHCV,' IDHCS = ',IDHCS
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPPHDDH',1,ZHOOK_HANDLE)
END SUBROUTINE CPPHDDH
