!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACMIXELEN ( KIDIA,KFDIA,KLON,KTDIA,KLEV,KSTEP,KMLTYPE,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPHI,PAPHIF,PAPRS,PAPRSF,PT,PQV,PQLI,PQICE,PQRAIN,PQSNOW,PTKE,&
 & PR,PU,PV,PALPH,PLNPR,&
 & PMN2PP,PRRCOR, &
 ! - INPUT 1D .
 & PGZ0,PTS,&
 ! - INPUT/OUTPUT 2D .
 & PLMU,PLMT,PLMLF,&
 ! - OUTPUT
 & PLML,PLMLTILD,&
 ! LOGICAL
 & LDMAF)

!**** *ACMIXELEN * - COMPUTATION OF MIXING LENGTHS FOR TKE SCHEMES.

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.
! KTDIA     : INDICE DE DEPART DES BOUCLES VERTICALES (1 EN GENERAL)
!              POUR LES CALCULS DE TURBULENCE.
! KTDIA     : START OF THE VERTICAL LOOP FOR THE TURBUL COMPUTATIONS.
! KLEV       : DIMENSION VERTICALE DES TABLEAUX "FULL LEVEL".
! KSTEP      : NUMERO DU PAS DE TEMPS.
! KMLTYPE   : TYPE OF MIXING LENGTH
!            = 1  pure BL89 
!            = 2  BL89, in convective case mixed with GC
!            = 3  stable case: const * sqrt(e)/N 
!                 unstable case: GC
!            = 4  stable case: const * sqrt(e)/N mixed with GC
!                 unstable case: GC
!            = 5  stable case: BL89 mixed with const * sqrt(e)/N
!                 unstable case: BL89


! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PAPHI      : GEOPOTENTIEL AUX DEMI-NIVEAUX.
! PAPRS      : PRESSION AUX DEMI-NIVEAUX.

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIEL AUX NIVEAUX DES COUCHES.
! PAPRSF     : PRESSION AUX NIVEAUX DES COUCHES.
! PT         : TEMPERATURE.
! PQV        : PROGNOSTIC VATER VAPOUR.
! PQLI       : PROGNOSTIC SUSPENDED CLOUD WATER.
! PQICE      : PROGNOSTIC SUSPENDED CLOUD ICE.
! PQRAIN     : PROGNOSTIC LIQUID PRECIPITATIONS.
! PQSNOW     : PROGNOSTIC FROZEN PRECIPITATIONS.
! PTKE       : PROGNOSTIC FULL LEVEL TKE.
! PR         : CONSTANTE DES GAZ POUR L'AIR.
! PU         : COMPOSANTE EN X DU VENT.
! PV         : COMPOSANTE EN Y DU VENT.
! PALPH      : LOG(PAPRS(JLEV)/PAPRSF(JLEV)) (FOR HYDROSTATICS).
! PLNPR      : LOG(PAPRS(JLEV)/PAPRS(JLEV-1)) (FOR HYDROSTATICS).


! - 1D (PROGNOSTIQUE) .

! PGZ0       : G FOIS LA LONGUEUR DE RUGOSITE COURANTE.
! PTS        : TEMPERATURE DE SURFACE

!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE/SORTIE.
!     --------------------------

! - 2D (0:KLEV) .
! PLMU        : LONGUEUR DE MELANGE POUR LE VENT.
! PLMT        : LONGUEUR DE MELANGE POUR T ET Q.

! PLML        : TKE type mixing length (on half levels)
! PLMLTILD    : 'STATIONARY' TKE type mixing length (on half levels) 

! - 2D (1:KLEV) .
! PLMLF      : PROGNOSTIC MIX. LENGTH ON FULL LEVELS

!LOGICAL
!LDMAF       : MIX. LENGTH FOR MOIST AF
!-----------------------------------------------------------------------

! -   ARGUMENTS IMPLICITES.
!     ---------------------

! COMMON/YOMPHY0/
! COMMON/YOMCST /

!-----------------------------------------------------------------------

!     Externals.
!     ---------

!     Method.
!     --------
!     Bougeault and Lacarrere (1989), Deadorff (1974),
!     Cheng, Canuto and Howard (2002), ...
!     and the implementation note:
!     F. Vana: Turbulent mixing length closure for the ALARO physics.

!     Author.
!     -------
!        F. Vana (LACE/CHMI) 

!     Modifications.
!     --------------
!        Original version 20-Sep-2007
!        F. Vana 26-Mar-2008: consistency with eTKE + more options
!        F. Vana 21-Jan-2009: phased with CY35T1
!        F. Vana 17-Jun-2009: extended by X-term computation
!        F. Vana 26-Jun-2009: reworked EL2, EL3, EL4, EL5
!        F. Vana 20-Aug-2009: L-type mixing length as direct output
!                             plus special 1st timestep treatment
!        I. Bastak Duran 12-Oct-2009: updated modules for eTKE
!        F. Vana 03-Jun-2010: All computations rewritten to 'moist'
!                               Brunt-Vaissala freq.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YOMPHY   , ONLY : LRRGUST, LCOEFKTKE,LCOEFK_PL,LCOEFK_ML
USE YOMCST   , ONLY : RG       ,RATM     ,RKAPPA  ,RETV
USE YOMPHY0  , ONLY : VKARMN   ,NUPTKE   ,TKEMULT, &
 &  C_EPSILON, ALMAV, GCISMIN,C3TKEFREE
USE YOM_YGFL , ONLY : YTKE_NL

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMLTYPE
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQV(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLI(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQICE(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQRAIN(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQSNOW(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTKE(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PLMLF(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMN2PP   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRRCOR   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLMU(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLMT(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLML(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLMLTILD(KLON,0:KLEV) 
LOGICAL   :: LDMAF 

INTEGER(KIND=JPIM) :: JLEV, JLON, JJLEV

REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZINTE(KLON),ZLWORK(KLON)

REAL(KIND=JPRB) :: ZQV, ZQC, ZTESTM, ZTEST0, ZTEST, ZKONV, &
 &                 ZPOWERED2, ZRKONV, ZKONVH
REAL(KIND=JPRB) :: ZDPHI, ZCIS, ZDTETA, ZRTI, ZSTA, ZGLU, ZZ
REAL(KIND=JPRB) :: ZQBLACK
REAL(KIND=JPRB) :: ZEPS, ZLMAX,ZEPSLML


REAL(KIND=JPRB) :: zlwork1_, zlwork2_, zpote_

REAL(KIND=JPRB) :: ZDELTVPT(KLON)
REAL(KIND=JPRB) :: ZHLVPT(KLON)
REAL(KIND=JPRB) :: ZVPT(KLON)
REAL(KIND=JPRB) :: ZLM(KLON,KLEV)
REAL(KIND=JPRB) :: ZMOIST(KLON,KLEV) 
REAL(KIND=JPRB) :: ZC3(KLON,0:KLEV) 
REAL(KIND=JPRB) :: ZLMD(KLON,0:KLEV) 
REAL(KIND=JPRB) :: ZLMU(KLON,0:KLEV) 
REAL(KIND=JPRB) :: ZDZZ (KLON,0:KLEV)
REAL(KIND=JPRB) :: ZDZZ2(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZBVMO(KLON,0:KLEV) 
    
! Generalized Blackadar formula
REAL(KIND=JPRB) :: BLACK
REAL(KIND=JPRB) :: ZMXL1, ZMXL2
REAL(KIND=JPRB) :: ZQEXP
BLACK(ZMXL1, ZMXL2, ZQEXP)=ZMXL2 &
  & /((1._JPRB + (ZMXL2/ZMXL1)**ZQEXP)**(1._JPRB/ZQEXP))


!---------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('ACMIXELEN',0,ZHOOK_HANDLE)

!---------------------------------------------------------------------

! 0. Initial settings

!   security constant
ZEPS    = EPSILON(1.0_JPRB)*100.0_JPRB
ZEPSLML = 1.E-8_JPRB

! 0.1 Setup of constants

!   conversion factor L -> l
ZKONV = NUPTKE*NUPTKE*NUPTKE/(C_EPSILON)
ZKONVH=ZKONV*0.5_JPRB
!   reverse value of ZKONV
ZRKONV= C_EPSILON/(NUPTKE*NUPTKE*NUPTKE)
!   precomputed 2^(5/4)
ZPOWERED2=2.378414230005442_JPRB

! Limit of mixing length for stable stratification.
! (here mostly to cure numerical problems of Deardorff's mixing length)
ZLMAX=1500._JPRB * ZKONV

! Coefficient for the generalized Blackadar formula
ZQBLACK=2._JPRB

ZGLU=RG*ALMAV

! 0.2 Setup of fields

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
    ! calculation of ZDZZ
    ZDZZ(JLON,JLEV)=(PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1))/RG
  ENDDO
ENDDO
ZDZZ(KIDIA:KFDIA,KLEV)=(PAPHIF(KIDIA:KFDIA,KLEV)-PAPHI(KIDIA:KFDIA,KLEV))/RG

! calculation of ZDZZ**2
ZDZZ2(KIDIA:KFDIA,KTDIA:KLEV)= ZDZZ(KIDIA:KFDIA,KTDIA:KLEV)**2

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ! calculation of moist contribution to Theta
    ZQV=PQV(JLON,JLEV)
    ZQC=PQLI(JLON,JLEV)+PQICE(JLON,JLEV)+PQRAIN(JLON,JLEV)+PQSNOW(JLON,JLEV)
    !ZMOIST(JLON,JLEV)=1._JPRB
    ZMOIST(JLON,JLEV)=(1._JPRB+RETV*ZQV-ZQC)
  ENDDO
ENDDO

!    Richardson number and Brunt Vaissala (moist) frequencey
DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
    ! Shear computation
    ZZ=VKARMN*(0.5_JPRB*(PAPHIF(JLON,JLEV)+PAPHIF(JLON,JLEV+1)) &
           & -PAPHI(JLON,KLEV)+PGZ0(JLON))
    ZDPHI=ZDZZ(JLON,JLEV)*RG
    !  CISAILLEMENT DE VENT./ WIND SHEAR. / STRIH VETRU
    ZCIS=(PU(JLON,JLEV)-PU(JLON,JLEV+1))**2 &
     & + (PV(JLON,JLEV)-PV(JLON,JLEV+1))**2 &
     & +(GCISMIN*ZDZZ(JLON,JLEV)/(1.0_JPRB+ZZ/ZGLU))**2
    IF(.NOT.LCOEFK_ML) THEN !'Dry' case
      ! COMPUTATION OF 'DRY' Richardson number (Ri_d) and N^2
      ! Here at least the moist is threated through the temperature,
      ! so it is not really the dry case either (unless the ZMOIST=1).
      !  PRECALCUL DE STABILITE. / PRELIMINARY STABILITY COMPUTATION.
      ZDTETA=PR(JLON,JLEV)*PT(JLON,JLEV)*ZMOIST(JLON,JLEV) &
       & -PR(JLON,JLEV+1)*PT(JLON,JLEV+1)*ZMOIST(JLON,JLEV+1)+RKAPPA*ZDPHI
      !  CALCUL DE STABILITE. / STABILITY COMPUTATION.
      ZRTI=2.0_JPRB/(PR(JLON,JLEV)*PT(JLON,JLEV)*ZMOIST(JLON,JLEV) &
       &            +PR(JLON,JLEV+1)*PT(JLON,JLEV+1)*ZMOIST(JLON,JLEV+1))
      !  FOR RICHARDSON NUMBER
      ZSTA=ZDPHI*(ZDTETA)*ZRTI
      ZBVMO(JLON,JLEV)=ZSTA/(ZDPHI/RG)**2
      ZBVMO(JLON,JLEV)=SIGN(MAX(ZEPS,ABS(ZBVMO(JLON,JLEV))),ZBVMO(JLON,JLEV))
    ELSE ! Ri'' 
      ! COMPUTATION OF  Richardson number with moist AF scheme - Ri''
      ZBVMO(JLON,JLEV)=PMN2PP(JLON,JLEV)
      ZBVMO(JLON,JLEV)=SIGN(MAX(ZEPS,ABS(ZBVMO(JLON,JLEV))),ZBVMO(JLON,JLEV))
    ENDIF
  ENDDO
ENDDO

! Lower boundary (only required by BL89 computation)
IF ((KMLTYPE == 1).or.(KMLTYPE == 2).or.(KMLTYPE == 5)) THEN
  ZVPT(KIDIA:KFDIA)= PT(KIDIA:KFDIA,KLEV)* &
    &  (RATM/PAPRSF(KIDIA:KFDIA,KLEV))**(RKAPPA) * ZMOIST(KIDIA:KFDIA,KLEV)
  ! For surface the moist part is taken from full level above
  ZHLVPT(KIDIA:KFDIA)=PTS(KIDIA:KFDIA)* &
    &    (RATM/PAPRS(KIDIA:KFDIA,KLEV))**(RKAPPA) * ZMOIST(KIDIA:KFDIA,KLEV)
  ZDELTVPT(KIDIA:KFDIA)=ZVPT(KIDIA:KFDIA)-ZHLVPT(KIDIA:KFDIA)
  ZBVMO(KIDIA:KFDIA,KLEV)= RG*ZDELTVPT(KIDIA:KFDIA)/ &
   & (ZHLVPT(KIDIA:KFDIA)*ZDZZ(KIDIA:KFDIA,KLEV))
ENDIF

! and a bit of security to avoid a division by 0 ...
WHERE (ABS(ZBVMO(KIDIA:KFDIA,KTDIA:KLEV))< ZEPS)
  ZBVMO(KIDIA:KFDIA,KTDIA:KLEV)=ZEPS
ENDWHERE

! C3 computation
ZC3(KIDIA:KFDIA,KTDIA:KLEV-1)=PLMT(KIDIA:KFDIA,KTDIA:KLEV-1) &
  &                         / PLMU(KIDIA:KFDIA,KTDIA:KLEV-1)
ZC3(KIDIA:KFDIA,KLEV)  = 1._JPRB

!---------------------------------------------------------------------
!  1. mixing length computation
IF (LCOEFK_PL) THEN
  PLMLF(KIDIA:KFDIA,KTDIA:KLEV)=MAX(PLMLF(KIDIA:KFDIA,KTDIA:KLEV), ZEPS)
ENDIF

IF ((KSTEP == 0).AND.(YTKE_NL%NREQIN == 0)) THEN

  ! special treatment for the first timestep
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZLMU(JLON,JLEV)=PLMU(JLON,JLEV)
    ENDDO
  ENDDO

  PLMLTILD(KIDIA:KFDIA,KTDIA:KLEV-1)=ZLMU(KIDIA:KFDIA,KTDIA:KLEV-1)*ZRKONV
  PLML(KIDIA:KFDIA,KTDIA:KLEV-1)=PLMLTILD(KIDIA:KFDIA,KTDIA:KLEV-1)

  IF (LCOEFK_PL) THEN
    DO JLEV=KTDIA+1,KLEV-1
      DO JLON=KIDIA,KFDIA
        PLMLF(JLON,JLEV)=(PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)*PLMLTILD(JLON,JLEV-1) + &
     &          (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))*PLMLTILD(JLON,JLEV))
      ENDDO
    ENDDO
  ENDIF

ELSE

  ! any other obvious timestep or when TKE is initialized

  !---------------------------------------------------------------------

  ! 1.1  Computation of the BL89 mixing length (on full levels) 

  IF ((KMLTYPE == 1).or.(KMLTYPE == 2).or.(KMLTYPE == 5)) THEN

    ! 1.1.0  Main loop on model levels

    DO JLEV=KTDIA,KLEV

      ! 1.1.1   mixing length for a downwards displacement

      ZINTE(KIDIA:KFDIA)=TKEMULT*PTKE(KIDIA:KFDIA,JLEV)
      ZLWORK(KIDIA:KFDIA)=0._JPRB
      ZTESTM=1._JPRB
      DO JJLEV=JLEV,KLEV
        IF(ZTESTM > 0._JPRB) THEN
          ZTESTM=0._JPRB
          DO JLON=KIDIA,KFDIA
            ZTEST0=0.5_JPRB+SIGN(0.5_JPRB,ZINTE(JLON))
            ZTESTM=ZTESTM+ZTEST0
            zlwork1_= ZDZZ(JLON,JJLEV)
            zpote_ = ZTEST0*ZBVMO(JLON,JJLEV)*ZDZZ2(JLON,JJLEV)
            ZTEST =0.5_JPRB+SIGN(0.5_JPRB,ZINTE(JLON)-zpote_)
            zlwork2_= SQRT(ABS(ZINTE(JLON)/ZBVMO(JLON,JJLEV)))
            ZLWORK(JLON)=ZLWORK(JLON)+ZTEST0*(ZTEST*zlwork1_+       &
             &     (1._JPRB-ZTEST)*zlwork2_)
            ZINTE(JLON) = ZINTE(JLON) - zpote_
          ENDDO
        ELSE
          EXIT
        ENDIF
      ENDDO

      ! 1.1.2.   intermediate storage (and security) of L_down

      ZLM(KIDIA:KFDIA,JLEV)=MIN(  ZLWORK(KIDIA:KFDIA), &
         &  (PAPHIF(KIDIA:KFDIA,JLEV)-PAPHI(KIDIA:KFDIA,KLEV))/RG  )
  
      ! 1.1.3.  mixing length for an upwards displacement
      !         (Note: The uppermost level computation is skipped,
      !              so results in L_up=0 there. )
             
  
      ZINTE(KIDIA:KFDIA)=TKEMULT*PTKE(KIDIA:KFDIA,JLEV)
      ZLWORK(KIDIA:KFDIA)=0._JPRB
      ZTESTM=1._JPRB
      DO JJLEV=JLEV-1,KTDIA,-1
        IF(ZTESTM > 0._JPRB) THEN
        ZTESTM=0._JPRB
          DO JLON=KIDIA,KFDIA
            ZTEST0=0.5_JPRB+SIGN(0.5_JPRB,ZINTE(JLON))
            ZTESTM=ZTESTM+ZTEST0
            zlwork1_=ZDZZ(JLON,JJLEV)
            zpote_ = ZTEST0*ZBVMO(JLON,JJLEV)*ZDZZ2(JLON,JJLEV)
            ZTEST =0.5_JPRB+SIGN(0.5_JPRB,ZINTE(JLON)-zpote_)
            zlwork2_= SQRT(ABS(ZINTE(JLON)/ZBVMO(JLON,JJLEV)))
            ZLWORK(JLON)=ZLWORK(JLON)+ZTEST0*(ZTEST*zlwork1_+  &
             &          (1._JPRB-ZTEST)*zlwork2_)
            ZINTE(JLON) = ZINTE(JLON) - zpote_
          ENDDO
        ELSE
          EXIT
        ENDIF
      ENDDO

      ! 1.1.4. final mixing length

      ! following formula should be compatible with the  
      ! ZLM(:,:)= ( 0.5 * (ZLM(:,:)**(-4/5) + ZLWORK(:)**(-4/5)) )**(-5/4)
      ! Note that this differs from the original BL89 formulation.
      DO JLON=KIDIA,KFDIA
         zlwork1_=MAX(ZLM(JLON,JLEV),ZEPS)
         zlwork2_=MAX(ZLWORK(JLON)  ,ZEPS)
         zpote_  = zlwork1_ / zlwork2_
         zlwork2_=(1._JPRB + (zpote_)**(0.8_JPRB))
         ZLM(JLON,JLEV) = ZPOWERED2 * zlwork1_/ &
          &               (zlwork2_*SQRT(SQRT(zlwork2_)))
      ENDDO

    ! end of the loop on the vertical levels

    ENDDO

    ! 1.1.5 Conversion of full level L-type mixing length to half level
    !        Prandtl (l_m) type
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        ZLMU(JLON,JLEV)=(ZLM(JLON,JLEV)+ZLM(JLON,JLEV+1))*ZKONVH
      ENDDO
    ENDDO

  ENDIF ! KMLTYPE = 1,2,5

  !---------------------------------------------------------------------
  ! 1.2. Computation of the Deardorff's (sqrt(e)/N) type mixing length 
  !      converted to Prandtl type (the unstable regions are prolongated
  !      by pure l_m mixing length)
  !
  IF ((KMLTYPE == 3).or.(KMLTYPE == 4).or.(KMLTYPE == 5)) THEN
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        IF (ZBVMO(JLON,JLEV) > 0.0_JPRB) THEN
          ZLMD(JLON,JLEV)=MAX(ZEPS,ZKONV*SQRT(2._JPRB*TKEMULT* &
           &  SQRT(PTKE(JLON,JLEV)*PTKE(JLON,JLEV+1))/ZBVMO(JLON,JLEV)))
        ELSE
          ZLMD(JLON,JLEV)=PLMU(JLON,JLEV)
        ENDIF
      ENDDO
    ENDDO
  ENDIF

  !---------------------------------------------------------------------
  ! 2. Combinations of the different types of mixing lengths into final one

  SELECT CASE (KMLTYPE)

    ! 2.2  BL89 secured by l_m for unstable regions
    CASE (2) 

      WHERE (ZBVMO(KIDIA:KFDIA,KTDIA:KLEV-1) < 0.0_JPRB ) 
        ZLMU(KIDIA:KFDIA,KTDIA:KLEV-1)=MIN(SQRT(ZLMU(KIDIA:KFDIA,KTDIA:KLEV-1) &
         & *PLMU(KIDIA:KFDIA,KTDIA:KLEV-1)),ZLMU(KIDIA:KFDIA,KTDIA:KLEV-1))
      ENDWHERE

    ! 2.3  Deardorff's (1974) mixing length secured for stable
    !      stratifications. (Unstable regions are pure l_m.)
    CASE (3)  

      WHERE (ZBVMO(KIDIA:KFDIA,KTDIA:KLEV-1) > 0.0_JPRB ) 
        ZLMU(KIDIA:KFDIA,KTDIA:KLEV-1)=MIN(ZLMD(KIDIA:KFDIA,KTDIA:KLEV-1), &
         &                             ZLMAX)
      ELSEWHERE
        ZLMU(KIDIA:KFDIA,KTDIA:KLEV-1)=ZLMD(KIDIA:KFDIA,KTDIA:KLEV-1)
      ENDWHERE

    ! 2.4  Deardorff mixing length asymptoticaly mixed by 
    !      generalized Blackadar formula (with power of 2) with l_m.
    !      Unstable regions are pure l_m.
    CASE (4)

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA
          IF (ZBVMO(JLON,JLEV) > 0.0_JPRB) THEN
            zlwork1_= PLMU(JLON,JLEV)
            ZLMU(JLON,JLEV)= ZLMD(JLON,JLEV)*zlwork1_ / &
             & SQRT(ZLMD(JLON,JLEV)*ZLMD(JLON,JLEV)+zlwork1_*zlwork1_)
          ELSE
            ZLMU(JLON,JLEV)=ZLMD(JLON,JLEV)
          ENDIF
          ! more general but more expensive Blackadar formulae
          !IF (ZBVMO(JLON,JLEV) > 0.0_JPRB)  ZLMU(JLON,JLEV)= &
          ! & BLACK(ZLMU(JLON,JLEV),PLMU(JLON,JLEV),ZQBLACK)
        ENDDO
      ENDDO

    ! 2.5  BL89 modified for stable regions as min(BL89,Deardorff)
    !      (this is very much inspired by CCH02)
    CASE (5)  

      WHERE (ZBVMO(KIDIA:KFDIA,KTDIA:KLEV-1) > 0.0_JPRB ) 
        ZLMU(KIDIA:KFDIA,KTDIA:KLEV-1)=MIN(ZLMU(KIDIA:KFDIA,KTDIA:KLEV-1), &
         &                                 ZLMD(KIDIA:KFDIA,KTDIA:KLEV-1))
      ENDWHERE
    CASE (0)  
        ZLMU(KIDIA:KFDIA,KTDIA:KLEV-1)=PLMU(KIDIA:KFDIA,KTDIA:KLEV-1)
  ENDSELECT


  PLMLTILD(KIDIA:KFDIA,KTDIA:KLEV-1)=ZLMU(KIDIA:KFDIA,KTDIA:KLEV-1)*ZRKONV
ENDIF 


IF (LCOEFK_PL.AND.(.NOT.LDMAF)) THEN
  IF (KSTEP == 0) THEN
    DO JLEV=KTDIA+1,KLEV
      DO JLON=KIDIA,KFDIA
        PLMLF(JLON,JLEV)=(PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)*PLMLTILD(JLON,JLEV-1) + &
     &          (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))*PLMLTILD(JLON,JLEV))
      ENDDO
    ENDDO
  ELSE
  !CONVERSION OF PROGNOSTIC L on full level to half level l_m
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        PLML(JLON,JLEV)=(PLMLF(JLON,JLEV)+PLMLF(JLON,JLEV+1))*0.5_JPRB
        ZLMU(JLON,JLEV)=PLML(JLON,JLEV)*ZKONV
      ENDDO
    ENDDO
  ENDIF
ELSE
  PLML(KIDIA:KFDIA,KTDIA:KLEV-1)=PLMLTILD(KIDIA:KFDIA,KTDIA:KLEV-1)
ENDIF


!---------------------------------------------------------------------
! 3. MOIST GUSTINESS CORRECTION 
IF(LRRGUST) THEN
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZLMU(JLON,JLEV)=ZLMU(JLON,JLEV)*PRRCOR(JLON,JLEV)
      PLML(JLON,JLEV)=PLML(JLON,JLEV)*PRRCOR(JLON,JLEV)
!     PLMLTILD(JLON,JLEV)=PLMLTILD(JLON,JLEV)*PRRCOR(JLON,JLEV)
    ENDDO
  ENDDO
ENDIF

!---------------------------------------------------------------------
! 4. Treatment for upper and lower bounds and final export
!    to output arrays.

!lower bound (note, that TOM solver require full level value at the bb)
ZLMU(KIDIA:KFDIA,KLEV)= PGZ0(KIDIA:KFDIA)*VKARMN/RG
PLML(KIDIA:KFDIA,KLEV)=ZRKONV*ZLMU(KIDIA:KFDIA,KLEV)
PLMLTILD(KIDIA:KFDIA,KLEV)=PLML(KIDIA:KFDIA,KLEV)

IF ((KSTEP == 0) .AND. LCOEFK_PL) THEN
    PLMLF(KIDIA:KFDIA,KLEV)=(PALPH(KIDIA:KFDIA,KLEV)/PLNPR(KIDIA:KFDIA,KLEV)*&
     & PLML(KIDIA:KFDIA,KLEV-1) + &
     & (1.0_JPRB-PALPH(KIDIA:KFDIA,KLEV)/PLNPR(KIDIA:KFDIA,KLEV))*&
     & PLML(KIDIA:KFDIA,KLEV))
    PLMLF(KIDIA:KFDIA,KTDIA)=PLMLF(KIDIA:KFDIA,KTDIA+1)
    PLMLF(KIDIA:KFDIA,KTDIA:KLEV)=MAX(ZEPSLML,PLMLF(KIDIA:KFDIA,KTDIA:KLEV))
ENDIF

!uppermost level is probably not needed but anyway...
ZLMU(KIDIA:KFDIA,KTDIA-1)=ZLMU(KIDIA:KFDIA,KTDIA)
! is needed for 3D turbulence
PLML(KIDIA:KFDIA,KTDIA-1)=PLML(KIDIA:KFDIA,KTDIA)

PLMLTILD(KIDIA:KFDIA,KTDIA-1)=PLMLTILD(KIDIA:KFDIA,KTDIA)


! Finaly overwrite the original mixing length by the new one.
PLMU(KIDIA:KFDIA,KTDIA-1:KLEV)=ZLMU(KIDIA:KFDIA,KTDIA-1:KLEV)

!mix. length for heat
IF(LCOEFKTKE) THEN
  !no vert. profile of C3
  PLMT(KIDIA:KFDIA,KTDIA:KLEV)=C3TKEFREE*PLMU(KIDIA:KFDIA,KTDIA:KLEV)
ELSE
  !vert. profile of C3
  PLMT(KIDIA:KFDIA,KTDIA:KLEV)=ZC3(KIDIA:KFDIA,KTDIA:KLEV)*&
  & PLMU(KIDIA:KFDIA,KTDIA:KLEV)
ENDIF

! ----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('ACMIXELEN',1,ZHOOK_HANDLE)

END SUBROUTINE ACMIXELEN
