SUBROUTINE SUGFL1

!     ----------------------------------------------------------------------
!**** *SUGFL1 * - Setting up GFL environment: part 1

!     Purpose.
!     --------
!      Initialization of YOM_YGFL, part 1.
!      Reads NAMGFL.
!      Does checkings on Y[X]_NL attributes.

!**   Interface.
!     ----------
!        *CALL* *SUGFL1*

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!      Mats Hamrud and Philippe Courtier  *ECMWF*
!      Original : 87-10-15
!      Code in SUDIM1 until 2011
!      Creation of SUGFL1 in Nov 2011.

!     Modifications.
!     --------------
!      K. YESSAD (Nov 2011) former GFL SUDIM1 code put in new SUGFL1
!     ----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULNAM
USE YOMDYNA  , ONLY : LSLDIA

USE YOM_YGFL , ONLY : YQ_NL    ,YI_NL    ,YL_NL    ,YS_NL    ,YR_NL    , &
 & YLCONV_NL,YICONV_NL,YRCONV_NL,YSCONV_NL, &
 & YG_NL    ,YH_NL   ,YTKE_NL  ,YA_NL    ,YO3_NL   ,YEXT_NL  ,YCPF_NL  , &
 & YSPF_NL  , YSRC_NL,YCVGQ_NL ,YAERO_NL ,YGHG_NL  ,YGRG_NL  ,YTRAC_NL , &
 & YSDSAT_NL, YCVV_NL,YFORC_NL ,YQVA_NL  ,YEZDIAG_NL, YUOM_NL  ,YUAL_NL, &
 & YDOM_NL  , YDAL_NL,YUEN_NL  ,YUNEBH_NL,YERA40_NL ,NGFL_EXT  ,YNOGW_NL, NACTAERO,&
 & YIRAD_NL    ,YLRAD_NL    ,YEFB1_NL  ,YEFB2_NL  ,YEFB3_NL, &
 & NAERO    , NGHG   ,NGRG     ,NTRAC    ,NGFL_FORC ,NGFL_EZDIAG,NERA40, &
 & YRKTH_NL ,YRKTQV_NL,YRKTQC_NL, YTTE_NL, YMXL_NL, &
 & LAEROSFC ,LAERODIU,LFIRE    ,LSF6SFC, LGRGSFC, LCO2SFC, YGRGTEND_NL , &
 & LCH4SFC, YLRCH4_NL, YCH4S_NL, LTRCMFIX, NGRG_ASSIM, NGRG_CPLO4 , &
 & NSLDIA, NSLDIAGP, YSLDIA_NL, YAEROUT_NL, YUVP_NL, NAEROUT, NUVP, LAEROUT, LUVPOUT
USE TYPE_GEMS_PROFILES, ONLY : NGEMS
USE YOMGRB, ONLY : NGRBGRG, NGRBGRGSFC, NGRBTCGRG, NGRBTRAC, NGRBTRACSFC, NGRBTCTRAC

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JGFL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "namgfl.h"

!     ------------------------------------------------------------------

#include "posnam.intfb.h"
#include "suctrl_gflattr.intfb.h"
#include "sudefo_gflattr.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUGFL1',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1. SET DEFAULT VALUES.
!           -------------------

!        1.1  Set implicit default values for GFL dimensions

NERA40=0
IF ( LSLDIA ) THEN
  NSLDIA=7
  NSLDIAGP=1
ELSE
  NSLDIA=0
  NSLDIAGP=0
ENDIF
NGHG=0
NTRAC=0
NGRG=0
NGRG_CPLO4=0
NGRG_ASSIM=0
NAERO=0
NACTAERO=0
NAEROUT=0
NUVP=0
NGFL_FORC=0
NGFL_EZDIAG=0
NGFL_EXT=0

!        1.2  Set implicit default values for logical variables other than GFL attributes

LAEROUT=.FALSE.
LUVPOUT=.FALSE.
LAEROSFC=.FALSE.
LAERODIU=.FALSE.
LGRGSFC=.FALSE.
LCO2SFC=.FALSE.
LFIRE=.FALSE.
LSF6SFC=.FALSE.
LCH4SFC=.FALSE.
LTRCMFIX=.FALSE.

!        1.3  Set implicit default values for GFL attributes
!             All the GFL attributes defaults computed before reading the
!             namelist NAMGFL SHOULD be defined in SUDEFO_GFLATTR

CALL SUDEFO_GFLATTR

!     ------------------------------------------------------------------

!*       2. READ NAMELIST.
!           --------------

CALL POSNAM(NULNAM,'NAMGFL')
READ(NULNAM,NAMGFL)

!     ------------------------------------------------------------------

!*       3. CHECKINGS.
!           ----------

CALL SUCTRL_GFLATTR

!     ------------------------------------------------------------------

!*       4. COMPUTE THE TOTAL NUMBER OF "GEMS" FIELDS.
!           ------------------------------------------

NGEMS=NGHG+NGRG+NAERO+NTRAC

!     ------------------------------------------------------------------

!*       5. FILL SOME GRIB CODES FOR GRG, TRAC.
!           -----------------------------------

!*       5.1  FILL NGRBGRG, NGRBGRGSFC and NGRBTCGRGF from Namelist input
!             one should get rid of NGRBGRG, NGRBGRGSFC and NGRBTCGRGF
!             in the future and always refer to YGRG 

NGRBGRG = -999
NGRBGRGSFC = -999
NGRBTCGRG = -999

IF(NGRG > 0 ) THEN
  DO JGFL = 1, NGRG
    NGRBGRG(JGFL) = YGRG_NL(JGFL)%IGRBCODE
    NGRBGRGSFC(JGFL) = YGRG_NL(JGFL)%IGRIBSFC
    NGRBTCGRG(JGFL) = YGRG_NL(JGFL)%IGRIBTC
    IF (YGRG_NL(JGFL)%NCOUPLO4 > 0 ) THEN 
      NGRG_CPLO4=NGRG_CPLO4 + 1 
      YGRGTEND_NL(NGRG_CPLO4)%CNAME=TRIM(YGRG_NL(JGFL)%CNAME)//'_tend'
    ENDIF
    IF (YGRG_NL(JGFL)%LASSIM) NGRG_ASSIM=NGRG_ASSIM + 1 
  ENDDO 
ENDIF

!*       5.2  FILL NGRBTRAC, NGRBTRACSFC and NGRBTCTRAC from Namelist input

NGRBTRAC = -999
NGRBTRACSFC = -999
NGRBTCTRAC = -999

IF(NTRAC > 0 ) THEN
  DO JGFL = 1, NTRAC
    NGRBTRAC(JGFL) = YTRAC_NL(JGFL)%IGRBCODE
    NGRBTRACSFC(JGFL) = YTRAC_NL(JGFL)%IGRIBSFC
    NGRBTCTRAC(JGFL) = YTRAC_NL(JGFL)%IGRIBTC
  ENDDO 
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGFL1',1,ZHOOK_HANDLE)
END SUBROUTINE SUGFL1
