SUBROUTINE SUSC2B

!**** *SUSC2B*  Initialize control of calc in gridpoint space

!     Purpose.
!     -------
!           Initialize YOMSC2, the module that controls the
!           grid - point calculations under SCAN2M.
!           Also prepare memory buffers.

!     Interface.
!     ---------
!        *CALL*  *SUSC2B*

!        Explicit arguments : None
!        ------------------
!        Implicit arguments :
!        ------------------
!        see above "USE MODULE"

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Author.
!     ------
!        Mats Hamrud  * ECMWF *
!        Original  : 91-01-15

!     Modifications.
!     -------------
!        Modified 08-2002 C. Smith : use "w" as prognostic variable in the
!         semi-lag advection of vertical divergence in the NH model.
!        R. El Khatib : 02-21-20 Fullpos B-level distrib. + remove IO scheme
!        M.Janiskova : 02-10-30 Cloud characteristics for new scheme for TL/AD (cloudst)
!        G.Mozdzynski: 03-6-25 allow radiation interpolation when lslag=f
!        M.Hamrud    : 03-08-01 GFL introduction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Janiskova: 11-11-03 perturbation of skin temperature for lin.phys.
!        M.Tudor       03-10-31 allocate GFLPT 
!        R. El Khatib 04-05-24 Allocate MASK_SL1 anytime because it is argument of subroutine
!        K. Yessad    04-07-06 Allocate GPNHBUF in susc2b instead of suesc2.
!        K. Yessad    04-07-07 Compute NFGPNH and the pointers of PTRGPPC now in susc2b.
!        20-Feb-2005 J. Vivoda 3TL Eul PC scheme (GFLPC)
!        D.Salmond     22-Nov-2005 Mods for coarser/finer physics
!        G. Desroziers 16-Mar-06 Reduced number of channels for sigmab random. if LECMWF=.F.
!        G. Desroziers 02-June-06 Add SSMI channels for sigmab random. if LECMWF=.F.
!        N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!        M. Janiskova  10-Apr-08 fields for the linearized LW radiation
!        30-Jun-2008 J. Masek    Auxiliary quantities for SLHD interpolators.
!        K. Yessad (Sep 2008): prune enhanced diffusion (lfrein), rm lpc_xidt.
!        13-Oct-2008 J. Masek    Fix for NUNDEFLD=1 case.
!        K.Yessad (Aug 2009): remove NTRSLTYPE/=2 cases
!        K. Yessad (Aug 2009): remove LPC_OLD in TL and AD codes.
!        K. Yessad (Aug 2009): use RIPI, RSLD
!        K. Yessad (Nov 2009): prune lpc_old.
!        K. Yessad (Jan 2010): revised code for obs horiz interpolator (first draft)
!	 H.Varella : 2010-04-20 Divergence included
!        04-Dec-2009 H. Hersbach Add +2 to NFLDOBB1
!        M. Janiskova  21-Sep-09 fields for TL/AD of non-orog.gravity wave
!        O. Spaniel  27-Sep-10 fix the array allocation size for NSLCORE
!        G. Mozdzynski (Jan 2011): OOPS cleaning, use of derived type SL_STRUCT
!        F. Vana  22-Feb-2011: Auxiliary quantities for horizontal turbulence.
!        2013-11, J. Masek: Allocation of ACRANEB2 intermittent storage.
! End Modifications
!-------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : MY_REGION_NS   ,MY_REGION_EW   ,&
 & MYSETW  ,NPTRLS ,NFRSTLOFF ,&
 & NPTRFLOFF,MYFRSTACTLAT,MYLSTACTLAT,NSTA,NONL ,&
 & NPTRFRSTLAT,NFRSTLAT,NLSTLAT  
USE YOMDIM   , ONLY : NDGLL    ,NPROMA   ,NPROMM   ,NDGLG    ,NDGSAL   , &
 & NFLEVG   ,NDGSUR   ,NDGENL   ,NDLON    ,NDGENG   ,NDGSAG   ,&
 & NGPBLKS  ,NDGSAH   ,NDGENH   ,NXMAX    ,NCPEC    ,NCPEC2   ,&
 & NDGUXL   ,NDGUXG   ,NDSUR1   ,NDLSUR   ,NFGPPC
USE YOM_YGFL , ONLY : YGFL     ,YO3, NGHG, NGRG, YGHG, YGRG
USE YOMCST   , ONLY : RPI      ,RA
USE YOMDPHY  , ONLY : NTILES
USE YOMCT0   , ONLY : LR2D     ,NCONF    ,LALLOPR  ,NPRINTLEV,LSLAG    ,LTWOTL   ,&
 & LELAM    ,LOBSC1   ,LOBS     ,NPROC    ,LOUTPUT  ,NSTOP    ,&
 & LSPRT    ,LNHDYN   ,LFPART2 ,LSFORC
USE YOMCT0B  , ONLY : LECMWF
USE YOMGEM   , ONLY : NLOEN    ,RSTRET   ,NGPTOT   ,&
 & NMEN     ,NLOENG   ,NMENG
USE YEMGEO   , ONLY : EDELX    ,EDELY
USE YOMLEG   , ONLY : YRCSGLEG ,RIPI0    ,RIPI1    ,RIPI2 ,RIPI ,&
 & RSLD     ,RSLDW    ,R3DTW
USE YOMDYN   , ONLY : TSTEP    ,VMAX1    ,VMAX2    ,LADVF    ,L2TLFF  
USE YOMDYNA  , ONLY : LSLHD    ,LSLHDQUAD,SLHDEPSH ,L3DTURB
USE YOMIOS   , ONLY : NPCKFCF  ,NPCKFXF  ,NPCKFGN  
USE YOMNMIA  , ONLY : NVMODM   ,LRPIMP   ,LNOINI  
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB15,NFLDSLB2  ,&
 & NDIEND   ,NDIST    ,NCIEND   ,NCIST
USE YOMGFL , ONLY : GFL, GFLSLP,GFL5,GFLPT,GFLPC
USE YOMSLREP , ONLY : LSLADREP
USE YOMCFU   , ONLY : NFDCFU   ,LCUMFU
USE YOMXFU   , ONLY : NFDXFU   ,LXFU
USE SURFACE_FIELDS_MIX , ONLY : ALLO_SURF, YSP_SBD
USE YOMGFUB  , ONLY : GFUBUF
USE YOMXFUB  , ONLY : XFUBUF
USE YOMSPHB  , ONLY : NLENSPHB ,SPHBUF   ,NSTASPHB
USE YOMHOUB  , ONLY : NLENHOUB ,HOUBUF   ,NSTAHOUB
USE YOMPINB  , ONLY : RPINBUF  ,NSTAPINB
USE YOMAUXB  , ONLY : GAUXBUF
USE YOMANEB  , ONLY : ANEBUF    ,NANEBU   ,NANEBV   ,NANEBZ   ,NANEBR  ,&
 & NANEBQ   ,NANEBT   ,NANEBDIV ,NANEBVO  ,NANEBO3  ,NANEBSP  ,&
 & NANEBRAD ,NANEBTCW ,NANEVARS ,&
 & NANEBRH2 ,NANEBU10 ,NANEBV10 ,NANEBT2  ,NANEBTO3 ,&
 & NANERADS ,NANERADL ,NANEBGHG ,NANEBGRG ,NANEBAERO
USE YOMVAR   , ONLY : NBGVECS  ,NOANEF   ,LAVCGL   ,LBGOBS
USE YOMTPHY  , ONLY : NLENGT95B,NLENGTT5B,NLENGTS5B,TRAJPHYS ,TRAJPHTE,TRAJPHSF  
USE YOMTRSL  , ONLY : NLENGTSLB,TRAJSL   ,NTRSLTYPE ,NPCKSLTRAJ  
USE YOMTNH   , ONLY : NLENNH95B,TRAJNH
USE YOMNCL   , ONLY : GPCLOUD5 ,GPCLOUDIN
USE YOMTRC   , ONLY : GDEOSI   ,GUEOSI   ,GMU0     ,GMU0_MIN ,GMU0_MAX ,&
 &                    GDEOTI   ,GDEOTI2  ,GUEOTI   ,GUEOTI2  ,GEOLT    ,&
 &                    GEOXT    ,GRPROX   ,GMIXP    ,GFLUXC   ,GRSURF
USE YOELWRAD , ONLY : RADLWM   ,RADLWM5
USE YOEGWDWMS, ONLY : TGWDWMS  ,TGWDWMS5
USE YOMIOP   , ONLY : NPCKFT95 ,NPCKFTS5 ,NPCKFPH5 ,&
 & NG3D95   ,NG3P95  ,NG3DT95  ,NG3PR95  ,&
 & NG3S95   ,NG2D95  ,NG2T95   ,NG2DS5   ,NG2DSF5  
USE YOMIOPNH , ONLY : NG3NH95  ,LTRAJNH
USE YOMSIMPHL, ONLY : LTRAJPS  ,LTRAJPST
USE YOMRCOEF , ONLY : LRCOEF    ,NG3SR   ,&
 & NPCKFSR  ,NLENGSRB ,SOLRAD    ,NLENGTRB ,LGLOBRAD ,&
 & THERRAD  ,TRWEIGHT ,TRMATSUM   
USE GRIDPOINT_BUFFERS_MIX , ONLY : ALLOCATE_GRIDPOINT_BUFFER  
USE YOPHNC   , ONLY : LETRAJP ,LETRAJPT ,LERADSW2 ,LERADN2 ,&
 & LERADLW2 ,LENCLD2  ,LEGWWMS2
USE YOMSRFTLAD  , ONLY : GPTSKIN0, GPTSKIN9
USE YOEPHY   , ONLY : LERADI 
USE YOMPHY   , ONLY : LRAY     ,LRPROX   ,NRAY     ,NSORAYFR ,NTHRAYFR ,&
 &                    NRAUTOEV
USE YOMSLPHY , ONLY : LSLPHY  , SAVTEND  ,NVTEND
USE YOMINI   , ONLY : NEINI
USE YOMGPPCB  , ONLY : GPPCBUF
USE GMV_SUBS_MOD , ONLY : SETUP_GMV
USE PTRGPPC  , ONLY : MGPPC    ,&
 & MGPPCF_U ,MGPPCF_V ,MGPPCF_T ,MGPPCF_SPD,MGPPCF_SVD,MGPPCF_SP,&
 & MGPPCF_UP,MGPPCF_VP,MGPPCF_TP,MGPPCF_GFLP,MGPPCF_NHX,MGPPCF_CP,MGPPCF_CPSI,&
 & MGPPCF_BBC,MGPPC5   ,&
 & MGPPCF_U5,MGPPCF_V5,MGPPCF_T5,MGPPCF_SPD5,MGPPCF_SVD5,MGPPCF_SP5
USE YOMJG , ONLY : JB_STRUCT ! N_SPJB_VARS,SPJB_VARS_INFO, NAEROCV
USE YOMGRB , ONLY : NGRBO3, NGRBAERLG
USE YOMCOAPHY , ONLY : NPHYINT
USE YOEWCOU  , ONLY : LWCOU
USE YOMLDDH  , ONLY : LSDDH
USE EINT_MOD , ONLY : YROB,YROA,YRSL,YRAD,YRRI

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: J, IU, JL, IDWIDE, ISTA, IEND, &
 & ISLWM2, JGL, JGLLOC, &
 & IGLGLO, ILBRLATI, IUBRLATI, ISUMMEN, &
 & ILEN, IS, IFLDSGTT5, &
 & IFLDSGT95, IFLDSGTS5, IFLDSGSR, INUMLEV, IFLDSNH95, &
 & IFLDSGTSL, &
 & IBITS, JK, JM, IORDR, IREC, ILENPINB, &
 & ISLWIDE, INFGPPC, INFGPPC5

LOGICAL :: LLINTERPSL, LLP, LLPHYS, LLO3, LLAERO

REAL(KIND=JPRB) :: ZD1, ZD2, ZD3, ZD4, ZD5, ZD6, ZDL, ZISLWM2, ZLTRAJ, ZRAPVMAX

INTEGER(KIND=JPIM) :: IANEINDEX
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!    -------------------------------------------------------------------

#include "abor1.intfb.h"
#include "slcset.intfb.h"
#include "suptrgppc.intfb.h"
#include "suslad1.intfb.h"
#include "suslad2.intfb.h"
#include "suslb.intfb.h"
#include "suslb2.intfb.h"

!    -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUSC2B',0,ZHOOK_HANDLE)
!    -------------------------------------------------------------------

!*        2. INITIALIZE YOMSC2.
!            -----------------

CALL GSTATS(1939,0)

!*        2.1   CONTROL FOR SCAN2, START ADRESSES FOR ROWS IN BUFFERS ETC.

IF (JB_STRUCT%CONFIG%N_SPJB_VARS > 0) THEN
  LLO3 = ANY(JB_STRUCT%SPJB_VARS_INFO(:)%IGRIBCODE==NGRBO3)
ELSE
  LLO3 = .FALSE.
ENDIF
IF (JB_STRUCT%CONFIG%N_SPJB_VARS > 0) THEN
  LLAERO = ANY(JB_STRUCT%SPJB_VARS_INFO(:)%IGRIBCODE==NGRBAERLG)
ELSE
  LLAERO = .FALSE.
ENDIF
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT

IDWIDE=NDGLG/2 
ISTA=1
IEND=NDGLL

!*        2.2   INTERPOLATION BUFFERS SETUP, NUMBER OF BUFFERS ETC.

!         2.2.1 Computation of NFLDSLB1.

IF ( NCONF/100  ==  7 ) THEN
  NFLDSLB1 = 0
  NFLDSLB2 = 0
  NFLDSLB15 = 0
ELSE  
  IF (LR2D) THEN
    CALL SUSLB2
  ELSE
    CALL SUSLB
  ENDIF
ENDIF

!         2.2.2 Computation of YR...%NSLWIDE.

! ky: calculation of all the YR...%NSLWIDE must be now done there.
!     Each value can be computed independently.
!     - SL scheme required: YRSL%NSLWIDE > 2 (depending on VMAX2)
!     - Eulerian scheme (or no advection scheme if no temporal advance): YRSL%NSLWIDE=0.
!     - AD of SL scheme required and LSLADREP=T: YRAD%NSLWIDE=YRSL%NSLWIDE*2; otherwise YRAD%NSLWIDE=0.
!     - Obs interpolator required: YROB%NSLWIDE=2; otherwise YROB%NSLWIDE=0.
!     - AD of Obs interpolator required: YROA%NSLWIDE=4; otherwise YROA%NSLWIDE=0.
!     - Radiation interpolator required: YRRI%NSLWIDE=2; otherwise YRRI%NSLWIDE=0.

IF ( NCONF/100  ==  7 ) THEN

  ! * configurations between 700 and 799.

  ! ky: no longer used in the obs interpolator;
  !     the former value "3" can probably be now replaced by 0.
  YRSL%NSLWIDE = 3
  ISLWIDE=YRSL%NSLWIDE

ELSE

  ! * configurations between 1 and 699 or between 801 and 999.

  IF (LSLAG) THEN
    IF (LTWOTL) THEN
      ZLTRAJ=1._JPRB
    ELSE
      ZLTRAJ=2._JPRB
    ENDIF
    IF (LELAM) THEN
      IF ( LSFORC ) THEN
        YRSL%NSLWIDE = IDWIDE
      ELSE
        ISLWM2=INT(ZLTRAJ*VMAX2*TSTEP*RSTRET/MIN(EDELX,EDELY))
        YRSL%NSLWIDE = ISLWM2 + 2
      ENDIF
      ISLWIDE=YRSL%NSLWIDE
    ELSE
      ZDL=ZLTRAJ*VMAX2*TSTEP*RSTRET/RA
      IF (ZDL > (0.5_JPRB*RPI)) THEN
        WRITE(NULOUT,*)'VMAX2 OR TSTEP TOO BIG, TAKE LOWER VALUES.'
        CALL ABOR1('SUSC2B : ABOR1 CALLED')
      ENDIF
      ZISLWM2 = ZDL * (REAL(NDGLG,JPRB) + 0.5_JPRB)/RPI
      ISLWM2 = CEILING(ZISLWM2)
      YRSL%NSLWIDE = ISLWM2 + NDGSUR
      ISLWIDE=YRSL%NSLWIDE
      ! ----------------------------------------------------------
      ! recompute VMAX1, VMAX2 to take full advantage of YRSL%NSLWIDE:
      ZRAPVMAX=VMAX1/VMAX2
      ZDL = REAL(ISLWM2,JPRB) * RPI/(REAL(NDGLG,JPRB) + 0.5_JPRB)
      IF (TSTEP /= 0.0_JPRB) THEN
        VMAX2 = ZDL * RA/(ZLTRAJ * TSTEP * RSTRET)
        VMAX1 = MIN(ZRAPVMAX * VMAX2, 250.0_JPRB)
        WRITE(NULOUT,'('' REVISED VALUES:'',&
         & '' VMAX1  = '',F6.2,'' VMAX2  = '',F6.2)') VMAX1,VMAX2  
      ENDIF
      ! ----------------------------------------------------------
    ENDIF

  ELSE

    IF (LOBSC1.OR. NCONF/100 == 1) THEN
      ! ky: no longer used in the obs interpolator;
      !     the former value "3" can probably be now replaced by 0.
      YRSL%NSLWIDE = 3
    ELSE
      YRSL%NSLWIDE = 0
    ENDIF
    ISLWIDE=YRSL%NSLWIDE

  ENDIF

ENDIF

IF( YRSL%NSLWIDE > IDWIDE )THEN
  WRITE(NULOUT,'('' SUSC2B : YRSL%NSLWIDE > IDWIDE, '',2I6)')YRSL%NSLWIDE,IDWIDE
  CALL ABOR1(' SUSC2B : YRSL%NSLWIDE > IDWIDE')
ENDIF

!         2.2.3 Allocate arrays based on YR[OB,SL,AD]%NSLWIDE and initialised 
!               in SLCSET, call SLCSET.

IF (LSLAG.OR.LOBS.OR.NCONF/100 == 7) THEN  ! WHAT ABOUT LOBSC1 ???

  IF( LOBS )THEN

!     ideally yrob%nslwide should be computed as a function of nobshor
!     assume nobshor=203 (12 points interp) as this requires the larger halo
    YROB%CVER='OB'
!     NSLWIDE=2 is required for slint and slinttl interpolations
    YROB%NSLWIDE=2
    CALL SLCSET(YROB,MY_REGION_NS,MY_REGION_EW,&
     & NDGLG,NDLON,NDGSAG,NDGENG,NDGUXL,NDGUXG,NDGSAL,NDGENL,&
     & NDSUR1,NDLSUR,NDGSUR,NGPTOT,&
     & NPTRFLOFF,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,&
     & NSTA,NONL,NLOENG,NPTRFRSTLAT,NFRSTLAT,NLSTLAT,&
     & YRCSGLEG%RMU,YRCSGLEG%RSQM2)

    YROA%CVER='OA'
!     NSLWIDE=4 i.e. double required for slintad interpolations
!     note only the basic slcset initialisation is required
    YROA%NSLWIDE=YROB%NSLWIDE*2
    CALL SLCSET(YROA,MY_REGION_NS,MY_REGION_EW,&
     & NDGLG,NDLON,NDGSAG,NDGENG,NDGUXL,NDGUXG,NDGSAL,NDGENL,&
     & NDSUR1,NDLSUR,NDGSUR,NGPTOT,&
     & NPTRFLOFF,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,&
     & NSTA,NONL,NLOENG,NPTRFRSTLAT,NFRSTLAT,NLSTLAT,&
     & YRCSGLEG%RMU,YRCSGLEG%RSQM2,LDCSETONLY=.TRUE.)

    YRSL%NSLWIDE=MAX(YROB%NSLWIDE,YRSL%NSLWIDE)
    YRSL%NSLWIDE=MAX(YROA%NSLWIDE,YRSL%NSLWIDE)

  ENDIF


!  * slcset is called to compute a halo based on a standard or double halo width

!   * call of slcset for a halo based on a standard halo width
  YRSL%CVER='SL'
  ISLWIDE=YRSL%NSLWIDE
  CALL SLCSET(YRSL,MY_REGION_NS,MY_REGION_EW,&
   & NDGLG,NDLON,NDGSAG,NDGENG,NDGUXL,NDGUXG,NDGSAL,NDGENL,&
   & NDSUR1,NDLSUR,NDGSUR,NGPTOT,&
   & NPTRFLOFF,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,&
   & NSTA,NONL,NLOENG,NPTRFRSTLAT,NFRSTLAT,NLSTLAT,&
   & YRCSGLEG%RMU,YRCSGLEG%RSQM2)

  IF (LSLADREP.AND.LSLAG.AND.(NCONF == 131.OR.NCONF == 401&
     & .OR.NCONF == 601.OR.NCONF == 801)) THEN  
!       * call of slcset for a halo based on a double standard halo width
    YRAD%CVER='AD'
    YRAD%NSLWIDE=YRSL%NSLWIDE*2
    ISLWIDE=YRAD%NSLWIDE
    CALL SLCSET(YRAD,MY_REGION_NS,MY_REGION_EW,&
     & NDGLG,NDLON,NDGSAG,NDGENG,NDGUXL,NDGUXG,NDGSAL,NDGENL,&
     & NDSUR1,NDLSUR,NDGSUR,NGPTOT,&
     & NPTRFLOFF,NFRSTLOFF,MYFRSTACTLAT,MYLSTACTLAT,&
     & NSTA,NONL,NLOENG,NPTRFRSTLAT,NFRSTLAT,NLSTLAT,&
     & YRCSGLEG%RMU,YRCSGLEG%RSQM2)

    CALL SUSLAD1

  ENDIF

  IF (LSLADREP.AND.LSLAG.AND.(NCONF == 131.OR.NCONF == 401&
     & .OR.NCONF == 601.OR.NCONF == 801)) THEN  
!     Initialise constant arrays for SL adjoint interpolations
    CALL SUSLAD2
  ENDIF

  IF(LSLPHY) THEN
    ALLOCATE(SAVTEND(NPROMA,NFLEVG,NVTEND,NGPBLKS))
    IF (LLP) WRITE(IU,9) 'SAVTEND  ',SIZE(SAVTEND  ),SHAPE(SAVTEND  )
    SAVTEND(:,:,:,:) = 0.0_JPRB
  ENDIF

ENDIF
! ** arrays used only in semi-Lagrangian and observation
!    interpolations: currently NSLWIDE is kept for dimension.
DEALLOCATE(NLOEN)
DEALLOCATE(NMEN)
ALLOCATE(NLOEN  (NDGSAL-ISLWIDE:NDGLL+ISLWIDE))
IF(LLP)WRITE(IU,9) 'NLOEN    ',SIZE(NLOEN    ),SHAPE(NLOEN    )
ALLOCATE(NMEN   (NDGSAL-ISLWIDE:NDGLL+ISLWIDE))
IF(LLP)WRITE(IU,9) 'NMEN     ',SIZE(NMEN     ),SHAPE(NMEN     )

DO JGLLOC=NDGSAL-ISLWIDE,NDGLL+ISLWIDE
  IGLGLO=NPTRLS(MYSETW)+JGLLOC-1
  IF( IGLGLO >= NDGSAG.AND.IGLGLO <= NDGENG )THEN
    NLOEN(JGLLOC)=NLOENG(IGLGLO)
    NMEN (JGLLOC)=NMENG(IGLGLO)
  ENDIF
ENDDO

! * Do we need to use an interpolator calling (E)LASCAW or (E)RDSCAW?
!   (for ex semi-Lagrangian, COBS+COBSLAG observation, radiation or
!   for coarse physics).
LLINTERPSL=.FALSE.
IF (LELAM) THEN
  IF (LSLAG) LLINTERPSL=.TRUE.
ELSE
  IF(NDGSAG <= 0.AND.NDGENG >= NDGLG+1) LLINTERPSL=.TRUE.
ENDIF
! check for new radiation interpolation
IF( (LERADI .AND. MAX(YRRI%NSLWIDEN,YRRI%NSLWIDES) > 0) .OR. NPHYINT > 0)LLINTERPSL=.TRUE.

ISLWIDE=MAX(ISLWIDE,YRRI%NSLWIDEN,YRRI%NSLWIDES)

IF(LLINTERPSL)THEN
  ! ** arrays used only in semi-Lagrangian ,observation and radiation
  !    interpolations (requiring a call to LASCAW or RDSCAW).

  ! allocations:
  ILBRLATI = MAX(NDGSAG,NDGSAL+NFRSTLOFF-ISLWIDE)-NFRSTLOFF
  IUBRLATI = MIN(NDGENG,NDGENL+NFRSTLOFF+ISLWIDE)-NFRSTLOFF
  ! ky: RIPI has replaced RIPI0 to RIPI2 in the SL and obs interpolators.
  ALLOCATE(RIPI(ILBRLATI:IUBRLATI,3))
  IF (LLP) WRITE(IU,9) 'RIPI     ',SIZE(RIPI),SHAPE(RIPI)
  ! ky: RIPI0, RIPI1, RIPI2 still used elsewhere
  ALLOCATE(RIPI0(ILBRLATI:IUBRLATI))
  IF (LLP) WRITE(IU,9) 'RIPI0    ',SIZE(RIPI0),SHAPE(RIPI0)
  ALLOCATE(RIPI1(ILBRLATI:IUBRLATI))
  IF (LLP) WRITE(IU,9) 'RIPI1    ',SIZE(RIPI1),SHAPE(RIPI1)
  ALLOCATE(RIPI2(ILBRLATI:IUBRLATI))
  IF (LLP) WRITE(IU,9) 'RIPI2    ',SIZE(RIPI2),SHAPE(RIPI2)

  ALLOCATE(YRCSGLEG%RLATI (ILBRLATI:IUBRLATI))

  ! ky: for RSLD.. arrays it is recommended to always allocate them,
  !     to avoid false alarms in "bound checking" runs.
  ALLOCATE(RSLD(ILBRLATI:IUBRLATI,3))
  IF (LLP) WRITE(IU,9) 'RSLD     ',SIZE(RSLD),SHAPE(RSLD)
  ALLOCATE(RSLDW(3,3,ILBRLATI:IUBRLATI))
  IF (LLP) WRITE(IU,9) 'RSLDW    ',SIZE(RSLDW),SHAPE(RSLDW)
  ALLOCATE(R3DTW(3,3,ILBRLATI:IUBRLATI))
  IF (LLP) WRITE(IU,9) 'R3DTW    ',SIZE(R3DTW),SHAPE(R3DTW)

  ! set-up:
  DO JGL= ILBRLATI,IUBRLATI
    IGLGLO=JGL+NFRSTLOFF
    IF(LELAM) THEN

      ! * RIPI arrays
      RIPI0(JGL)=-1.0_JPRB
      RIPI1(JGL)= 1.0_JPRB
      RIPI2(JGL)=-1.0_JPRB
      RIPI(JGL,1)=RIPI0(JGL)
      RIPI(JGL,2)=RIPI1(JGL)
      RIPI(JGL,3)=RIPI2(JGL)

      ! * RSLD arrays
      IF (LSLHDQUAD) THEN
        ! These arrays are not used in ELASCAW, where corresponding
        ! weights are hardcoded for higher efficiency. Still they
        ! are correctly initialized here even for LELAM case.
        RSLD(JGL,1)=-1.0_JPRB
        RSLD(JGL,2)=-1.0_JPRB
        RSLD(JGL,3)=-0.25_JPRB
      ELSE
        ! put an absurd value in these arrays which should not be used in this case.
        RSLD(JGL,1)=-9999._JPRB
        RSLD(JGL,2)=-9999._JPRB
        RSLD(JGL,3)=-9999._JPRB
      ENDIF
      IF (LSLHD) THEN
        ! this array is not used in ELASCAW, where corresponding
        ! weights are hardcoded for higher efficiency. Still they
        ! are correctly initialized here even for LELAM case.
        RSLDW(1,1,JGL)=1.0_JPRB-2.0_JPRB*SLHDEPSH
        RSLDW(1,2,JGL)=SLHDEPSH
        RSLDW(1,3,JGL)=0.0_JPRB
        RSLDW(2,1,JGL)=SLHDEPSH
        RSLDW(2,2,JGL)=1.0_JPRB-2.0_JPRB*SLHDEPSH
        RSLDW(2,3,JGL)=0.0_JPRB
        RSLDW(3,1,JGL)=0.0_JPRB
        RSLDW(3,2,JGL)=SLHDEPSH
        RSLDW(3,3,JGL)=1.0_JPRB
      ELSE
        ! put an absurd value in this array which should not be used in this case.
        RSLDW(:,:,:)=-9999._JPRB
      ENDIF
      IF (L3DTURB) THEN
        ! this array is not used in ELASCAW, where corresponding
        ! weights are hardcoded for higher efficiency. Still they
        ! are correctly initialized here even for LELAM case.
        R3DTW(1,1,JGL)=0.0_JPRB
        R3DTW(1,2,JGL)=0.5_JPRB
        R3DTW(1,3,JGL)=0.0_JPRB
        R3DTW(2,1,JGL)=0.5_JPRB
        R3DTW(2,2,JGL)=0.0_JPRB
        R3DTW(2,3,JGL)=0.0_JPRB
        R3DTW(3,1,JGL)=0.0_JPRB
        R3DTW(3,2,JGL)=0.5_JPRB
        R3DTW(3,3,JGL)=1.0_JPRB
      ELSE
        ! put an absurd value in this array which should not be used in this case.
        R3DTW(:,:,:)=-9999._JPRB
      ENDIF

    ELSE
      ! ky: this condition should be always T if LLINTERPSL=T (otherwise there
      !     is a bug somewhere).
      IF(IGLGLO >= 0.AND.IGLGLO <= NDGLG) THEN

        ! * RIPI arrays
        ZD1=YRCSGLEG%RLATIG(IGLGLO-1)-YRCSGLEG%RLATIG(IGLGLO)
        ZD2=YRCSGLEG%RLATIG(IGLGLO-1)-YRCSGLEG%RLATIG(IGLGLO+1)
        ZD3=YRCSGLEG%RLATIG(IGLGLO-1)-YRCSGLEG%RLATIG(IGLGLO+2)
        ZD4=YRCSGLEG%RLATIG(IGLGLO  )-YRCSGLEG%RLATIG(IGLGLO+1)
        ZD5=YRCSGLEG%RLATIG(IGLGLO  )-YRCSGLEG%RLATIG(IGLGLO+2)
        ZD6=YRCSGLEG%RLATIG(IGLGLO+1)-YRCSGLEG%RLATIG(IGLGLO+2)

        RIPI0(JGL)=-1.0_JPRB/(ZD1*ZD4*ZD5)
        RIPI1(JGL)= 1.0_JPRB/(ZD2*ZD4*ZD6)
        RIPI2(JGL)=-1.0_JPRB/(ZD3*ZD5*ZD6)
        RIPI(JGL,1)=RIPI0(JGL)
        RIPI(JGL,2)=RIPI1(JGL)
        RIPI(JGL,3)=RIPI2(JGL)

        ! * RSLD arrays
        IF (LSLHDQUAD) THEN
          RSLD(JGL,1)=(ZD6-ZD2)/ZD4
          RSLD(JGL,2)=(ZD1-ZD5)/ZD4
          RSLD(JGL,3)=-ZD4*ZD4/(ZD1*ZD2+ZD5*ZD6)
        ELSE
          ! put an absurd value in these arrays which should not be used in this case.
          RSLD(JGL,1)=-9999._JPRB
          RSLD(JGL,2)=-9999._JPRB
          RSLD(JGL,3)=-9999._JPRB
        ENDIF
        IF (LSLHD) THEN
          RSLDW(1,1,JGL)=1.0_JPRB-2.0_JPRB*SLHDEPSH*ZD4/ZD1
          RSLDW(1,2,JGL)=2.0_JPRB*SLHDEPSH*ZD4/ZD5
          RSLDW(1,3,JGL)=0.0_JPRB
          RSLDW(2,1,JGL)=2.0_JPRB*SLHDEPSH*ZD4/ZD2
          RSLDW(2,2,JGL)=1.0_JPRB-2.0_JPRB*SLHDEPSH*ZD4/ZD6
          RSLDW(2,3,JGL)=0.0_JPRB
          RSLDW(3,1,JGL)=0.0_JPRB
          RSLDW(3,2,JGL)=2.0_JPRB*SLHDEPSH*(ZD4*ZD4)/(ZD5*ZD6)
          RSLDW(3,3,JGL)=1.0_JPRB
        ELSE
          ! put an absurd value in this array which should not be used in this case.
          RSLDW(:,:,:)=-9999._JPRB 
        ENDIF
        IF (L3DTURB) THEN
          R3DTW(1,1,JGL)=1.0_JPRB-1.0_JPRB*ZD4/ZD1
          R3DTW(1,2,JGL)=1.0_JPRB*ZD4/ZD5
          R3DTW(1,3,JGL)=0.0_JPRB
          R3DTW(2,1,JGL)=1.0_JPRB*ZD4/ZD2
          R3DTW(2,2,JGL)=1.0_JPRB-1.0_JPRB*ZD4/ZD6
          R3DTW(2,3,JGL)=0.0_JPRB
          R3DTW(3,1,JGL)=0.0_JPRB
          R3DTW(3,2,JGL)=1.0_JPRB*(ZD4*ZD4)/(ZD5*ZD6)
          R3DTW(3,3,JGL)=1.0_JPRB
        ELSE
          ! put an absurd value in this array which should not be used in this case.
          R3DTW(:,:,:)=-9999._JPRB 
        ENDIF

      ENDIF
    ENDIF
    YRCSGLEG%RLATI(JGL)=YRCSGLEG%RLATIG(IGLGLO)
  ENDDO
ELSE
  ALLOCATE(YRCSGLEG%RLATI (1))
ENDIF

!         2.2.4 compute NDGSAH, NDGENH for SL, observation and
!               radiation interpolations.

NDGSAH = MAX(NDGSAG,NDGSAL+NFRSTLOFF-ISLWIDE)-NFRSTLOFF
NDGENH = MIN(NDGENG,NDGENL+NFRSTLOFF+ISLWIDE)-NFRSTLOFF

!*        2.3   PRINTS ON OUTPUT

IF (LOUTPUT) THEN
  WRITE(NULOUT,'('' COMMON YOMSC2'')')
  IF (NPRINTLEV >= 1) THEN
    WRITE(NULOUT,'('' NDIST  ,NDIEND '')')
    WRITE(NULOUT,'(16(1X,I6))')(NDIST(JGL),NDIEND(JGL),JGL=ISTA,IEND)
    WRITE(NULOUT,'('' NCIST  ,NCIEND '')')
    WRITE(NULOUT,'(16(1X,I6))')(NCIST(JGL),NCIEND(JGL),JGL=ISTA,IEND)
  ENDIF
  WRITE(NULOUT,'(''ISLWIDE ='',I4 &
   & ,'' NFLDSLB1 ='',I4,'' NFLDSLB2 ='',I4,'' NFLDSLB15 ='',I4,/ &
   & ,'' NGPTOT ='',I8)') ISLWIDE &
   & ,NFLDSLB1,NFLDSLB2,NFLDSLB15 &
   & ,NGPTOT  
  WRITE(NULOUT,'(1X,A,I4,A,I4)')' NDGSAH = ',NDGSAH,' NDGENH = ',NDGENH
ENDIF

!    -------------------------------------------------------------------

!*        3.  BUFFER ALLOCATIONS.
!         -----------------------

!         3.1   SETUP GMV STRUCTURE

CALL SETUP_GMV

!*        3.2   TIMESTEP T - DT

!*        3.2.1   TIMESTEP T-DT FOR TRAJECTORY
!                 (PHYSICS OR SIMPLIFIED PHYSICS )

IF(LTRAJPS)THEN
  IFLDSGT95 = NG3D95*NFLEVG+NG3P95*(NFLEVG+1)+NG3S95*YSP_SBD%NLEVS+&
   & NG2T95*NTILES+NG2D95  
  ILEN=0
  DO JL=1,NGPTOT,NPROMA
    IS=MIN(NGPTOT-JL+1,NPROMA)
    ILEN=ILEN+(IS+NPCKFT95-1)/NPCKFT95
  ENDDO
  NLENGT95B = ILEN    * IFLDSGT95
  ALLOCATE(TRAJPHYS(NLENGT95B,0:NSTOP))
  IF (LLP) WRITE(IU,9) 'TRAJPHYS ',SIZE(TRAJPHYS),SHAPE(TRAJPHYS)
ENDIF

LLPHYS = ((LERADSW2.OR.LERADN2.OR.LERADLW2.OR.LENCLD2.OR.LEGWWMS2).AND.LETRAJP)&
 & .OR. LETRAJPT .OR. LTRAJPST  
IF(LLPHYS)THEN
  IFLDSGTT5 = NG3DT95*NFLEVG+NG3PR95*(NFLEVG+1)+NG2DSF5
  ILEN=0
  DO JL=1,NGPTOT,NPROMA
    IS=MIN(NGPTOT-JL+1,NPROMA)
    ILEN=ILEN+(IS+NPCKFPH5-1)/NPCKFPH5
  ENDDO
  NLENGTT5B = ILEN    * IFLDSGTT5
  ALLOCATE(TRAJPHTE(NLENGTT5B,0:NSTOP))
  IF (LLP) WRITE(IU,9) 'TRAJPHTE ',SIZE(TRAJPHTE),SHAPE(TRAJPHTE)
ENDIF

IF (LTRAJPS) THEN
  IFLDSGTS5 = NG2DS5
  ILEN=0
  DO JL=1,NGPTOT,NPROMA
    IS=MIN(NGPTOT-JL+1,NPROMA)
    ILEN=ILEN+(IS+NPCKFTS5-1)/NPCKFTS5
  ENDDO
  NLENGTS5B = ILEN    * IFLDSGTS5
  ALLOCATE(TRAJPHSF(NLENGTS5B,0:NSTOP+1))
  IF (LLP) WRITE(IU,9) 'TRAJPHSF ',SIZE(TRAJPHSF),SHAPE(TRAJPHSF)
ENDIF

!*        3.2.2 CLOUD CHARACTERISTICS FOR SIMPLE CLOUD SCHEME

IF (LENCLD2 .AND. LETRAJP) THEN
  ALLOCATE(GPCLOUD5(NPROMA,3*NFLEVG,NGPBLKS))
  ALLOCATE(GPCLOUDIN(NPROMA,3*NFLEVG,NGPBLKS))
ENDIF

!*        3.2.3 LONGWAVE RADIATION FOR TL/AD

IF (LERADLW2) THEN
  ALLOCATE(RADLWM (NPROMA,0:NFLEVG,NGPBLKS))
  ALLOCATE(RADLWM5(NPROMA,0:NFLEVG,NGPBLKS))
ENDIF

!*        3.2.4 NON-OROGRAPHIC GRAVITY WAVE SCHEME FOR TL/AD

IF (LEGWWMS2) THEN
  ALLOCATE(TGWDWMS (NPROMA,2*NFLEVG,NGPBLKS))
  ALLOCATE(TGWDWMS5(NPROMA,2*NFLEVG,NGPBLKS))
ENDIF

!*        3.2.5 PERTURBATION OF SKIN TEMPERATURE FOR LINEARIZED PHYSICS

IF (LETRAJP) THEN
  ALLOCATE(GPTSKIN0(NPROMA,NGPBLKS))
  ALLOCATE(GPTSKIN9(NPROMA,NGPBLKS))
ENDIF

!*        3.2.6 RADIATION COEFFICIENTS

!         Solar radiation coefficients and correction for
!         thermal radiation

IF (LRCOEF) THEN

  IFLDSGSR = NG3SR*NFLEVG
  ILEN=0
  DO JL=1,NGPTOT,NPROMA
    IS=MIN(NGPTOT-JL+1,NPROMA)
    ILEN=ILEN+(IS+NPCKFSR-1)/NPCKFSR
  ENDDO
  NLENGSRB = ILEN    * IFLDSGSR

  ALLOCATE(SOLRAD(NLENGSRB))
  IF (LLP) WRITE(IU,9) 'SOLRAD ',SIZE(SOLRAD),SHAPE(SOLRAD)

!          Matrix of thermal radiation coefficients
!          (warning: it is supposed that npckfthr has to be 1 !!!)

  NLENGTRB = NGPBLKS

  INUMLEV=(NFLEVG+1)*(NFLEVG+1)

  IF (LGLOBRAD) THEN
    ALLOCATE(THERRAD(1,INUMLEV))
    THERRAD(:,:)=0.0_JPRB
    IF (LLP) WRITE(IU,9) 'THERRAD ',SIZE(THERRAD),SHAPE(THERRAD)

    ALLOCATE(TRWEIGHT(NFLEVG+1,NFLEVG+1))
    TRWEIGHT(:,:)=0.0_JPRB
    IF(LLP) WRITE(IU,9) 'TRWEIGHT ',SIZE(TRWEIGHT),SHAPE(TRWEIGHT)

    ALLOCATE(TRMATSUM(NFLEVG+1,NFLEVG+1))
    TRMATSUM(:,:)=0.0_JPRB
    IF(LLP) WRITE(IU,9) 'TRMATSUM ',SIZE(TRMATSUM),SHAPE(TRMATSUM)
  ELSE
    ALLOCATE(THERRAD(NLENGTRB,INUMLEV))
    THERRAD(:,:)=0.0_JPRB
    IF (LLP) WRITE(IU,9) 'THERRAD ',SIZE(THERRAD),SHAPE(THERRAD)
  ENDIF

ENDIF

!*        3.2.7   TIMESTEP T-DT FOR TRAJECTORY
!                 (NON-HYDROSTATIC AND ITERATION IN NHS)

IF(LNHDYN.AND.LTRAJNH) THEN
  IFLDSNH95 = NG3NH95*NFLEVG+1
  ILEN=0
  DO JL=1,NGPTOT,NPROMA
    IS=MIN(NGPTOT-JL+1,NPROMA)
    ILEN=ILEN+(IS+NPCKFT95-1)/NPCKFT95
  ENDDO
  NLENNH95B = ILEN    * IFLDSNH95
  ALLOCATE(TRAJNH(NLENNH95B,0:NSTOP))
  IF(LLP) WRITE(IU,9) 'TRAJNH ',SIZE(TRAJNH),SHAPE(TRAJNH)
ENDIF

!*        3.2.8   TIMESTEP T-DT FOR TRAJECTORY
!              (T/L OR ADJOINT OF SEMI-LAGRANGIAN)

IF (LSLAG) THEN
  IF (NTRSLTYPE <= 1) THEN
    IF (LR2D) THEN
      IFLDSGTSL = 3
    ELSE
      CALL ABOR1(' SUSC2B 3.2.7a: NTRSLTYPE <= 1 not coded in 3D model')
    ENDIF
  ELSEIF (NTRSLTYPE == 2) THEN
    IF (LR2D) THEN
      IFLDSGTSL = 14
    ELSE
      IFLDSGTSL = 14*NFLEVG+1
      IF (LADVF.AND.L2TLFF) IFLDSGTSL = IFLDSGTSL+2*NFLEVG
      IF (LSPRT) IFLDSGTSL = IFLDSGTSL+2*NFLEVG
      IF (YO3%LACTIVE) IFLDSGTSL = IFLDSGTSL+NFLEVG
    ENDIF
  ENDIF
ELSE             !! (Eulerian case)
  IFLDSGTSL=0
  IF (LSPRT) IFLDSGTSL = IFLDSGTSL+2*NFLEVG
ENDIF

IF ((NCONF == 131.OR.NCONF == 401.OR.NCONF == 501.OR.&
   & NCONF == 421.OR.NCONF == 521.OR.NCONF == 601.OR.NCONF == 801)&
   & .AND.(LSLAG.OR.LSPRT)) THEN  

  IF (LR2D) THEN
    ILEN=0
    DO JL=1,NGPTOT,NPROMA
      IS=MIN(NGPTOT-JL+1,NPROMA)
      ILEN=ILEN+(IS+NPCKSLTRAJ-1)/NPCKSLTRAJ
    ENDDO
    NLENGTSLB = ILEN    * IFLDSGTSL
  ELSE
    IF (NTRSLTYPE == 2) THEN
      NLENGTSLB=1
    ELSE
      CALL ABOR1(' SUSC2B 3.2.7b: NTRSLTYPE <= 1 not coded in 3D model')
    ENDIF
  ENDIF
  ALLOCATE(TRAJSL(NLENGTSLB,0:NSTOP))
  IF (LLP) WRITE(IU,9) 'TRAJSL ',SIZE(TRAJSL),SHAPE(TRAJSL)
ENDIF

!*        3.3   SURFACE FIELDS

CALL ALLO_SURF

!*        3.4   FLUXES WORK ARRAYS

IF (LCUMFU) THEN
  CALL ALLOCATE_GRIDPOINT_BUFFER ('GFUBUF',GFUBUF,NFDCFU,NPCKFCF)
ENDIF

IF (LXFU) THEN
  CALL ALLOCATE_GRIDPOINT_BUFFER ('XFUBUF',XFUBUF,NFDXFU,NPCKFXF)
ENDIF

!*        3.6   MODELS WITH PC scheme (GPPCBUF)

! * Compute NFGPPC and the pointers of PTRGPPC.

NFGPPC=0

CALL SUPTRGPPC(MGPPC,&
 & MGPPCF_U,MGPPCF_V,MGPPCF_T,MGPPCF_SPD,MGPPCF_SVD,MGPPCF_SP,&
 & MGPPCF_CP,MGPPCF_CPSI,&
 & MGPPCF_NHX,MGPPCF_UP,MGPPCF_VP,MGPPCF_TP,MGPPCF_GFLP,MGPPCF_BBC, &
 & INFGPPC)
IF (NPRINTLEV >= 1) THEN
  WRITE(UNIT=IU,FMT='('' MGPPC ='',I6)') MGPPC
  WRITE(UNIT=IU,FMT='('' MGPPCF_U ='',I6)') MGPPCF_U
  WRITE(UNIT=IU,FMT='('' MGPPCF_V ='',I6)') MGPPCF_V
  WRITE(UNIT=IU,FMT='('' MGPPCF_T ='',I6)') MGPPCF_T
  WRITE(UNIT=IU,FMT='('' MGPPCF_SPD ='',I6)') MGPPCF_SPD
  WRITE(UNIT=IU,FMT='('' MGPPCF_SVD ='',I6)') MGPPCF_SVD
  WRITE(UNIT=IU,FMT='('' MGPPCF_SP ='',I6)') MGPPCF_SP
  WRITE(UNIT=IU,FMT='('' MGPPCF_CP ='',I6)') MGPPCF_CP
  WRITE(UNIT=IU,FMT='('' MGPPCF_CPSI ='',I6)') MGPPCF_CPSI
  WRITE(UNIT=IU,FMT='('' MGPPCF_NHX ='',I6)') MGPPCF_NHX
  WRITE(UNIT=IU,FMT='('' MGPPCF_UP ='',I6)') MGPPCF_UP
  WRITE(UNIT=IU,FMT='('' MGPPCF_VP ='',I6)') MGPPCF_VP
  WRITE(UNIT=IU,FMT='('' MGPPCF_TP ='',I6)') MGPPCF_TP
  WRITE(UNIT=IU,FMT='('' MGPPCF_GFLP ='',I6)') MGPPCF_GFLP
  WRITE(UNIT=IU,FMT='('' MGPPCF_BBC ='',I6)') MGPPCF_BBC
ENDIF

IF( NCONF == 131 .OR. NCONF == 401 .OR. NCONF == 501 .OR. &
 & NCONF == 601 .OR. NCONF == 801 )THEN
  ! * This is required for all configurations calling CPGTL or CPGAD.
  CALL SUPTRGPPC(MGPPC5,&
   & MGPPCF_U5,MGPPCF_V5,MGPPCF_T5,MGPPCF_SPD5,MGPPCF_SVD5,MGPPCF_SP5,&
   ! this line to be updated
   & MGPPCF_CP,MGPPCF_CPSI,&
   & MGPPCF_NHX,MGPPCF_UP,MGPPCF_VP,MGPPCF_TP,MGPPCF_GFLP,MGPPCF_BBC, &
   & INFGPPC5)
  IF (NPRINTLEV >= 1) THEN
    WRITE(UNIT=IU,FMT='('' MGPPC5 ='',I6)') MGPPC5
    WRITE(UNIT=IU,FMT='('' MGPPCF_U5 ='',I6)') MGPPCF_U5
    WRITE(UNIT=IU,FMT='('' MGPPCF_V5 ='',I6)') MGPPCF_V5
    WRITE(UNIT=IU,FMT='('' MGPPCF_T5 ='',I6)') MGPPCF_T5
    WRITE(UNIT=IU,FMT='('' MGPPCF_SPD5 ='',I6)') MGPPCF_SPD5
    WRITE(UNIT=IU,FMT='('' MGPPCF_SVD5 ='',I6)') MGPPCF_SVD5
    WRITE(UNIT=IU,FMT='('' MGPPCF_SP5 ='',I6)') MGPPCF_SP5
  ENDIF
ENDIF

NFGPPC=INFGPPC
WRITE(UNIT=IU,FMT='('' NFGPPC ='',I6)') NFGPPC

! * Allocate GPPCBUF.

! ky: this test should be consistent with the one done for SI setup (SUDYN)
!     and what is done in CNT4.
IF ( (NFGPPC > 0.AND.((NSTOP > 0).OR.(NEINI /= 0).OR.(NSTOP==0.AND.LSDDH).OR. &
  & (NSTOP==0.AND.LWCOU))) ) THEN
  CALL ALLOCATE_GRIDPOINT_BUFFER ('GPPCBUF',GPPCBUF,NFGPPC,NPCKFGN)
ELSE
  NULLIFY(GPPCBUF%GPBUF)
ENDIF

!*        3.7 GLOBAL STORAGE FOR INTERMITTENT ACRANEB2 RADIATION SCHEME

IF (LRAY.AND.NRAY > 1 ) THEN

  ! storage for solar gaseous transmissions
  IF (NSORAYFR /= 1) THEN
    ALLOCATE(GDEOSI  (NPROMM,0:NFLEVG,2,NGPBLKS))
    ALLOCATE(GUEOSI  (NPROMM,0:NFLEVG,2,NGPBLKS))
    ALLOCATE(GMU0    (NPROMM,NSORAYFR  ,NGPBLKS))
    ALLOCATE(GMU0_MIN(NPROMM           ,NGPBLKS))
    ALLOCATE(GMU0_MAX(NPROMM           ,NGPBLKS))
    IF (LLP) THEN
      WRITE(IU,9) 'GDEOSI   ',SIZE(GDEOSI  ),SHAPE(GDEOSI  )
      WRITE(IU,9) 'GUEOSI   ',SIZE(GUEOSI  ),SHAPE(GUEOSI  )
      WRITE(IU,9) 'GMU0     ',SIZE(GMU0    ),SHAPE(GMU0    )
      WRITE(IU,9) 'GMU0_MIN ',SIZE(GMU0_MIN),SHAPE(GMU0_MIN)
      WRITE(IU,9) 'GMU0_MAX ',SIZE(GMU0_MAX),SHAPE(GMU0_MAX)
    ENDIF
  ENDIF

  ! storage for thermal gaseous transmissions
  IF (NTHRAYFR /= 1) THEN
    ALLOCATE(GRSURF  (NPROMM,         NGPBLKS))
    ALLOCATE(GDEOTI  (NPROMM,0:NFLEVG,NGPBLKS))
    ALLOCATE(GDEOTI2 (NPROMM,0:NFLEVG,NGPBLKS))
    ALLOCATE(GUEOTI  (NPROMM,0:NFLEVG,NGPBLKS))
    ALLOCATE(GUEOTI2 (NPROMM,0:NFLEVG,NGPBLKS))
    ALLOCATE(GEOXT   (NPROMM,  NFLEVG,NGPBLKS))
    IF (LLP) THEN
      WRITE(IU,9) 'GRSURF   ',SIZE(GRSURF ),SHAPE(GRSURF )
      WRITE(IU,9) 'GDEOTI   ',SIZE(GDEOTI ),SHAPE(GDEOTI )
      WRITE(IU,9) 'GDEOTI2  ',SIZE(GDEOTI2),SHAPE(GDEOTI2)
      WRITE(IU,9) 'GUEOTI   ',SIZE(GUEOTI ),SHAPE(GUEOTI )
      WRITE(IU,9) 'GUEOTI2  ',SIZE(GUEOTI2),SHAPE(GUEOTI2)
      WRITE(IU,9) 'GEOXT    ',SIZE(GEOXT  ),SHAPE(GEOXT  )
    ENDIF
    IF (LRPROX) THEN
      ALLOCATE(GEOLT (NPROMM,  NFLEVG,NGPBLKS))
      ALLOCATE(GRPROX(NPROMM,0:NFLEVG,NGPBLKS))
      IF (LLP) THEN
        WRITE(IU,9) 'GEOLT    ',SIZE(GEOLT ),SHAPE(GEOLT )
        WRITE(IU,9) 'GRPROX   ',SIZE(GRPROX),SHAPE(GRPROX)
      ENDIF
    ENDIF
  ENDIF

  ! storage for bracketing weights and offset
  IF ((NRAUTOEV == 0.AND.NTHRAYFR          /= 1).OR. &
   &  (NRAUTOEV >  0.AND.NTHRAYFR*NRAUTOEV /= 1)) THEN
    ALLOCATE(GMIXP (NPROMM,0:NFLEVG,NGPBLKS))
    ALLOCATE(GFLUXC(NPROMM,0:NFLEVG,NGPBLKS))
    IF (LLP) THEN
      WRITE(IU,9) 'GMIXP    ',SIZE(GMIXP ),SHAPE(GMIXP )
      WRITE(IU,9) 'GFLUXC   ',SIZE(GFLUXC),SHAPE(GFLUXC)
    ENDIF
  ENDIF

ENDIF

!    -------------------------------------------------------------------

!*        4. INITIALIZE MEMORY BUFFERS (CONT).
!            --------------------------------

!*        4.1   NMI

IF (.NOT.LNOINI) THEN
  IBITS = NVMODM * (NXMAX + 1)
  DO J = 1,IBITS
    NSTASPHB(J) = -999
    NSTAHOUB(J) = -999
  ENDDO
  ISUMMEN = 0
  DO JK = 1,NVMODM
    DO JM = 0,NXMAX
      IORDR = 3 * (NXMAX + 1 - JM)
      ILEN = IORDR * IORDR
      IREC = (JK - 1) * (NXMAX + 1) + JM + 1
      NSTASPHB(IREC) = ISUMMEN
      NSTAHOUB(IREC) = ISUMMEN
      ISUMMEN = ISUMMEN + ILEN
    ENDDO
  ENDDO
  NLENSPHB = ISUMMEN
  NLENHOUB = ISUMMEN
  ALLOCATE(SPHBUF(NLENSPHB))
  ALLOCATE(HOUBUF(NLENHOUB))
  IF (LLP) WRITE(IU,9) 'SPHBUF   ',SIZE(SPHBUF),SHAPE(SPHBUF)
  IF (LLP) WRITE(IU,9) 'HOUBUF   ',SIZE(HOUBUF),SHAPE(HOUBUF)
ENDIF
IF (.NOT.LNOINI .AND. LRPIMP) THEN
  IBITS = 5 + 2 * NVMODM
  ISUMMEN = 0
  NSTAPINB(1) = ISUMMEN
  ISUMMEN = ISUMMEN + NCPEC
  NSTAPINB(2) = ISUMMEN
  ISUMMEN = ISUMMEN + NCPEC
  DO JK = 3,IBITS
    NSTAPINB(JK) = ISUMMEN
    ISUMMEN = ISUMMEN + NCPEC2
  ENDDO
  ILENPINB = ISUMMEN
  ALLOCATE(RPINBUF(ILENPINB))
  IF (LLP) WRITE(IU,9) 'RPINBUF  ',SIZE(RPINBUF),SHAPE(RPINBUF)
ENDIF

!*        4.2   ALLOCATE GFL ARRAYS

IF(LFPART2) DEALLOCATE(GFL)
ALLOCATE(GFL(NPROMA,NFLEVG,YGFL%NDIM,NGPBLKS))
IF (LLP) WRITE(IU,9) 'GFL      ',SIZE(GFL),SHAPE(GFL)

IF(LFPART2) DEALLOCATE(GFLSLP)
ALLOCATE(GFLSLP(NPROMA,NFLEVG,YGFL%NDIMSLP,NGPBLKS))
IF (LLP) WRITE(IU,9) 'GFLSLP   ',SIZE(GFLSLP),SHAPE(GFLSLP)

IF(LFPART2) DEALLOCATE(GFL5)
ALLOCATE(GFL5(NPROMA,NFLEVG,YGFL%NDIM5,NGPBLKS))
IF (LLP) WRITE(IU,9) 'GFL5     ',SIZE(GFL5),SHAPE(GFL5)

WRITE(NULOUT,*)'YGFL%NDIMPT=',YGFL%NDIMPT
IF(LFPART2) DEALLOCATE(GFLPT)
ALLOCATE(GFLPT(NPROMA,NFLEVG,YGFL%NDIMPT,NGPBLKS))
IF (LLP) WRITE(IU,9) 'GFLPT    ',SIZE(GFLPT),SHAPE(GFLPT)

WRITE(NULOUT,*)'YGFL%NDIMPC=',YGFL%NDIMPC
IF(LFPART2) DEALLOCATE(GFLPC)
ALLOCATE(GFLPC(NPROMA,NFLEVG,YGFL%NDIMPC,NGPBLKS))
IF (LLP) WRITE(IU,9) 'GFLPC    ',SIZE(GFLPC),SHAPE(GFLPC)

!*        4.3   GRIDPOINT WORKFILE FOR ANALYSIS/FORECAST ERRORS

IF (LAVCGL .OR. NBGVECS > 0) THEN
  NANEBU   = 0
  NANEBV   = NFLEVG
  NANEBZ   = NFLEVG*2
  NANEBQ   = NFLEVG*3
  NANEBT   = NFLEVG*4
  NANEBDIV = NFLEVG*5
  NANEBVO  = NFLEVG*6
  NANEBSP  = NFLEVG*7
  NANEBR   = -1
  NANEBO3  = -1
  NANEBTO3 = -1
  NANEBRAD = -1
  NANEBTCW = -1
  NANEBRH2 = -1
  NANEBU10 = -1
  NANEBV10 = -1
  NANEBT2  = -1
  NANEBGHG(:)  = -1
  NANEBGRG(:)  = -1
  NANEBAERO(:)  = -1
  NANEVARS = 8

  IF (LLO3) THEN
    NANEBO3  = NFLEVG*7
    NANEBSP  = NFLEVG*8
    NANEVARS = 9
  ENDIF

  IF (NGHG > 0) THEN
    IANEINDEX=NANEBSP
    DO JL=1,NGHG
      IF (YGHG(JL)%LGP) THEN
        NANEBGHG(JL)  = IANEINDEX
        IANEINDEX=IANEINDEX+NFLEVG
        NANEVARS  = NANEVARS+1
      ENDIF
    ENDDO
    NANEBSP   = NANEBSP+NGHG*NFLEVG
  ENDIF

  IF (NGRG > 0) THEN
    IANEINDEX=NANEBSP
    DO JL=1,NGRG
      IF (YGRG(JL)%LGP) THEN
        NANEBGRG(JL)  = IANEINDEX
        IANEINDEX=IANEINDEX+NFLEVG
        NANEVARS  = NANEVARS+1
      ENDIF
    ENDDO
    NANEBSP   = NANEBSP+NGRG*NFLEVG
  ENDIF

  IF (LLAERO) THEN
    IANEINDEX=NANEBSP
    DO JL=1,JB_STRUCT%JB_DATA%NAEROCV
        NANEBAERO(JL)  = IANEINDEX
        IANEINDEX=IANEINDEX+NFLEVG
        NANEVARS  = NANEVARS+1
    ENDDO
    NANEBSP   = NANEBSP+JB_STRUCT%JB_DATA%NAEROCV*NFLEVG
  ENDIF


  IF(LBGOBS) THEN
    IF(LECMWF) THEN
      NANERADS = 48
      ALLOCATE(NANERADL(NANERADS))
      IF(LLP) WRITE(IU,9) 'NANERADL ',SIZE(NANERADL),SHAPE(NANERADL)
      NANERADL=&
       & (/    1, 2, 3, 4, 5, 6, 7, 8, 9,10,&
       & 11,12,13,14,15,&
       & 22,23,24,25,26,27,28,29,30,&
       & 31,32,33,34,35,36,37,38,39,40,&
       & 41,42,43,44,45,46,47,&
       & 48,49,50,51,52,53,54            /)
    ELSE
      NANERADS = 27
      ALLOCATE(NANERADL(NANERADS))
      IF(LLP) WRITE(IU,9) 'NANERADL ',SIZE(NANERADL),SHAPE(NANERADL)
      NANERADL=&
       & (/  4, 5, 6, 7,11,12,14,15,&
       &    32,33,34,35,36,37,38,39,40,&
       &    45,46,47,&
       &    48,49,50,51,52,53,54 /)
    ENDIF
    NANEBR   = NFLEVG*7
    NANEBRAD = NFLEVG*8
    NANEBTCW = NFLEVG*8+NANERADS
    NANEBRH2 = NFLEVG*8+NANERADS+1
    NANEBU10 = NFLEVG*8+NANERADS+2
    NANEBV10 = NFLEVG*8+NANERADS+3
    NANEBT2  = NFLEVG*8+NANERADS+4
    NANEBSP  = NFLEVG*8+NANERADS+5
    NANEVARS = 15

    IF (LLO3) THEN
      NANEBR   = NFLEVG*7
      NANEBO3  = NFLEVG*8
      NANEBTO3 = NFLEVG*9
      NANEBRAD = NFLEVG*9+1
      NANEBTCW = NFLEVG*9+NANERADS+1
      NANEBRH2 = NFLEVG*9+NANERADS+2
      NANEBU10 = NFLEVG*9+NANERADS+3
      NANEBV10 = NFLEVG*9+NANERADS+4
      NANEBT2  = NFLEVG*9+NANERADS+5
      NANEBSP  = NFLEVG*9+NANERADS+6
      NANEVARS = 17
    ENDIF

    IF (NGHG > 0) THEN
      IANEINDEX=NANEBSP
      DO JL=1,NGHG
        IF (YGHG(JL)%LGP) THEN
          NANEBGHG(JL)  = IANEINDEX
          IANEINDEX=IANEINDEX+NFLEVG
          NANEVARS  = NANEVARS+1
        ENDIF
      ENDDO
      NANEBSP   = NANEBSP+NGHG*NFLEVG
    ENDIF

    IF (NGRG > 0) THEN
      IANEINDEX=NANEBSP
      DO JL=1,NGRG
        IF (YGRG(JL)%LGP) THEN
          NANEBGRG(JL)  = IANEINDEX
          IANEINDEX=IANEINDEX+NFLEVG
          NANEVARS  = NANEVARS+1
        ENDIF
      ENDDO
      NANEBSP   = NANEBSP+NGRG*NFLEVG
    ENDIF

   IF(LLAERO) THEN
     IANEINDEX=NANEBSP
     DO JL=1,JB_STRUCT%JB_DATA%NAEROCV
         NANEBAERO(JL)  = IANEINDEX
         IANEINDEX=IANEINDEX+NFLEVG
         NANEVARS  = NANEVARS+1
     ENDDO
     NANEBSP   = NANEBSP+JB_STRUCT%JB_DATA%NAEROCV*NFLEVG
   ENDIF

  ENDIF

  NOANEF   = NANEBSP + 1       !  SP has to be last in list

  CALL ALLOCATE_GRIDPOINT_BUFFER ('ANEBUF',ANEBUF,NOANEF,1)
ELSE
  NULLIFY(ANEBUF%GPBUF)
ENDIF

!*        4.3   GRIDPOINT BUFFERS WITHOUT DEDICATED SETUP

NULLIFY(GAUXBUF%GPBUF)

9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

!    -------------------------------------------------------------------

CALL GSTATS(1939,1)
IF (LHOOK) CALL DR_HOOK('SUSC2B',1,ZHOOK_HANDLE)
END SUBROUTINE SUSC2B
